// DatafoxCommIV.h : Hauptheaderdatei fr die DatafoxCommIV-DLL
//

#pragma once

#ifdef _WIN32
#ifndef __AFXWIN_H__
	#error "'stdafx.h' vor dieser Datei fr PCH einschlieen"
#endif
#endif

#include "resource.h"		// Hauptsymbole

// Kommunikationsarray fuer Verbindungen
#include "AEChannelArray.h"
// Versionspruefung
#include "../Shared/Common/DFComUtils.h"

#ifdef _WIN32

// GetDllVersion / GetAppVersion
#include <shlwapi.h>

#include <afxmt.h> // Fuer CCriticalSection


// Fuer XML-RPC
#include "../Shared/xmlrpc/XmlRpc.h"

// Kontainerklassen fuer Prioritaetswarteschlange importieren.
#include <deque>
#include <queue>

typedef std::deque<CNotifyRecord> notify_deque;
typedef std::priority_queue<CNotifyRecord, notify_deque> notify_priority_deque;

class CPriorityDeque : public notify_priority_deque
{
public:
	int CheckFor(CNotifyRecord &nr)
	{
		int i, res = 0;
		for (i=0; i < (int)c.size(); i++) {

			//if ((nr.serialNum == c.at(i).serialNum) && (nr.deviceType == c.at(i).deviceType)) {
			if (nr.channelID == c.at(i).channelID) {

				res = 1;
				break;
			}
		}

		return res;
	}
	int Erase(int channelID)
	{
		int i, res = 0;
		for (i=0; i < (int)c.size(); i++) {

			if (channelID == c.at(i).channelID) {

				c.erase(c.begin() + i);
				res = 1;
				break;
			}
		}

		return res;
	}

};
#endif

// CDatafoxCommIVApp
// Siehe DatafoxCommIV.cpp fr die Implementierung dieser Klasse
//
#ifdef _WIN32
class CDatafoxCommIVApp : public CWinApp
#else
class CDatafoxCommIVApp
#endif
{
public:
	CDatafoxCommIVApp();

	static CDatafoxCommIVApp *m_this;

#ifdef _WIN32

	// berschreibungen
public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();

	DECLARE_MESSAGE_MAP()

public:
	int StartNotify();
	int EndNotify();

	// CallBack - Funktion zur Meldung von Datensaetzen.
	// Diese Funktion wird von den Einzelnen CChannel4 Kanalklassen genutzt.
	static int CBNotifyRecord(CNotifyRecord &nr)
	{
		CCriticalSection cs;
		cs.Lock();

		// Ist das Melden von Datensaetzen erlaubt?
		if (!m_this->IsNotifyRecord()) {

			return -IV_ERROR_AC_NO_NOTIFY_RECORD;
		}

		if (nr.erase) {

			// Eintrag vom Geraet Entfernen
			if (m_this->m_queue.Erase(nr.channelID)) {

				TRACE("Erase: %i, %i\n", nr.channelID, m_this->m_queue.size());
				return IV_ERROR_SUCCESS;
			}

		} else {

			// Bereits Eintrag vom Geraet vorhanden?
			if (m_this->m_queue.CheckFor(nr)) {

				TRACE("Skip: %i, %i\n", nr.channelID, m_this->m_queue.size());
				return IV_ERROR_SUCCESS;
			}

			m_this->m_queue.push(nr);
			TRACE("Put: %i, %i\n", nr.channelID, m_this->m_queue.size());
		}

		cs.Unlock();

		return IV_ERROR_SUCCESS;
	}

	// Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:
	static unsigned long WINAPI ListenThread(LPVOID lpParam);
	static unsigned long WINAPI NotifyThread(LPVOID lpParam);
	volatile int m_endSignal[2];
	volatile int m_initSignal[1];
	enum t_state {DOWN, RESET, LISTENING, ACCEPTING};
	t_state m_listenThreadState;
	HANDLE m_workerThread[2];
	// Socket fuer Listen
	SOCKET m_listenSocket;
	struct listenThreadData {
		char ip[64];
		int port;
		int deviceID;
		int timeout;
		int aliveTO;
		int maxConnections;
		int infoFlags;
	} m_listenThreadData;
	struct notifyThreadData {
		char ip[64];
		int port;
		int deviceID;
		int timeout;
		int aliveTO;
		int maxConnections;
		int infoFlags;
	} m_notifyThreadData;
#endif

	// Helper
protected:
	/// Anfrage an Kanalarray stellen.
	bool AEChannelRequest(int &channelID);
#ifdef WIN32
	//	bool Channel4Request(short &channelID);
#endif

	// Helper
public:
	/// Ermitteln der Kanalklasse aus Array.
	CAEChannel *GetAEChannel(int &channelID);
#ifdef WIN32
	/// Flag ob eine Kanalbindung durchgefuehrt wurde.
	volatile int m_bindSignal;
	/// Ermitteln ob die Kanalklasse an eine bestimmte ID gebunden werden soll.
	int ShouldBoundTo(CAEChannel *channel);
	/// Kanalobjekt von IDFrom zu IDTo verschieben.
	bool ChannelSwitch(int channelIDFrom, int channelIDTo);
	/// Kanalklasse mit Kanalnummer verbinden.
	bool SetAEChannel(int channelID, CAEChannel *channel);
	CPriorityDeque m_queue;

	int RecordAvailable(int *channelID, int *deviceID, char *infoString, int *infoStringLength)
	{
		int res = IV_ERROR_SUCCESS;

		// Datensatzmeldung aktiviert?
		if (!IsNotifyRecord()) {

			return -IV_ERROR_AC_NO_NOTIFY_RECORD;
		}

		if (m_queue.size()) {
			CCriticalSection cs;
			cs.Lock();

			CNotifyRecord &nr = m_queue.top();

			*channelID = nr.channelID;
			*deviceID = nr.deviceID;
			strncpy(infoString, (char*)nr.infoString, *infoStringLength);
			*infoStringLength = strlen(infoString);

			m_queue.pop();
			cs.Unlock();

		} else {

			res = -IV_ERROR_NO_DATA_AVAILABLE;
		}

		return res;
	}

protected:
	int m_notifyRecord; // Datensatzmeldungen verarbeiten (-1 = nicht gesetzt (ja), 0 = nein, 1 = ja)
public:
	int IsNotifyRecord() { return m_notifyRecord != 0; }

public:
	int StartActiveConnection(char *ip, int port, int deviceID, int timeout, int aliveTO, int maxConnections, int infoFlags, int reserved, int *piError);
	int StopActiveConnection();
	int SetRecordAvailable(int enable);
	int SetNotifySoftware(int type, char *ip, int port, int retry, int delay, int state);
	inline bool IsActiveConnection() { return (m_workerThread[0] != NULL); }

#endif

	// Membervars
public:
	/// Arrayobjekt fuer Kanalklassen.
	CAEChannelArray m_channelArray;

public:
	// Versionspruefungsobjekt
	CDFComUtils m_util;
};

extern CDatafoxCommIVApp theApp;
