/**
 * @file  AEChannel.h
 * @brief Schnittstelle fuer die Klasse CAEChannel.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_)
#define AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Basisklassen
#include "../Common/AEBuffer.h"
#include "../Log/DFLog.h"

// Klassen fuer Multimasterprotokolle.
#include "Channel1_2.h" // Version 1/2
#include "Channel3.h" // Version 3
#include "Channel4.h" // Version 4

struct StructMemFileHeader {
	int structSize;          // Strukturgroesse
	int type;                // Speichertyp
	long baseAddress;        // Basisadresse
	long memAddress;         // Anfangsadresse
	long memSize;
	char tmp[76];
	//...
	int crcHead;    // 32bit CRC fuer Head (Immer letzter Wert in der Struktur; 
                    // Berechnungsmenge: (sizeof(SDFFileDeviceHeader) - sizeof(unsigned short)), also ohne sich selbst!)
};

struct StructBindInfo {
	int state;
	int typeDevice;
	unsigned long serial;
	char *ip[20];
};

// Struktur fr die Datei der Fingertemplates
//############################################
// Innerer Aufbau der Datei ist wie folgt:
// StructFTFileHeader, (count * [(type == 0) ? structFingertemplateDIN : structFingertemplateIdencom, CRC16 ])
// - Zuerst liegt die Headerstruktur StructFTFileHeader in der Datei.
// - Der erste 16bit-Wert der Datei muss 0xDFDF sein.
// - Der crc der Headerstruktur berechnet sich ber die Struktur ohne die zwei Byte des CRC selbst.
// - Die Daten beginnen ab der Position Seek(StructFTFileHeader.offs, Filebegin).
// - Der Datenteil besteht aus StructFTFileHeader.count Segmenten, die folgenden Aufbau haben.
//   + Zuerst kommt die Sturktur des Fingertemplates je nach StructFTFileHeader.type
//   + Darauf folgt ein 16bit CRC der sich ber diese Struktur berechnet.
struct StructFTFileHeader {
	unsigned short Id;    // Kennung der Datei: muss immer 0xDFDF sein.
	unsigned short offs;  // Offset auf den Datenbeginn, normalerweise Strukturgre, falls Erweiterung ntig.
	unsigned short count; // Anzahl enthaltener Fingertemplates (zur Sicherheit mit aufgenommen)
	unsigned char type;   // Templatetyp. 0=>DIN V66400 (161Byte), 1=>Idencom-Compact (216Byte)
	unsigned short crc;   // 16bit CRC ber die Headerstruktur (ohne diesen letzten 16bit Wert)
};

class CAEChannel : public CAEBuffer, public CDFLog
{
public:
	// Konstruktor fuer Kanalinitialisierung
	CAEChannel(short channelID);
	virtual ~CAEChannel();

#ifdef WIN32
	// Setzt Klassenzustand
	inline void SetActiveState(ActiveState as) { m_channel4.SetActiveState(as); }
	// Prueft ob Klasse bereits "in Verwendung"
	inline ActiveState GetActiveState() { return m_channel4.GetActiveState(); }
	// Gibt direkt zurck ob der Kanal Aktiv ist
	inline bool IsActive() { return (GetActiveState() == is_active); }

	StructBindInfo m_bindInfo;
	// Setzt die Information ob die Kanalklasse an ein bestimmtes Geraet gebunden ist.
	inline void SetBind(StructBindInfo bi) { m_bindInfo = bi; }
	// Prueft ob Klasse alle Informationen fuer eine Kanalbindung hat.
	inline int IsBind() { return m_bindInfo.state; }

	// Setzt Klassenzustand auf "Wartung"
	inline void SetServiceState(int state) { m_channel4.SetServiceState(state); }
	// Prueft ob Klasse bereits "im Wartungsmodus"
	inline int GetServiceState() { return m_channel4.GetServiceState(); }
#endif

	// Setzt die Kanalnummer
	inline void SetChannelID(int channelID) {

		// Kanalnummer setzen
		m_channel4.SetID(channelID);
		// Kommunikationsbasis von m_channel4 an andere uebergeben
		m_channel1_2.SetChannelBase(m_channel4.GetChannelBase());
		m_channel3.SetChannelBase(m_channel4.GetChannelBase());
	}

#ifdef WIN32
	// Verbindungsannahme von uebergebenem Listen-Socket
	int Connect(SOCKET *listenSocket, short channelID, short deviceID = IV_DEFAULT_DEVICEID)
	{ return m_channel4.Connect(listenSocket, channelID, deviceID); }
	// Endesignal im Workerthread setzen und auf dessen Beendigung warten
	void DirectClose()
	{ m_channel4.DirectClose(); }

	// Setzen der CallBack-Funktion fuer Datensatzbenachrichtigung
	void SetCBNotifyRecord(int (*CBNotifyRecord)(CNotifyRecord&))
	{
		m_channel4.m_CBNotifyRecord = CBNotifyRecord;
	}

// Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:

	//#################################
	// CallRoutinen fuer die Abarbeitung anstehender Kommunikationsbefehle
//	virtual int CallFunction(short module, short function, unsigned char *data, int &count);
	// NotifyRoutine fuer die Abarbeitung von Benachrichtigungen unter den Klassen
//	int CallFunctionNotify(short module, short function, unsigned char *data, int &count)
//	{ return m_channel4.CallFunctionNotify(module, function, data, count); }

/*
protected:
	char icmp_data[IV_PACKET_LENGTH_MAX];
	char recvbuf[IV_PACKET_LENGTH_MAX];

public:
	int PingICMP(char *remote);
	int BroadcastXPort(char *address_net);
	int CmdXPort(char *remote, char *data, unsigned short &count, int needAnswer = TRUE);
	void PreparePacket(char * icmp_data, int datasize);
	void CheckPacket(char *buf, int bytes, struct sockaddr_in *from);
	unsigned short Checksum(unsigned short *buffer, int size);
*/
#endif

public:
	/////////////////////////
	// Schnittstelle

	int IsOpen();
	int Open(int deviceID, int iType, char *szComString, int iComValue, int iComTimeOut);
	int OpenAEIII(int iType, char *szComString, int iComValue, int iComTimeOut);
	void Close(void);

	/// Ermitteln der uebergebenen Parameter an Open.
	inline void GetOpenParams(int &type, char *string, int &value, int &to) 
	{ m_channel4.GetOpenParams((CChannelBase::enumType&)type, string, value, to); };

	// Busnummer des Kommunikationspartners
	short SetDeviceIDRemote(short deviceIDRemote); 
	short GetDeviceIDRemote();

	// Busnummer der DLL - Klassen
	short SetDeviceID(short deviceID); 
	short GetDeviceID();

	// Schnittstelle nach Fehler bereinigen
	void ResetWithError(int err);

	// Speicher bereinigen
	void Reset();

public:
	/////////////////////////
	// Kommunikationshelper

	int Receive(void *buffer, int receiveCount, int *receivedCount);
	int Transmit(void *buffer, int transmitCount, int *transmittedCount);

	void SetCBFunction(int (*pFunction)(void));
	int GetProtocolVersion2(short deviceIDRemote, unsigned char detailed, int *piError);
	// Gibt die ermittelte Protokollversion zurueck
	unsigned char GetProtocolVersion(short deviceIDRemote);
	unsigned char SetProtocolVersion(short deviceIDRemote, unsigned char version);
	unsigned char m_protocolVersion[32];
	int m_preferAEIII; // Bevorzugt auf AEIII+ pruefen


	/////////////////////////
	// Zugriff auf HANDLE/SOCKET

	/// Ermitteln des Schnittstellenhandle.
	HANDLE GetComPort();
	/// Setzen der Schnittstellenparameter.
	int SetComPort(HANDLE hCom, int iType, int iComValue, int iTimeOut);

	int ComReleaseHandle();
	int ComUseHandle(HANDLE h);


	/////////////////////////
	// Helper

	// Aufbereiten und senden eines Protokollpakets
	int TransmitPacket(short deviceIDRemote, unsigned char type, short module, short function, unsigned char *buf, int count, int transmitTrials = IV_TRANSMIT_TRIALS);

	// Empfang und aufbereiten eines Protokollpakets
	int ReceivePacket(short deviceID, unsigned char &type, short &module, short &function, unsigned char *buf, int &count, int receiveTrials = IV_RECEIVE_TRIALS);

	/////////////////////////
	// Befehle

	// Allgeimein

	// Anzahl der letzten Ermittlungsversuche halten.
	int m_iDevicePollRetry;
	// Geraeteerreichbarkeit pruefen
	int CheckDevice(short deviceIDRemote, int *piError, int iPollRetry);

	// PZE Master IV warmstart.
	int Restart(short deviceIDRemote, int *piError);
	// Reset des Gertes
	int Reset(short deviceIDRemote, int mode, int *piError);

	// Senden einer Bestaetigungsmeldung an das AEIII+
	int SendMessage(short deviceIDRemote, unsigned char ucDelayTime,
		unsigned char ucAction,	unsigned char ucAudio, char *szMessage, 
		int iLength, int *piError);

	// Senden eines Infotextes fuer Geraetestatus.
	int SendInfotext(short deviceIDRemote, char *szMessage, int iLength, int *piError);

	// RTC - Wert schreiben / lesen
	int SetDateTime(short deviceIDRemote, unsigned char *pucDateTime, int *piError);
	int GetDateTime(short deviceIDRemote, unsigned char *pucDateTime, int *piError);

	// Globale-/Systemvariable schreiben / lesen
	int SetGlobVar(short deviceIDRemote, unsigned char *pucVar, int iType, char *value, int *piError);
	int GetGlobVar(short deviceIDRemote, unsigned char *pucVar, int iType, char *value, int iValueLength, int *piError);

	// Relais setzen/ruecksetzen und zustand abfragen.
	int CloseRelay(short deviceIDRemote, int iNum, int iTime, int *piError);
	int OpenRelay(short deviceIDRemote, int iNum, int *piError);
	int GetRelayState(short deviceIDRemote, int iNum, int *piState, int *piTime, int *piError);
	
	// Seriennummer auslesen
	int GetSerialNum(short deviceIDRemote, int *piSerialNum, int *piError);
	// Firmwareversion auslesen
	int GetVersionFirmware(short deviceIDRemote, char *pVersion, int *piError);
	// Firmwareversion aus Dateiheader lesen
	int GetVersionFirmwareFromFile(char *pFileName, char *pVersion, int *piError);

	/////////////////////////
	// System Device Log
	// Datensatz lesen
	int SystemReadRecord(int deviceIDRemote, char *buf, int *length, int *error);
	// Datensatz quittieren.
	int SystemQuitRecord(int deviceIDRemote, int *error);
	// Datensatzzeiger ruecksetzen
	int SystemRestoreRecords(short deviceIDRemote, int *piError);

	// Aktuellen Status des Flash abrufen
	int GetFlashStatus(short deviceIDRemote, int *piStatus, int *piError);
public:
	/////////////////////////
	// Datensatz- / Listenbeschreibungen

	int GetTableDescription(short deviceIDRemote,	unsigned char ucList, int *piError);
protected:
	int GetTables3(short deviceIDRemote, char *typeName);
	int GetTables4(short deviceIDRemote, char *typeName);
	int GetListName(short deviceIDRemote, unsigned char ucList, char *name, int *piError);


public:
	/////////////////////////
	// Datensaetze

	// Ermitteln des ersten Datensatzes
	int CollectData(short deviceIDRemote, int *piError);
	// Ermitteln des naechsten Datensatzes
	int GetDatensatz(unsigned char *pucBuf, int *piLength, int *piError);
	// Pruefen ob min. ein Datensatz im AEIII+ vorhanden.
	int CheckData(short deviceIDRemote, int *piError);
	// Loescht alle Datensaetzte im AEIII+
	int ClearData(short deviceIDRemote, int *piError);
	// Datensatzzeiger ruecksetzen
	int RestoreRecords(short deviceIDRemote, int *piError);
	// Datensatz lesen
	int DFReadRecord(int deviceIDRemote, char *buf, int *length, int *error);
	// Datensatz quittieren.
	int DFQuitRecord(int deviceIDRemote, int *error);
protected:
	// Datensatzfunktionen, Variable und Puffer fuer letzten gelesenen.
	int m_iRecordLength, m_iRecordError;
	unsigned char m_pucRecordBuf[MAX_MB_PAKET];


public:
	/////////////////////////
	// Listen

	// Laden der Importierten Listendaten
	int LoadListData(short deviceIDRemote, int *piError);
protected:
	int LoadListData4(short deviceIDRemote);
	//int DownloadLists(short deviceIDRemote, int *piError);

public:
	/////////////////////////
	// Zutrittslisten (Version 1 / Version 2)

	// Laden der Importierten Listendaten fuer Zutrittskontrolle
	int LoadEntranceListData(short deviceIDRemote, int iListNum, int *piError);
	int LoadEntrance2ListData(short deviceIDRemote, int iListNum, int *piError);
protected:
	int LoadEntrance2ListData4(short deviceIDRemote, unsigned char listNum);
	int LoadEntrance2List4(short deviceIDRemote, unsigned char listNum);


public:
	/////////////////////////
	// Timeboylisten

	// Laden der Importierten Listendaten fuer Timeboy
	int LoadTimeboyListData(short deviceIDRemote, unsigned char id, unsigned char num, int reserve, int *piError);
protected:
	int LoadTimeboyListData4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve);
	int LoadTimeboyList4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve);


public:
	/////////////////////////
	// Setup

	int UpLoadSetupData(short deviceIDRemote, char *szFileName, int *piError);
	int DownLoadSetupData(short deviceIDRemote, char *szFileName, int *piError);
	// Speicher initialisieren
	void ClearMem();

	// Veraenderungen an der Setupdatei vornehmen
	int ModifyStudioFile(char *sourceFile, char *destFile, char *changeData, int flags, int *piError);

protected:
	int UploadSetup4(short deviceIDRemote);
	int DownLoadSetup4(short deviceIDRemote);

public:
	/////////////////////////
	// Firmware
	char m_info[2048]; // Puffer fuer die Hardwareinfo in Textform.
	int UploadFileDevice(short deviceIDRemote, char *szFileName, int *piError);
protected:
	int UploadDevice3(short deviceIDRemote);
	int UploadDevice4(short deviceIDRemote);
	int CheckFileDisplay(CDFFileDisplay *pFileDisplay, int *piError);
	
public:
	/////////////////////////
	// Module
	int UploadModule(short deviceIDRemote, int type, char *szFileName, int *piError);
	int m_loadAsDefault; // Nachfolgende Uebertragung der Textdaten als Default.lng
	int UploadTextData(short deviceIDRemote, char *szFileName);

public:
	/////////////////////////
	// Log

	/// Setzen des Logdateinamen.
	void SetLogFileName(const char *szLogFileName);
	
	/// Log anschalten.
	inline void SetLogOn()  
	{ m_channel4.SetTrace(TRUE); }
	
	/// Log ausschalten.
	inline void SetLogOff() 
	{ m_channel4.SetTrace(FALSE); }

	/////////////////////////
	// Passwort
	void SetPassword(char *password)
	{
		m_channel4.SetPasswordCommunication(password);
	}
public:
	int TransferMemToFile(short deviceIDRemote, int type, long address, int size, char *fileName);
	int TransferFileToMem(short deviceIDRemote, char *fileName);

public:	
	/////////////////////////
	// Kanalklassen fuer Protokollverwendung.
	CChannel1_2 m_channel1_2;
	CChannel3 m_channel3;
	CChannel4 m_channel4;

public:
	/////////////////////////
	// Schnittstelle zu Channel1/Channel2
	int GetModus(short deviceIDRemote, int *error);

	/////////////////////////
	// Schnittstelle zu Channel3/Channel4
	void SetCBFunctionWithParams(int (*pFunction)(unsigned long, unsigned long));
	unsigned long GetCRC32(unsigned char *data, unsigned long size, unsigned long crc = 0xFFFFFFFF, unsigned char closure = TRUE);

	// MODULE_HARDWARE

	// Einfache Anfrage ob das Geraet antwortet.
	int Ping(short deviceIDRemote);

	// Typ lesen
	int SetTypeDevice(unsigned char deviceID, unsigned char type);
	int GetTypeDevice(short deviceIDRemote, unsigned char &type);
	int SetTypeBoard(unsigned char deviceID, unsigned char type);
	int GetTypeBoard(short deviceIDRemote, unsigned char &type);
	int SetTypeTCPIP(unsigned char deviceID, unsigned char value);
	int GetTypeTCPIP(short deviceIDRemote, unsigned char &value);
	int SetTypeModemMC(unsigned char deviceID, unsigned char value);
	int GetTypeModemMC(short deviceIDRemote, unsigned char &value);
	int SetTypeDisplay(unsigned char deviceID, unsigned char type);
	int GetTypeDisplay(short deviceIDRemote, unsigned char &type);
	int SetTypeKeypad(unsigned char deviceID, unsigned char type);
	int GetTypeKeypad(short deviceIDRemote, unsigned char &type);
	int SetTypeTransponder(unsigned char deviceID, unsigned char type);
	int GetTypeTransponder(short deviceIDRemote, unsigned char &type);
	int SetTypeMMC(unsigned char deviceID, unsigned char type);
	int GetTypeMMC(short deviceIDRemote, unsigned char &type);
	int SetTypeRadio(unsigned char deviceID, unsigned char type);
	int GetTypeRadio(short deviceIDRemote, unsigned char &type);
	int SetTypeGPS(unsigned char deviceID, unsigned char value);
	int GetTypeGPS(short deviceIDRemote, unsigned char &type);
	int SetTypeDCF77(unsigned char deviceID, unsigned char type);
	int GetTypeDCF77(short deviceIDRemote, unsigned char &type);
	int SetTypeFingerprint(unsigned char deviceID, unsigned char type);
	int GetTypeFingerprint(short deviceIDRemote, unsigned char &type);
	int SetTypeExternalReader(unsigned char deviceIDRemote, unsigned char value);
	int GetTypeExternalReader(short deviceIDRemote, unsigned char &value);

	// Komponente lesen
	int SetComponentRS485_0(unsigned char deviceID, unsigned char value);
	int GetComponentRS485_0(short deviceIDRemote, unsigned char &value);
	int SetComponentRS485_1(unsigned char deviceID, unsigned char value);
	int GetComponentRS485_1(short deviceIDRemote, unsigned char &value);
	int SetComponentUSB(unsigned char deviceID, unsigned char value);
	int GetComponentUSB(short deviceIDRemote, unsigned char &value);
	int SetComponentInAnalog(unsigned char deviceID, unsigned long value);
	int GetComponentInAnalog(short deviceIDRemote, unsigned long &value);
	int SetComponentInDigital(unsigned char deviceID, unsigned long value);
	int GetComponentInDigital(short deviceIDRemote, unsigned long &value);
	int SetComponentOutDigital(unsigned char deviceID, unsigned long value);
	int GetComponentOutDigital(short deviceIDRemote, unsigned long &value);

	// Option schreiben/lesen
	int SetOptionIOModule(unsigned char deviceID, unsigned char value);
	int GetOptionIOModule(short deviceIDRemote, unsigned char &value);
	int SetOptionTBAdaption(unsigned char deviceID, unsigned char value);
	int GetOptionTBAdaption(short deviceIDRemote, unsigned char &value);
	int SetOptionComServer(unsigned char deviceID, unsigned char value);
	int GetOptionComServer(short deviceIDRemote, unsigned char &value);
	int SetOptionTPWrite(unsigned char deviceID, unsigned char value);
	int GetOptionTPWrite(short deviceIDRemote, unsigned char &value);
	int SetOptionHTTP(unsigned char deviceID, unsigned char value);
	int GetOptionHTTP(short deviceIDRemote, unsigned char &value);
	int SetOptionLanguage(unsigned char deviceID, unsigned char value);
	int GetOptionLanguage(short deviceIDRemote, unsigned char &value);
	int SetOptionBiokeyUpdate(unsigned char deviceID, unsigned char value);
	int GetOptionBiokeyUpdate(short deviceIDRemote, unsigned char &value);
	int SetSetupOptions(short deviceIDRemote, unsigned short mask, unsigned short options);
	int GetSetupOptions(short deviceIDRemote, unsigned short &mask, unsigned short &options);
#define OFF_EXTERNAL_READER 0x00000001 // External Reader options 0=GIS, 1=PHG
#define OFF_IOMODULE        0x00000002 // IOModul
#define OFF_TBADAPTION      0x00000004 // TBAdaption
#define OFF_COMSERVER       0x00000008 // ComServer
#define OFF_TPWRITE         0x00000010 // TransponderWrite
#define OFF_HTTP            0x00000020 // Http-Protokoll
#define OFF_BIOKEY_UPDATE   0x00000040 // UpdateBiokey
#define OFF_SO_IOCONTROL    0x00000080 // SetupOption: Signalverarbeitung

	int SetOptionFirmware(short deviceIDRemote, int mask, int options);
	int GetOptionFirmware(short deviceIDRemote, int &mask, int &options);



	// HWConfig lesen
	int SetHWConfig(unsigned char deviceID, unsigned char value);
	int GetHWConfig(short deviceIDRemote, unsigned char &value);
	
	// Gesamte Hardwareinfo lesen
	int GetHardwareInfo(short deviceIDRemote, TDFHardwareInfo &hwInfo);
	// Gesamte Hardwareinfo in String umsetzen.
	int GetInfos(TDFHardwareInfo hwInfo, char *info);


	// MODULE_SYSTEM

	// Systemreset; Speicherbereiche loeschen
//	int SystemReset(short deviceIDRemote, unsigned short modus);

	// Firmwareversion lesen
	int GetVersion(short deviceIDRemote, char *version);
	int GetVersionBootloader(short deviceIDRemote, char *version);

	// Seriennummer 
	int SetSerial(unsigned char deviceID, unsigned long serial);
	int GetSerial(short deviceIDRemote, unsigned long &serial);

	// Datum/Uhrzeit setzen/lesen
//	int SetRTC(short deviceIDRemote, char *time);
//	int GetRTC(short deviceIDRemote, char *time);

	// Systemvariable setzen/lesen
//	int SetSystemVariable(short deviceIDRemote, char *name, char *value);
//	int GetSystemVariable(short deviceIDRemote, char *name, char *value);

	// Relais setzen/lesen
//	int SetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time);
//	int GetRelay(short deviceIDRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time);

	// Datensegmentuebertragung beginnen/enden
	int BeginSegment(short deviceIDRemote, unsigned short art, unsigned long &address, unsigned long size, unsigned long crc);
	int EndSegment(short deviceIDRemote, unsigned short art, unsigned long crc);

	// Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
	int SetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetFlash(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetFlashPage(short deviceIDRemote, unsigned short pageNum, unsigned char *buf, int &count);

	// EEPROM setzen/lesen
	int SetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetEEPROM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);

	// RAM setzen/lesen
	int SetRAM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetRAM(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);

	// PROGRAM setzen/(lesen unterbunden)
//	int SetProgram(short deviceIDRemote, unsigned long address, unsigned char *buf, unsigned short count);
//	int GetProgram(short deviceIDRemote, unsigned long address, unsigned char *buf, unsigned short count);

	// MMC setzen/lesen 
	int SetMMC(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);
	int GetMMC(short deviceIDRemote, unsigned long address, unsigned char *buf, int count);

	// Systembios aufrufen/verlassen
	int EnterBios(short deviceIDRemote, unsigned char protocolVersion);
	int ExitBios(short deviceIDRemote);

	// Interface setzen/lesen (RS232, TCPIP...)
	int SetInterface(short deviceIDRemote, unsigned char type);
	int GetInterface(short deviceIDRemote, unsigned char &type);
	
	// Baud-Rate setzen/lesen
	int SetBaud(short deviceIDRemote, unsigned long br);
	int GetBaud(short deviceIDRemote, unsigned long &br);

	// DeviceID setzen/lesen
	int SetDeviceIDBios(short deviceIDRemote, unsigned short id);
	int GetDeviceIDBios(short deviceIDRemote, unsigned short &id);

	// Bootloader aktivieren
	int EnterBootloader(short deviceIDRemote, unsigned char protocolVersion);

	// Monitor Mode vom XPort
	int EnterXPortMonitor(short deviceIDRemote, char *data);
	int EnterXPortMonitorExt(short deviceIDRemote, char *data);
	int CommandXPort(short deviceIDRemote, char *buf, int &count);
	int ExitXPortMonitor(short deviceIDRemote);
	int ExitXPortMonitorExt(short deviceIDRemote);

	// MatchPort Set Factory default
	int MatchPortSetFactoryDefault(short deviceIDRemote, char *buf, int &count);

	// Oeffnen einer Tabelle
	int TableOpen(short deviceIDRemote, char *name, int options, int &handle, char *reserved);
	// Schliessen der Tabelle
	int TableClose(short deviceIDRemote, int handle);

	// Setzen eines Filters auf die Tabelle
	int TableSetFilter(short deviceIDRemote, int handle, char *filter);
	// Lesen des Filters auf die Tabelle
	int TableGetFilter(short deviceIDRemote, int handle, char *filter, short &len);
	// Loeschen des Filters der Tabelle
	int TableClearFilter(short deviceIDRemote, int handle);
	
	// Ueberspringen von n Datensaetzen (offset kann negativ sein), ab top/aktuell/bottom
	int TableSkip(short deviceIDRemote, int handle, int offset, short origin);
	
	// Aendern eines Spaltenwerts, des aktuellen Datensatzes
	int TableSetField(short deviceIDRemote, int handle, char *name, char *value);
	// Abfragen eines Spaltenwerts, des aktuellen Datensatzes
	int TableGetField(short deviceIDRemote, int handle, char *name, char *value, short &len);

	// Abfragen einer Modulinformation
	int GetInfo(short deviceIDRemote, char *name, char *params, char *info, int &len);

	// Zutrittsbuchung durchfuehren
	int Entrance2Identification(short deviceIDRemote, char *TM, char *Id, char *Pin, char *status, int &statusLength);
	// LED, Relais auf Tuermodul schalten
	int Entrance2OnlineAction(short deviceID, char *TM, unsigned long mask, unsigned char type, unsigned char duration);

	// Transparentmodus aktivieren/deaktivieren
	int OpenComServerMode(short deviceIDRemote, char *comm, unsigned long baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose);
	int CloseComServerMode(short deviceIDRemote);

	// Aktive Meldungen aktivieren/deaktieren
	int SetNotifyOnDevice(short deviceIDRemote, int type, int state);

	// Softwareinformation der Anwendungsfirmware abfragen
	int GetSoftwareInfo(short deviceIDRemote, unsigned char *swi, int &infoLength);

	// Passwort pruefen
	int PasswordCheck(short deviceID, char *password);
	// Passwortschlssel abrufen
	int GetPasswordKey(short deviceID, char *key);

	// Virtuellen Tastendruck ausfhren.
	int PressVirtualKey(short deviceIDRemote, unsigned char vk, unsigned char flags, int *piError);

	// MODULE_TABLE

	// Datensatz an Fingerprint-Tabelle anfuegen.
	int FingerprintAppendRecord(short deviceIDRemote, unsigned char type, unsigned char *fingertemplate, int size);
	// Datensatz aus Fingerprint-Tabelle lesen.
	int FingerprintGetRecord(short deviceIDRemote, unsigned char &type, int pid, int fid, unsigned char *fingertemplate, int &size);
	// Datensatz aus Fingerprint-Tabelle loeschen.
	int FingerprintDeleteRecord(short deviceIDremote, int pid, int fid);
	// Tabelle mit Fingerprintdaten erzeugen
	int FingerprintList(short deviceIDRemote, int flags);
	// Backup der Fingerprints erstellen
	int FingerprintBackup(short deviceIDRemote, char *backupFile, int flags);
	// Wiederherstellen der Fingerprints
	int FingerprintRestore(short deviceIDRemote, char *backupFile, int flags);

	// Systemflags setzen/lesen
	int SetSystemFlags(short deviceIDRemote, unsigned long flags);
	int GetSystemFlags(short deviceIDRemote, unsigned long &flags);

	// MODULE_APPLICATION
	// Setzt den Font fr die Anzeige der Message und Info - Text
	int SetTypeFont(short deviceIDRemote, int iType, int *piError);
};

#endif // !defined(AFX_AE3CHANNEL_H__D822D71E_AB5D_4A01_8121_C09FDB26DE7D__INCLUDED_)

