/**
 * @file  Channel1_2".cpp
 * @brief Implementierung der Klasse CChannel1_2.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Channel1_2.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

void CChannel1_2::InitClass(short channelID, short deviceID)
{
	// Kommunikationsklasse
	m_channelBase = NULL;

	// Protokollversion Unbestimmt
	m_protocolVersion = 0;

	// Busnummer
	m_deviceID = (unsigned char)deviceID;
	m_deviceIDRemote = 0;

	// CallBack nicht definiert
	m_callBack1 = NULL;
}

CChannel1_2::~CChannel1_2()
{

}

//////////////////////////////////////////////////////////////////////
// Members
//////////////////////////////////////////////////////////////////////

unsigned short CChannel1_2::BinToUShort(unsigned char *Buf)
{
	unsigned short Val;

	Val = Buf[0];
	Val <<= 8;
	Val += Buf[1];
	return Val;
}

unsigned long CChannel1_2::BinToULong(unsigned char *Buf)
{
	unsigned long Val;

	Val = Buf[0];
	Val <<= 8;
	Val += Buf[1];
	Val <<= 8;
	Val += Buf[2];
	Val <<= 8;
	Val += Buf[3];
	return Val;
}

// Konvertierung MSB First
void CChannel1_2::UShortToBin(unsigned short Val,unsigned char *Buf)
{
	Buf[0] = (unsigned char)(Val >> 8);
	Buf[1] = (unsigned char)(Val & 0xFF);
}

// Konvertierung MSB First
void CChannel1_2::ULongToBin(unsigned long Val,unsigned char *Buf)
{
	Buf[0] = (unsigned char)((Val >> 24)&0xFF);
	Buf[1] = (unsigned char)((Val >> 16)&0xFF);
	Buf[2] = (unsigned char)((Val >> 8)&0xFF);
	Buf[3] = (unsigned char)(Val & 0xFF);
}

// Sende und Empfang von Modbus - Paket
int CChannel1_2::Command(unsigned char *pucTxBuf, int iTxCount, 
						unsigned char *pucRxBuf, int *piRxCount, 
						int *piError)
{
	// Bei 2er Protokollversion verzweigen.
	if (m_protocolVersion == 2) {

		int i;
		unsigned char buf[MAX_MB_PAKET];

		for (i=0; i < iTxCount; i++) {

			buf[i] = pucTxBuf[i];
		}

		if (!NewMB(GetDeviceIDRemote(), buf, &i, piError)) {

			return FALSE;
		}

		for (*piRxCount=0; *piRxCount < i; (*piRxCount)++) {

			pucRxBuf[*piRxCount] = buf[*piRxCount];
		}

		return TRUE;
	}

	// Paketlaenge
	unsigned int uiPaketLength;
	// Anzahl geschriebener oder gelesender Zeichen
	ULONG ulReadWriteCount = 0;

	// Nicht aktuelle Busnummer
	if (pucTxBuf[0] != GetDeviceIDRemote()) { 

		GetChannelBase().Trace("Command: Gesetzte Busnummer ist nicht die des zu sendenen Paketes.\n");	

		*piError = IV_ERROR_DEVICE_ID_REMOTE;
		return FALSE;
	}

//@@: Callback mit Parameter andenken.
	if (!ExeCallBack()) {
	
		*piError = IV_ERROR_USER_CANCEL;
		return FALSE;
	}


	// Befehl losschicken
	if (GetChannelBase().TransmitData((void*)pucTxBuf, iTxCount, (int&)ulReadWriteCount) != IV_ERROR_SUCCESS){
		
		GetChannelBase().Trace("Command: Paket konnte nicht versendet werden.\n");	
		
		*piError = IV_ERROR_TRANSMIT;
		return FALSE;
	}

	if (ulReadWriteCount != (unsigned)iTxCount) {
	
		GetChannelBase().Trace("Command: Paket wurde nicht vollstaendig versendet.(%i) von (%i)\n", ulReadWriteCount, iTxCount);	

		// zu wendig gesendet	
		*piError = IV_ERROR_TRANSMIT;
		return FALSE;
	}

	// Die ersten 4 Bytes der Antwort lesen
	if (GetChannelBase().ReceiveData((void*)pucRxBuf, 4, (int&)ulReadWriteCount, 4) != IV_ERROR_SUCCESS) {
			
		GetChannelBase().Trace("Command: Warunung beim Lesen des Paketkopfes.(%i) von (%i)\n", ulReadWriteCount, 4);	
	}
	// Empfang pruefen
	if (ulReadWriteCount == 0) {
		
		GetChannelBase().Trace("Command: Es wurde kein Paketkopf empfangen.\n");	
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}
	// Nachlesen?
	if (ulReadWriteCount < 4) {
	
		unsigned long lReadCount = 0;
		// Die ersten 4 Bytes der Antwort lesen
		if (GetChannelBase().ReceiveData((void*)&pucRxBuf[ulReadWriteCount], 4 - ulReadWriteCount, (int&)lReadCount, 2) != IV_ERROR_SUCCESS) {
				
			GetChannelBase().Trace("Command: Das Nachlesen zum Paketkopf schlug fehl.(%i) von (%i)\n", lReadCount, (4 - ulReadWriteCount));	

			// AW1 kann nicht gelesen werden
			*piError = IV_ERROR_RECEIVE;
			return FALSE; 
		}

		ulReadWriteCount += lReadCount;
	}
	// Empfang pruefen
	if (ulReadWriteCount != 4) {
		
		GetChannelBase().Trace("Command: Es wurden zu wenige Zeichen fr den Paketkopf empfangen.(%i) von (%i)\n", ulReadWriteCount, 4);	

		// Schnittstelle fuer naechste Anfrage bereinigen
		GetChannelBase().Flush();

		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	if (pucTxBuf[0] != pucRxBuf[0]) {

		GetChannelBase().Trace("Command: Busnummer von Sender und Empfaenger ungleich Tx(%i): Rx(%i)\n", pucTxBuf[0], pucRxBuf[0]);	

		// Schnittstelle fuer naechste Anfrage bereinigen
		GetChannelBase().Flush();

		// Absender falsch	
		*piError = IV_ERROR_ADDRESS;
		return FALSE; 
	}

	// Die Laenge vom Paket lesen
	uiPaketLength = BinToUShort(&pucRxBuf[1]);

	// Rest vom Paket lesen
	if(GetChannelBase().ReceiveData((void*)&pucRxBuf[4], uiPaketLength - 4, (int&)ulReadWriteCount, 5) != IV_ERROR_SUCCESS){

		GetChannelBase().Trace("Command: Warnung beim Lesen der Paketdaten.(%i) von (%i)\n", ulReadWriteCount, uiPaketLength - 4);	
	}
	if (ulReadWriteCount == 0) {
	
		GetChannelBase().Trace("Command: Es wurden keine Daten zum Paketlaenge empfangen.(%i) von (%i)\n", ulReadWriteCount, uiPaketLength - 4);	

		// AW2 zu kurz
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}
	// Nachlesen?
	if (ulReadWriteCount < (uiPaketLength - 4)) {
		
		unsigned long lReadCount = 0;
		// Die Paketdaten lesen
		if (GetChannelBase().ReceiveData((void*)&pucRxBuf[4 + ulReadWriteCount], (uiPaketLength - 4) - ulReadWriteCount, (int&)lReadCount, 2) != IV_ERROR_SUCCESS) {
				
			GetChannelBase().Trace("Command: Das Nachlesen der Paketdaten schlug fehl.(%i) von (%i)\n", lReadCount, (uiPaketLength - 4) - ulReadWriteCount);	

			// AW1 kann nicht gelesen werden
			*piError = IV_ERROR_RECEIVE;
			return FALSE; 
		}

		ulReadWriteCount += lReadCount;
	}

	if (ulReadWriteCount != (uiPaketLength - 4)) {
		
		GetChannelBase().Trace("Command: Die gelesene Paketlaenge stimmt nicht mit der angegebenen ueberein.(%i) von (%i)\n", ulReadWriteCount + 4, uiPaketLength);	

		// Schnittstelle fuer naechste Anfrage bereinigen
		GetChannelBase().Flush();

		// AW2 zu kurz
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// Paketgroesse zurueckgeben
	*piRxCount = uiPaketLength;

	// Kommando erfolgreich ausgefuehrt
	return TRUE;

}

int CChannel1_2::Command2(unsigned char *sbuf,int cnt,unsigned char *ebuf,int &ret, int *Err)
{
	unsigned long nb = 0;
	int Cnt;

	if (!ExeCallBack()) {*Err = IV_ERROR_BREAK; return FALSE;}
	if (cnt) {
		if (GetChannelBase().TransmitData(sbuf,cnt,(int&)nb) != IV_ERROR_SUCCESS) {*Err = IV_ERROR_TRANSMIT; return FALSE;}
		if (nb != (unsigned)cnt) {*Err = IV_ERROR_TRANSMIT; return FALSE;}
	}

	if (ret > 0) {
		
		// Lnge der Antwort ist bekannt
		if (GetChannelBase().ReceiveData(ebuf,ret,(int&)nb, 3) != IV_ERROR_SUCCESS) {*Err = IV_ERROR_RECEIVE; return FALSE;}
		if (nb == 0) {*Err = IV_ERROR_UNREACHABLE; return FALSE;}
		if (nb != (unsigned int)ret) {*Err = IV_ERROR_RECEIVE; return FALSE;}
		*Err = IV_ERROR_SUCCESS;
		GetChannelBase().Trace("OK..\n");

	} else {

		// Laenge der Antwort ist nicht bekannt und mu aus dem Protokoll ermittelt werden
		//	Protokoll-Kopf lesen
		Cnt = 8;
		if (GetChannelBase().ReceiveData(ebuf, Cnt, (int&)nb, 4) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("WARNING_00: ReceiveData(%i != %i)\n", Cnt, nb);
		}
		if (nb == 0) {

			*Err = IV_ERROR_UNREACHABLE; 
			GetChannelBase().Trace("ERROR_01: ReceiveData(%i != %i)\n", Cnt, nb);
			return FALSE;
		}
		// Nachlesen?
		if (nb > 0 && nb < (unsigned)Cnt) {

			unsigned long lnb = 0;
			// Noch nicht alle Byte gelesen -> versuchen weitere Bytes zu lesen
			if (GetChannelBase().ReceiveData(&ebuf[nb], Cnt-nb, (int&)lnb, 2) != IV_ERROR_SUCCESS) {
				
				*Err = IV_ERROR_RECEIVE; 
				GetChannelBase().Trace("ERROR_02: ReceiveData.2 (%i != %i)\n", Cnt-nb, lnb);
				return FALSE;
			}

			// Nachgelesene Anzahl Zeichen
			nb += lnb;
		}
		if (nb != (unsigned int)Cnt) {

			*Err = IV_ERROR_RECEIVE; 
			GetChannelBase().Trace("ERROR_03: ReceiveData(%i != %i)\n", Cnt, nb);
			return FALSE;
		}
	
		// Gesamtpaketlaenge
		Cnt = BinToUShort(&ebuf[V_O_LAENGE]);

		// Laenge auf plausibilitaet pruefen
		if (Cnt < 15 || Cnt > MAX_MB_PAKET_2) {

			*Err = IV_ERROR_PACKETLENGTH; 
			GetChannelBase().Trace("ERROR_04: %i\n", Cnt);
			GetChannelBase().Flush();
			return FALSE;
		}

		// Die restlichen Bytes lesen
		if (GetChannelBase().ReceiveData(&ebuf[8], Cnt-8, (int&)nb, 5) != IV_ERROR_SUCCESS) {

			GetChannelBase().Trace("WARNING_01: ReceiveData(%i != %i)\n", Cnt-8, nb);
		}
		if (nb == 0) {

			*Err = IV_ERROR_UNREACHABLE; 
			GetChannelBase().Trace("ERROR_01: ReceiveData(%i != %i)\n", Cnt, nb);
			return FALSE;
		}
		// Nachlesen?
		if (nb > 0 && nb < (unsigned)(Cnt-8)) {

			unsigned long lnb = 0;
			// Noch nicht alle Byte gelesen -> versuchen weitere Bytes zu lesen
			if (GetChannelBase().ReceiveData(&ebuf[nb], (Cnt-8)-nb, (int&)lnb, 2) != IV_ERROR_SUCCESS) {
				
				*Err = IV_ERROR_RECEIVE; 
				GetChannelBase().Trace("ERROR_02: ReceiveData.2 (%i, %i)\n", Cnt-nb, lnb);
				return FALSE;
			}

			// Nachgelesene Anzahl Zeichen
			nb += lnb;
		}
		if (nb != (unsigned int)(Cnt-8)) {

			*Err = IV_ERROR_RECEIVE; 
			GetChannelBase().Trace("ERROR_06: ReceiveData.2 (%i != %i)\n", Cnt-8, nb);
			return FALSE;
		}

		ret = Cnt;
		*Err = IV_ERROR_SUCCESS;
	}

	return TRUE;
}

// Einfache Anfrage ob Geraet mit Protokollversion 2 erreichbar.
int CChannel1_2::Request2(unsigned char deviceIDRemote)
{
	int Versuch,transmitCount,receiveCount;
	int len, *Len, err, *Err;
	Len = &len;
	Err = &err;

	// Laenge des alten Modbusprotokollpackets
	len = 0x0C;

	// Sendebuffer aufbereiten
	//	Adressen
	UShortToBin(deviceIDRemote, m_transmit);	// Adresse Empfnger
	UShortToBin(GetDeviceID(), &m_transmit[V_O_AD_ABSENDER_S2]);	// Adresse Sender

	// Paketlnge
	transmitCount = V_KOPF+1+1+1+2+1+len+2;	// Sendepaket
	receiveCount = -1;	// Empfangspaket
	UShortToBin(transmitCount,&m_transmit[V_O_LAENGE]);
	m_transmit[V_O_B_S2] = B_S2_NO;
	m_transmit[V_O_PAKET_NR] = 0;
	m_transmit[V_O_TIME_TO_LIVE] = 8;
	m_transmit[V_O_B_S3] = B_S3_NO;
	m_transmit[V_O_ANZ_BLOECKE] = 1;
	m_transmit[V_O_AKT_BLOCK] = 1;
	UShortToBin(GetDeviceID(), &m_transmit[V_O_AD_ABSENDER_S4]);	// Adresse Sender
	m_transmit[V_O_B_S4] = 0;//B_S4_LESEN|BB_S4_ANFRAGE;		// Anfrage lesen
	//ULongToBin(Add,&m_transmit[V_O_ADD]);

/*
	for (i=0;i<*Len;i++) {
		m_transmit[i+V_O_ADD] = buf[i];	// Daten in Sendebuffer kopieren
		// Bei OldMB entfllt die Daten-Adresse.
		// Daher sind die 4 fr die Adresse reservierten Byte schon Daten
	}
*/
	// Altes Packet in neues, als Daten einbinden
	m_transmit[V_O_ADD + 0] = (unsigned char)deviceIDRemote;
	m_transmit[V_O_ADD + 1] = 0;
	m_transmit[V_O_ADD + 2] = 0x0C;		// Laenge des Paketes
	m_transmit[V_O_ADD + 3] = 0x21;//MB_READ;
	ULongToBin(0x42000000/*MB_GET_VERSION*/, &m_transmit[V_O_ADD + 4]);
	UShortToBin(2, &m_transmit[V_O_ADD + 8]);
	MkPaketCrc(&m_transmit[V_O_ADD], 10);

	// CRC - Gesamtpacket
	MkPaketCrc(m_transmit, transmitCount-2);

	// Daten lesen incl. Wiederholungen
	for (Versuch=A_VERSUCHE_OMB - 1;Versuch < A_VERSUCHE_OMB; Versuch++) {
		// Im Fehlerfall zur Sicherheit groe Anzahl Zeichen lesen um Empfangsbuffer
		// des Betriebssystem zu lehren
		// Paket senden und Antwort abholen
		if (!Command2(m_transmit,transmitCount,m_receive,receiveCount,Err)) {
			continue;
		}
		// Antwort auswerten
		//	CRC prfen
		if (!CheckPaketCrc(m_receive,receiveCount)) { 
			*Err = IV_ERROR_CRC_PACKET;
			continue;
		}
		//	Empfnger-/Absender-Adresse prfen
		if (BinToUShort(m_receive) != GetDeviceID() || 
			  BinToUShort(&m_receive[V_O_AD_ABSENDER_S2]) != deviceIDRemote ||
			  BinToUShort(&m_receive[V_O_AD_ABSENDER_S4]) != deviceIDRemote) {
			*Err = IV_ERROR_ADDRESS;
			continue;
		}
		//	Lngenfeld prfen
		if (BinToUShort(&m_receive[V_O_LAENGE]) != receiveCount) {
			*Err = IV_ERROR_PACKETLENGTH;
			continue;
		}
		//	Befehl Schicht 4 prfen
		if (m_receive[V_O_B_S4] != (B_S4_LESEN|BB_S4_ANTWORT)) {
			*Err = IV_ERROR_COMMAND;
			continue;
		}
		// Anzahl der gelesenen Datenbyte berechnen
		*Len = receiveCount - (V_KOPF+1+1+1+2+1+2);
		// Daten in Buffer der Applikation kopieren
//		for(i=0;i<*Len;i++) buf[i] = m_receive[i+V_O_ADD];
		*Err = IV_ERROR_SUCCESS;
	
		// Ermittelte Protokollversion halten
		m_protocolVersion = 2;

		return TRUE; // Daten erfolgreich gelesen
	}

	return FALSE;
}

int CChannel1_2::NewMB(unsigned short deviceIDRemote,unsigned char *buf, int *Len, int *Err)
// Mod-Bus Protokoll (AEIII+) in neues Protokoll verpacken, senden und Antwort zurckgeben
//  AdEmpfaenger -> Adresse (Gertenummer) des Empfngers
//	buf	-> Buffer fr Mod-Bus-Paket senden und empfangen
//	Len	-> Anzahl zu sende Byte. Nach dem Aufruf -> Anzahl Empfangene Byte
//	Err -> Bei einem Fehler -> Fehlernummer
//  return -> 1 -> senden/empfangen erfolgreich
//			  0 -> Fehler -> Fehlernummer in Var. Err	
{
	int Versuch,transmitCount,receiveCount,i;

	// Sendebuffer aufbereiten
	//	Adressen
	UShortToBin(deviceIDRemote, m_transmit);	// Adresse Empfnger
	UShortToBin(GetDeviceID(),&m_transmit[V_O_AD_ABSENDER_S2]);	// Adresse Sender
	// Paketlnge
	transmitCount = V_KOPF+1+1+1+2+1+*Len+2;	// Sendepaket
	receiveCount = -1;	// Empfangspaket
	UShortToBin(transmitCount,&m_transmit[V_O_LAENGE]);
	m_transmit[V_O_B_S2] = B_S2_NO;
	m_transmit[V_O_PAKET_NR] = 0;
	m_transmit[V_O_TIME_TO_LIVE] = 5;
	m_transmit[V_O_B_S3] = B_S3_NO;
	m_transmit[V_O_ANZ_BLOECKE] = 1;
	m_transmit[V_O_AKT_BLOCK] = 1;
	UShortToBin(GetDeviceID(),&m_transmit[V_O_AD_ABSENDER_S4]);	// Adresse Sender
	m_transmit[V_O_B_S4] = 0;//B_S4_LESEN|BB_S4_ANFRAGE;		// Anfrage lesen
	//ULongToBin(Add,&m_transmit[V_O_ADD]);
	for (i=0;i<*Len;i++) {
		m_transmit[i+V_O_ADD] = buf[i];	// Daten in Sendebuffer kopieren
		// Bei OldMB entfllt die Daten-Adresse.
		// Daher sind die 4 fr die Adresse reservierten Byte schon Daten
	}
	MkPaketCrc(m_transmit,transmitCount-2);
	// Daten lesen incl. Wiederholungen
	for (Versuch=0;Versuch < A_VERSUCHE_OMB; Versuch++) {
		// Im Fehlerfall zur Sicherheit groe Anzahl Zeichen lesen um Empfangsbuffer
		// des Betriebssystem zu lehren
		if (Versuch >= 2) Command2(m_transmit,0,m_receive,i=MAX_MB_PAKET_2,Err);
		// Paket senden und Antwort abholen
		if (!Command2(m_transmit,transmitCount,m_receive,receiveCount,Err)) {
			continue;
		}
		// Antwort auswerten
		//	CRC prfen
		if (!CheckPaketCrc(m_receive,receiveCount)) { 
			*Err = IV_ERROR_CRC_PACKET;
			continue;
		}
		//	Empfnger-/Absender-Adresse prfen
		if (BinToUShort(m_receive) != GetDeviceID() || 
			  BinToUShort(&m_receive[V_O_AD_ABSENDER_S2]) != deviceIDRemote ||
			  BinToUShort(&m_receive[V_O_AD_ABSENDER_S4]) != deviceIDRemote) {
			*Err = IV_ERROR_ADDRESS;
			continue;
		}
		//	Lngenfeld prfen
		if (BinToUShort(&m_receive[V_O_LAENGE]) != receiveCount) {
			*Err = IV_ERROR_PACKETLENGTH;
			continue;
		}
		//	Befehl Schicht 4 prfen
		if (m_receive[V_O_B_S4] != (B_S4_LESEN|BB_S4_ANTWORT)) {
			*Err = IV_ERROR_COMMAND;
			continue;
		}
		// Anzahl der gelesenen Datenbyte berechnen
		*Len = receiveCount - (V_KOPF+1+1+1+2+1+2);
		// Daten in Buffer der Applikation kopieren
		for(i=0;i<*Len;i++) buf[i] = m_receive[i+V_O_ADD];
		*Err = IV_ERROR_SUCCESS;
		return TRUE; // Daten erfolgreich gelesen
	}
	return FALSE;
}

// Testen ob PZE - Master IV im Bootloader oder Applicationsmodus ist.
// return
// 0 -> Kommunikationsfehler
// 1 -> Applicationsmodus
// 2 -> Bootloadermodus
int CChannel1_2::GetModus(int *error)
{
	unsigned char tmp[2];

	// Befehl ist nur im Zusammenhang mit AEIV sinnvoll.
//	if (GetChannelType() != ClassSerial && GetChannelType() != ClassSocket) {

//		*error = IV_ERROR_CHANNEL_CLOSED;
//		return FALSE;
//	}

	// Allgemeine Erreichbarkeit pruefen
	if (ReadXByte(0, 2, tmp, error)) {

		// Der Befehl wird vom Bootloader nicht unterstuetzt
		// MB_GET_VERSION
		if (ReadXByte(0x42000000, 2, tmp, error)) {

			return 2;
		}

		return 1;
	}

	return FALSE;
}

// CRC berechnen und an das Ende des Buffers anhngen
void CChannel1_2::MkPaketCrc(unsigned char *buf,int cnt)	
{
	int i,j;
	unsigned short int crc;
  
	crc = 0xFFFF;
    for (i=0;i<cnt;i++) {
        crc = crc ^ buf[i];
        for (j=0;j<8;j++) {
            if (crc & 0x1) {crc >>= 1; crc = crc^0xA001;} else crc >>= 1;
        }
    }
    buf[cnt] = (unsigned char)(crc&0xFF);
    buf[cnt+1] = (unsigned char)(crc>>8);
}

int CChannel1_2::CheckPaketCrc(unsigned char *buf, int cnt)
{
    unsigned short int crc;
    int i,j;

    crc = 0xFFFF;
    for (i=0;i<cnt-2;i++) {
        crc = crc ^ buf[i];
        for (j=0;j<8;j++) {
            if (crc & 0x1) {crc >>= 1; crc = crc^0xA001;} else crc >>= 1;
        }
    }
    if (buf[cnt-2]!=(unsigned char)(crc&0xFF)) return FALSE;
    if (buf[cnt-1]!=(unsigned char)(crc>>8)) return FALSE;
    return TRUE;
}

// Schreiben in AEIII+ (High-Level)
// Add -> Byte Adresse
// Cnt -> Anzahl Zeichen (max. 1MByte)
// Buf -> Buffer fuer empfangene Zeichen
int CChannel1_2::WriteMem(unsigned long Add, int Cnt, unsigned char *Buf, int *piError, int MaxB)
{
	int i, j, ac, End = FALSE, ok;

	// Fehler loeschen
	*piError = IV_ERROR_SUCCESS;

	if (m_protocolVersion == 2) {

		if (MaxB == 0) {
			// Wenn EEPROM geschrieben wird
			MaxB = 8;
		} else if (MaxB == 32) {

		} else {
			MaxB = MAX_MB_PAKET_DATA;
		}
	} else {

		// Bis zu (Gesamtpaket[ 0xFF ] - Overhead[ MAX_MB_PAKET_HEADER ]) Byte Daten in Geraet schreiben
		if (MaxB > MAX_MB_PAKET_DATA) {
		
			*piError = IV_ERROR_FUNCTION_PARAMETER;
			return FALSE;
		}
	}

	// Daten in Pakete zerlegen
	for (i=0; !End; i += MaxB) {

		// Aufteilen
		if ((Cnt - i) < MaxB) {

			ac = Cnt - i;
		} else {

			ac = MaxB;
		}

		// Noch zu versendende Daten
		if ( ac > 0 ) {

			ok = FALSE;
			
			// Senden in mehreren Versuchen
			for (j=0; j < 4 && !ok; j++) {

				if (WriteXByte(Add + i, ac, &Buf[i], piError)) {

					ok = TRUE;
				} else {

					// Abbruch durch CallBack!
					if (*piError == IV_ERROR_USER_CANCEL) {
						
						return FALSE;
					}

					Sleep(50);
				}
			}

			// Senden Fehlgeschlagen?
			if (!ok) {

				// Letzten Fehler zurueckgeben
				return FALSE;
			}

		} else {

			// Alle versendet
			End = TRUE;
		}
	}

	return TRUE;
}

// X Byte in AEIII+ schreiben (Low-Level)
// Add -> Byte Adresse
// Cnt -> Anzahl Zeichen (max. 242 Byte)
// Buf -> Buffer fuer empfangene Zeichen
int CChannel1_2::WriteXByte (unsigned long Add, int Cnt, unsigned char *Buf, int *piError)
{
	int ret, i, h;
	unsigned char WBuf[MAX_WBUF];
	unsigned char RBuf[MAX_RBUF];

	if (Cnt > MAX_MB_PAKET_DATA) {
	
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	}

	// Modbus-Befehl aufbauen
	WBuf[0] = GetDeviceIDRemote();
	h = Cnt + 10;
	WBuf[1] = (unsigned char)(h >> 8);
	WBuf[2] = (unsigned char)(h & 0xFF);	// Laenge des Paketes
	WBuf[3] = MB_WRITE;
	ULongToBin(Add, &WBuf[4]);
	
	// Daten in Paket kopieren
	for (i=0; i < Cnt; i++) {

		WBuf[i + 8] = Buf[i];
	}
	MkPaketCrc(WBuf, 8 + Cnt); 
	ret = -1;

	bool bCRCFail = false;

crc_fail:

	if (!Command(WBuf, 10 + Cnt, RBuf, &ret, piError)) {
		
		return FALSE;
	}

	// Mindestens 6 fuer Fehlerpaket
	if (ret <= 4) {
	
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	if (!CheckPaketCrc(RBuf, ret)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	if (RBuf[3] == MB_WRITE && ret == 12) {

		return TRUE;

	} else if (RBuf[3] == (MB_WRITE | 0x80) && ret == 7 && RBuf[4] == 0x22) {

		Sleep(150);
		return TRUE;

	} 

	// Antwort vom Falschen Geraet oder nicht sinnvoll
	*piError = IV_ERROR_ADDRESS;
	return FALSE;
}

// Lesen aus AEIII+ (High-Level) 
// Add -> Byte Adresse
// Cnt -> Anzahl Zeichen (max. 1MByte)
// Buf -> Buffer fuer empfangene Zeichen
int CChannel1_2::ReadMem(unsigned long Add, int Cnt, unsigned char *Buf, int *piError)
{
	int i, j, ac, End = FALSE, ok, MaxB = 32;

	// Fehler loeschen
	*piError = IV_ERROR_SUCCESS;

	// Bis zu 32 Byte pro Paket aus Geraet lesen (anfordern).
	// Daten in Paket zerlegen
	for (i=0; !End; i += MaxB) {

		// Zerlegen
		if ((Cnt - i) < MaxB) {

			ac = Cnt - i;
		} else {

			ac = MaxB;
		}

		// Daten vorhanden?
		if (ac > 0) {

			ok = FALSE;

			// Versuchtes anfragen
			for (j=0; j < 4 && !ok; j++) {

				if (ReadXByte(Add + i, ac, &Buf[i], piError)) {
					
					ok = TRUE; 
				} else {
					
					// Abbruch durch CallBack!
					if (*piError == IV_ERROR_USER_CANCEL) {
						
						return FALSE;
					}

					Sleep(50);
				}
			}
			
			// Fehlgeschlagen
			if (!ok) {

				// Letzten Fehler zurueckgeben
				return FALSE;
			}

		} else {

			End = TRUE;
		}
	}

	return TRUE;
}

// Einfache Anfrage ob Geraet mit Protokollversion 2 erreichbar.
int CChannel1_2::Request(unsigned char deviceIDRemote)
{
	int ret, err;
	unsigned char WBuf[MAX_WBUF];
	unsigned char RBuf[MAX_RBUF];

	// Busnummer einstellen
	if (!SetDeviceIDRemote(deviceIDRemote)) {

		//*piError = IV_ERROR_DEVICE_ID_REMOTE;
		return FALSE;
	}

	// Modbus-Befehl aufbauen
	WBuf[0] = deviceIDRemote;
	WBuf[1] = 0;
	WBuf[2] = 0x0C;		// Laenge des Paketes
	WBuf[3] = MB_READ;
	ULongToBin(0, &WBuf[4]);
	UShortToBin(2, &WBuf[8]);
	MkPaketCrc(WBuf, 10);

	ret = -1;

	if (!Command(WBuf, 12, RBuf, &ret, &err)) {

		return FALSE;
	}

	if (ret <= 4) {
	
		//*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// Paket pruefen
	if (!CheckPaketCrc(RBuf, ret)) {
	
		//*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	if (BinToUShort(&RBuf[1]) != (2 + 10)) {
	
		//*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// auf Fehler pruefen
	if (RBuf[3] != MB_READ || RBuf[0] != GetDeviceIDRemote()) {

		//*piError = IV_ERROR_ADDRESS;
		return FALSE;
	}

	// Ermittelte Protokollversion halten
	m_protocolVersion = 1;

	return TRUE;
}

// X Byte von AEIII+ lesen (Low-Level)
// Add -> Byte Adresse
// Cnt -> Anzahl Zeichen (max. 32 Byte)
// Buf -> Buffer fuer empfangene Zeichen
int CChannel1_2::ReadXByte (unsigned long Add, int Cnt, unsigned char *Buf, int *piError)
{
	int ret, i;
	unsigned char WBuf[MAX_WBUF];
	unsigned char RBuf[MAX_RBUF];

	if (Cnt > MAX_MB_PAKET_DATA) {
		
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	}

	if (Cnt > 32) {

		GetChannelBase().Trace("?");
	}

	// Modbus-Befehl aufbauen
	WBuf[0] = GetDeviceIDRemote();
	WBuf[1] = 0;
	WBuf[2] = 0x0C;		// Laenge des Paketes
	WBuf[3] = MB_READ;
	ULongToBin(Add, &WBuf[4]);
	UShortToBin(Cnt, &WBuf[8]);
	MkPaketCrc(WBuf, 10);
	ret = -1;

	bool bCRCFail = false;

crc_fail:

	if (!Command(WBuf, 12, RBuf, &ret, piError)) {

		return FALSE;
	}

	if (ret <= 4) {
	
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// Paket pruefen
	if (!CheckPaketCrc(RBuf, ret)) {
	
		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	if (RBuf[3] == (MB_READ | 0x80) && ret == 7 && RBuf[4] == 0x22) {

		Sleep(150);
	}

	if (BinToUShort(&RBuf[1]) != (Cnt + 10)) {
	
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// auf Fehler pruefen
	if (RBuf[3] != MB_READ || RBuf[0] != GetDeviceIDRemote()) {

		GetChannelBase().Trace("Fehler Befehl\n");
		*piError = IV_ERROR_ADDRESS;
		return FALSE;
	}

	// Adresse pruefen
	for (i=0; i < 4; i++) {

		if (RBuf[i+4] != WBuf[i+4]) {

			*piError = IV_ERROR_RECEIVE;
			return FALSE;
		}
	}

	// Daten kopieren
	for (i=0; i < Cnt; i++) {

		Buf[i] = RBuf[i + 8];
	}

	return TRUE;
}

void CChannel1_2::SetCBFunction(int (*pFunction)()) 
{
	m_callBack1 = pFunction;
}

int CChannel1_2::ExeCallBack()
{
	if (m_callBack1) {

		return m_callBack1();

	} else {

		return TRUE;
	}
}

// Abfrage ob Datensaetze im AEIII+ vorliegen.
// Erst ab Datensatzweisem lesen.
int CChannel1_2::CheckData(unsigned char deviceIDRemote, int *piError)
{
	unsigned char ucIsData = FALSE;

	// Testen ob Schnittstelle geoeffnet
	if (GetChannelBase().GetType() == CChannelBase::Undefined) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return -1;
	}

	// Busnummer wechsel, ohne Daten zu loeschen.
	if (!SetDeviceIDRemote(deviceIDRemote)) {

		*piError = IV_ERROR_DEVICE_ID_REMOTE;
		return -1;
	}

	// Abfrage ob Daten vorhanden
	if (!ReadMem(MB_RECORD_AVAILABLE, 1, &ucIsData, piError)) {
		
		return -1;
	}

	return ucIsData;
}

// Lesen eines Datensatzes.
// Rueckgabewerte: -1 - Lesen gescheitert
//					0 - Keine Datensaetzte vorhanden
//					1 - Online Datensatz gelesen
//					2 - Wiederholt Online Datensatz gelesen
//					3 - Offline Datensatz gelesen
//					4 - Wiederholt Offline Datensatz gelesen
int CChannel1_2::ReadRecord(unsigned char deviceIDRemote, unsigned char *pucRxBuf, int *piRxCount, int *piError)
{
	if (GetChannelBase().GetType() == CChannelBase::Undefined) {
		
		*piError = IV_ERROR_CHANNEL_CLOSED;
		return -1;
	}

	int iResult = -1;
	//Senden des Lese-Datensatz-Befehl
	m_pucTxBuf[0] = deviceIDRemote;			// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x06;				// Befehlslaenge = 6
	m_pucTxBuf[3] = MB_CMD_READ_RECORD;	// Befehl: Lesen eines Datensatzes
	MkPaketCrc(m_pucTxBuf, 4);
	
	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 3) && !bEnd; i++) { // 3 Versuche 

		if(!Command(m_pucTxBuf, 6, pucRxBuf, &iResult, piError)) {
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

		} else {
		
			bEnd = true;
		}
	}

	// Fehlerhafte versuche
	if (!bEnd) {

		return -1;
	}

	// Paket kleiner als Mindestlaenge?
	if(iResult < 6){

		*piError = IV_ERROR_RECEIVE;
		return -1;
	}
	
	// Pruefsumme OK?
	if(!CheckPaketCrc(pucRxBuf, iResult)){
		
		if (!bCRCFail) { // nach CRC - Fehler nochmals versuchen

			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return -1;
	}
	
	*piRxCount = iResult;

	// Pruefen der Antwort fuer Rueckgabe
	switch(pucRxBuf[3]) {
	case MB_ANSW_NO_RECORD:         // kein Datensatz vorhanden 

		return 0;

	case MB_ANSW_ONLINE_RECORD:     // Online Datensatz 

		return 1;

	case MB_ANSW_OLD_ONLINE_RECORD: // wiederholt gesendeter Online Datensatz 

		return 2;

	case MB_ANSW_OFFLINE_RECORD:    // Offline Datensatz 

		return 3;

	case MB_ANSW_OLD_OFFLINE_RECORD:// wiederholt gesendeter Offline Datensatz 

		return 4;
	};

	// Keine gueltige Antwort auf Datensatzanfrage
	*piError = IV_ERROR_ADDRESS;

	return -1;
}

// Quittieren eines Datensatzes
// Rueckgabewerte:	-1 - Fehler bei Quittierung
//					 0 - Keine Quittung erforderlich
//					 1 - Quittierung erfolgt (online)
//					 2 - Quittierung erfolgt (wiederholte online)
//					 3 - Quittierung erfolgt (offline)
//					 4 - Quittierung erfolgt (wiederholte offline)
int CChannel1_2::QuitRecord(unsigned char deviceIDRemote, unsigned char *pucRxBuf, int *piRxCount, int *piError)
{
	int iResult = FALSE;
	
	if (GetChannelBase().GetType() == CChannelBase::Undefined){
		
		*piError = IV_ERROR_CHANNEL_CLOSED;
		return -1;
	}

	// Senden des Lese - Befehls
	m_pucTxBuf[0] = deviceIDRemote;			// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x06;				// Befehlslaenge = 6
	m_pucTxBuf[3] = MB_CMD_ACK_RECORD;	// Befehl: Letzen Datensatz quittieren
	MkPaketCrc(m_pucTxBuf, 4);
	
	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, 6, pucRxBuf, &iResult, piError)) {
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

		} else {
			
			bEnd = true;
		}
	}

	// Fehlerhafte Anfrage
	if (!bEnd) {

		return -1;
	}

	// Paket kleiner als Mindestlaenge?
	if(iResult < 6) {
		
		*piError = IV_ERROR_RECEIVE;
		return -1;
	}

	// Pruefsumme OK?
	if(!CheckPaketCrc(pucRxBuf, iResult)) {
		
		if (!bCRCFail) { // Nach CRC - Fehler nochmals anfordern
			
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return -1;
	}
	
	*piRxCount = iResult;

	switch (pucRxBuf[3]) {
	case 0: // Keine vorherige Datenabholung
	case 2: // Keine vorherige Datenabholung (wiederholt)
		return 0; // Keine Quitierung erforderlich
		 
	case MB_ANSW_ACK_ON_RECORD:          // Quittung erhalten (fuer Online-Datensatz)
	case MB_ANSW_TIMEOUT_ACK_RECORD:     // Quittung kam nach Timeout (fuer Online-Datensatz)
		return 1;

	case MB_ANSW_OLD_ACK_ON_RECORD:      // wiederholt gesendete Quittung
	case MB_ANSW_OLD_TIMEOUT_ACK_RECORD: // wiederholte Quittung nach Timeout
		return 2;

	case MB_ANSW_ACK_OFF_RECORD:	     // Quittung erhalten (fuer Offline-Datensatz)
		return 3;

	case MB_ANSW_OLD_ACK_OFF_RECORD:     // wiederholt gesendete Quittung
		return 4;
	}

	// Keine gueltige Antwort auf Datensatzanfrage
	*piError = IV_ERROR_ADDRESS;
	return -1;
}


// Loescht alle Datensaetze
int CChannel1_2::ClearData(unsigned char deviceIDRemote, int *piError)
{
	int iRes;

	if (GetChannelBase().GetType() == CChannelBase::Undefined){

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return FALSE;
	}

	// Senden des Lesen-Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x06;		// Befehlslaenge = 6
	m_pucTxBuf[3] = MB_CMD_DEL;	// Befehl -Loeschen-

	MkPaketCrc(m_pucTxBuf, 4);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, 6, m_pucRxBuf, &iRes, piError)){
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iRes != 6){
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// Antwort muss genauso aussehen wie der Befehl, also jedes Byte ueberpruefen
	// Checksumme muss daher nicht ueberprueft werden
	for(int counter=0; counter < iRes; counter++) {
		
		if(m_pucTxBuf[counter] != m_pucRxBuf[counter]){

			if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

				// Nachfolgende Zeichen verwerfen
				GetChannelBase().Flush();

				bCRCFail = true;
				goto crc_fail;
			}

			*piError = IV_ERROR_CRC_PACKET;
			return FALSE;
		}
	}

	return TRUE;
}

// Schliesst Relais mit Nummer fuer Zeit.
int CChannel1_2::CloseRelay(unsigned char deviceIDRemote, int iNum, int iTime, int *piError)
{
	int iResult = -1, iError;

	// Senden des Lese - Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x0E;		// Befehlslaenge = 14
	m_pucTxBuf[3] = MB_WRITE;	// Befehl: Relais schreiben.
	
	// Adresse fuer Relais
	ULongToBin(MB_RELAY, &m_pucTxBuf[4]);

	m_pucTxBuf[8] = 0x80;           // Relais schliessen
	m_pucTxBuf[8] |= (iNum & 0x0F); // Relaisnummer

	m_pucTxBuf[9] = 0;

	// Zeit die Relais geschlossen wird.
	UShortToBin(iTime, &m_pucTxBuf[10]);
	
	MkPaketCrc(m_pucTxBuf, 12);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, 14, m_pucRxBuf, &iResult, &iError)){
			
			*piError = iError;
			Sleep(50);
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iResult != 14){
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// CRC - pruefen
	if (!CheckPaketCrc(m_pucRxBuf, iResult)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	return TRUE;
}

// Ermittelt aktuellen Zustand von Relais mit Nummer fuer Zeit.
int CChannel1_2::GetRelayState(unsigned char deviceIDRemote, int iNum, int *piState, int *piTime, int *piError)
{
	if (GetChannelBase().GetType() == CChannelBase::Undefined){

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return FALSE;
	}

	int iResult = FALSE;

	// Senden des Lesen-Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus - Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x0E;		// Befehlslaenge = 14
	m_pucTxBuf[3] = MB_READ;	// Befehl: Relais lesen
	
	// Adresse fuer Relais
	ULongToBin(MB_RELAY, &m_pucTxBuf[4]);

	m_pucTxBuf[8] = (iNum & 0x0F); // Nummer des Relais
	m_pucTxBuf[9] = 0;
	// Dauer
	m_pucTxBuf[10] = 0;
	m_pucTxBuf[11] = 0;

	MkPaketCrc(m_pucTxBuf, 12);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, 14, m_pucRxBuf, &iResult, piError)){
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iResult != 14){
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// CRC - pruefen
	if (!CheckPaketCrc(m_pucRxBuf, iResult)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	if (m_pucRxBuf[8] & 0x80) {
	
		*piState = 1;
		*piTime = BinToUShort(&m_pucRxBuf[10]);
	} else {
	
		*piState = 0; // nicht gesetzt
		*piTime = 0;  // Zeit nicht relevant
	}

	return TRUE;
}

// Oeffnet Relais mit Nummer.
int CChannel1_2::OpenRelay(unsigned char deviceIDRemote, int iNum, int *piError)
{
	if (GetChannelBase().GetType() == CChannelBase::Undefined){

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return FALSE;
	}

	int iResult = FALSE;

	// Senden des Lese - Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x0E;		// Befehlslaenge = 14
	m_pucTxBuf[3] = MB_WRITE;	// Befehl: Relais schreiben
	
	// Adresse fuer Relais
	ULongToBin(MB_RELAY, &m_pucTxBuf[4]);

	m_pucTxBuf[8] = (iNum & 0x0F);
	m_pucTxBuf[9] = 0x00;
	m_pucTxBuf[10] = 0x00;
	m_pucTxBuf[11] = 0x00;

	MkPaketCrc(m_pucTxBuf, 12);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, 14, m_pucRxBuf, &iResult, piError)){
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iResult != 14){
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	// CRC - pruefen
	if (!CheckPaketCrc(m_pucRxBuf, iResult)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	return TRUE;
}

// Daten einer Variablen setzen.
int CChannel1_2::SetGlobVar(unsigned char deviceIDRemote, unsigned char *pucVar, int iType, char *value, int *piError)
{
	if (GetChannelBase().GetType() == CChannelBase::Undefined) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return FALSE;
	}

	int iResult = FALSE;

	// Laenge ermitteln.
	int iValueLength = strlen((const char*)value);

	// Senden des Lesen-Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus-Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x0C;		// Rahmen Befehlslaenge = 12
	// Maximal 60 Zeichen schreiben.
	if (iValueLength > 60) {
		
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	}
	iValueLength++; // 0 terminie

	m_pucTxBuf[3] = MB_WRITE;	// Befehl: Globale Variable schreiben.
	
	// Adresse fuer Globale Variable
	ULongToBin(MB_GLOBAL_VAR, &m_pucTxBuf[4]);

	unsigned int iNameLength = 0;
	switch(iType) {
	case 0:
		m_pucTxBuf[8] = (*pucVar & 0x7F); // Nummer der zu lesenden Globalen Variable
		break;
	case 1:
		m_pucTxBuf[8] = 0x80;
		iNameLength = (strlen((const char*)pucVar) & 0x7F) + 1;
		strcpy((char*)&m_pucTxBuf[10], (const char*)pucVar);
		m_pucTxBuf[10 + (iNameLength - 1)] = 0;
		break;
	case 2:
		m_pucTxBuf[8] = 0x80;
		iNameLength = (strlen((const char*)pucVar) & 0x7F) + 1;
		strcpy((char*)&m_pucTxBuf[10], (const char*)pucVar);
		m_pucTxBuf[10 + (iNameLength - 1)] = 0;
		break;
	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	}

	m_pucTxBuf[9] = 0;

	// Wert in Paket kopieren
	strcpy((char*)&m_pucTxBuf[10 + iNameLength], (const char*)value);
	m_pucTxBuf[10 + iNameLength + (iValueLength - 1)] = 0;

	UShortToBin( m_pucTxBuf[2] + iNameLength + iValueLength + 2, &m_pucTxBuf[1]);
	MkPaketCrc(m_pucTxBuf, BinToUShort(&m_pucTxBuf[1]) - 2);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, BinToUShort(&m_pucTxBuf[1]), m_pucRxBuf, &iResult, piError)){
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iResult != BinToUShort(&m_pucRxBuf[1])) {
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	if(BinToUShort(&m_pucRxBuf[1]) == 6) {
		
		// Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
		// und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
		if (m_pucRxBuf[3] & 0x80) {
			// Fehler liegt bei Wert
			*piError = IV_ERROR_VARIABLE_VALUE;
		} else {
			// Variable nicht bekannt
			*piError = IV_ERROR_VARIABLE_UNDEFINED;
		}

		return FALSE;
	}

	// CRC - pruefen
	if (!CheckPaketCrc(m_pucRxBuf, iResult)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}

	return TRUE;
}

// Daten einer Globalen Variablen lesen.
int CChannel1_2::GetGlobVar(unsigned char deviceIDRemote, unsigned char *pucVar, int iType, char *value, int iValueLength, int *piError)
{
	if (GetChannelBase().GetType() == CChannelBase::Undefined){

		*piError = IV_ERROR_CHANNEL_CLOSED;
		return FALSE;
	}

	// muss mindestens ein Zeichen anfordern.
	if ((iValueLength - 1) <= 0) {
	
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	} 

	int iResult = FALSE;

	// Senden des Lesen-Befehls
	m_pucTxBuf[0] = deviceIDRemote;	// Bus - Nummer
	m_pucTxBuf[1] = 0x00; 
	m_pucTxBuf[2] = 0x0C;		// Rahmen Befehlslaenge = 12
	m_pucTxBuf[3] = MB_READ;	// Befehl: Globale Variable lesen.
	
	// Adresse fuer Globale Variable
	ULongToBin(MB_GLOBAL_VAR, &m_pucTxBuf[4]);

	unsigned int iNameLength = 0;
	switch(iType) {
	case 0:
		m_pucTxBuf[8] = (*pucVar & 0x7F); // Nummer der zu lesenden Globalen Variable
		break;
	case 1:
		m_pucTxBuf[8] = 0x80;
		iNameLength = (strlen((const char*)pucVar) & 0x7F) + 1;
		strcpy((char*)&m_pucTxBuf[10], (const char*)pucVar);
		m_pucTxBuf[10 + (iNameLength - 1)] = 0;
		break;
	case 2:
		m_pucTxBuf[8] = 0x80;
		iNameLength = (strlen((const char*)pucVar) & 0x7F) + 1;
		strcpy((char*)&m_pucTxBuf[10], (const char*)pucVar);
		m_pucTxBuf[10 + (iNameLength - 1)] = 0;
		break;
	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;
	}

	// Laenge des zur Verfuegung stehenden Puffer - Nullterminie
	m_pucTxBuf[9] = (iValueLength > 60) ? 59 : (iValueLength - 1); 

	UShortToBin(m_pucTxBuf[2] + iNameLength + 2, &m_pucTxBuf[1]);
	MkPaketCrc(m_pucTxBuf, BinToUShort(&m_pucTxBuf[1]) - 2);

	bool bEnd = false, bCRCFail = false;

crc_fail:

	for (int i=0; (i < 4) && !bEnd; i++) {

		if(!Command(m_pucTxBuf, BinToUShort(&m_pucTxBuf[1]), m_pucRxBuf, &iResult, piError)){
			
			Sleep(50);
			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();
			
		} else {
		
			bEnd = true;
		}
	}

	if (!bEnd) {
	
		return FALSE;
	}

	if(iResult != BinToUShort(&m_pucRxBuf[1])){
		
		// Paket ungleich der Vorgeschriebenen Antwortlaenge?
		*piError = IV_ERROR_RECEIVE;
		return FALSE;
	}

	if(BinToUShort(&m_pucRxBuf[1]) == 6) {
		
		// Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
		// und wenn der Wert nicht gelesen werden konnte wird das hoechste bit gesetzt.
		if (m_pucRxBuf[3] & 0x80) {
			// Fehler liegt bei Wert
			*piError = IV_ERROR_VARIABLE_VALUE;
		} else {
			// Variable nicht bekannt
			*piError = IV_ERROR_VARIABLE_UNDEFINED;
		}

		return FALSE;
	}

	// CRC - pruefen
	if (!CheckPaketCrc(m_pucRxBuf, iResult)) {

		if (!bCRCFail) { // nach CRC - Fehler nochmals anfordern

			// Nachfolgende Zeichen verwerfen
			GetChannelBase().Flush();

			bCRCFail = true;
			goto crc_fail;
		}

		*piError = IV_ERROR_CRC_PACKET;
		return FALSE;
	}


	// Pruefe die zu kopierende Laenge.
	if (m_pucRxBuf[9] > (iValueLength - 1)) {
	
		strncpy(value, (const char*)&m_pucRxBuf[10 + iNameLength], (iValueLength - 1));
		value[iValueLength - 1] = 0;

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		return FALSE;

	} else {

		strncpy(value, (const char*)&m_pucRxBuf[10 + iNameLength], m_pucRxBuf[9]);
		value[m_pucRxBuf[9]] = 0;
	}

	return TRUE;
}

int CChannel1_2::GetHardwareInfo(short deviceIDRemote, TDFHardwareInfo &hwInfo, int *piError)
{
	// Struktur initialisieren
	memset(&hwInfo, 0, sizeof(TDFHardwareInfo));


	// Geraetetyp ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FCB, 1, &hwInfo.typeDevice, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeDevice == 0xFF) {

		hwInfo.typeDevice = 0; // PZE-MasterIV
	}

	// Boardtyp ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FCD, 1, &hwInfo.typeBoard, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeBoard == 0xFF) {

		hwInfo.typeBoard = 0x13; // Baugruppe 1.3
	}
	
	// RS485 ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FCA, 1, &hwInfo.componentRS485_0, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentRS485_0 == 0xFF) {

		hwInfo.componentRS485_0 = 0; // Nein
	}

	// RS485 und TCPIP ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FD2, 1, &hwInfo.typeTCPIP, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeTCPIP == 0xFF) {

		hwInfo.typeTCPIP = 0;
	}
	// RS485 und TCPIP pruefen
	if (hwInfo.componentRS485_0 == 0 && hwInfo.typeTCPIP == 0) {
		// Wenn keines von beiden, dann wird RS485 angenommen.
		hwInfo.componentRS485_0 = 1;
	}
	
	// ModemMC35i ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FD3, 1, &hwInfo.typeModemMC, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeModemMC == 0xFF) {

		hwInfo.typeModemMC = 0;
	}
	
	// RS485 (intern) ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FC9, 1, &hwInfo.componentRS485_1, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentRS485_1 == 0xFF) {

		hwInfo.componentRS485_1 = 1;
	}

	// Displaytyp ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FC8, 1, &hwInfo.typeDisplay, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeDisplay == 0xFF) {

		hwInfo.typeDisplay = 1; // Lehner Klein
	}

	// Tastaturtyp ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FC6, 1, &hwInfo.typeKeypad, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeKeypad == 0xFF) {

		hwInfo.typeKeypad = 1; // 9 Tasten
	}
	
	// Transpondertyp ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FD5, 1, &hwInfo.typeTransponder, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeTransponder == 0xFF) {

		hwInfo.typeTransponder = 0; // Kein Transponder
	}

	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FC7, 1, &hwInfo.typeMMC, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeMMC == 0xFF) {

		hwInfo.typeMMC = 0; // Keine MMC
	}
	
	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0F99, 1, &hwInfo.typeRadio, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeRadio == 0xFF) {

		hwInfo.typeRadio = 0;
	}
	
	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FBC, 1, &hwInfo.typeGPS, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeGPS == 0xFF) {

		hwInfo.typeGPS = 0;
	}
	
	// Komponende ermitteln
	if (!ReadMem(MB_START_EEP + 0x0FBD, 1, &hwInfo.componentUSB, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentUSB == 0xFF) {

		hwInfo.componentUSB = 0;
	}
	
	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0F98, 1, &hwInfo.typeDCF77, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeDCF77 == 0xFF) {

		hwInfo.typeDCF77 = 0;
	}
	
	// Komponende ermitteln
	if (!ReadMem(MB_START_EEP + 0x0EDC, 4, (unsigned char*)&hwInfo.componentInAnalog, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentInAnalog == 0xFFFFFFFF) {

		hwInfo.componentInAnalog = 0;
	}
	
	// Komponende ermitteln
	if (!ReadMem(MB_START_EEP + 0x0F90, 4, (unsigned char*)&hwInfo.componentInDigital, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentInDigital == 0xFFFFFFFF) {

		hwInfo.componentInDigital = 0;
	}
	
	// Komponende ermitteln
	if (!ReadMem(MB_START_EEP + 0x0F94, 4, (unsigned char*)&hwInfo.componentOutDigital, piError)) {
		
		return FALSE;
	}
	if (hwInfo.componentOutDigital == 0xFFFFFFFF) {

		hwInfo.componentOutDigital = 0;
	}
	
	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0EDB, 1, &hwInfo.typeFingerprint, piError)) {
		
		return FALSE;
	}
	if (hwInfo.typeFingerprint == 0xFF) {

		hwInfo.typeFingerprint = 0;
	}

	// Typ ermitteln
	if (!ReadMem(MB_START_EEP + 0x0EDA, 1, &hwInfo.typeExternalReader, piError)) {
		
		return FALSE;
	}
	// Wenn ZK Option vorhanden
	if (hwInfo.componentRS485_1) {
		// aber der Externe Lesertyp undefiniert ist,
		// dann wird von TS-Lesern ausgegangen.
		if (hwInfo.typeExternalReader == 0xFF) {

			hwInfo.typeExternalReader = 0x01;
		}
	} else {

		// Keine Externen leser
		hwInfo.typeExternalReader = 0x00;
	}
	// BUGFIX
	// Eine Firmware die die Serien TS und Protect untersttzt setzt als untersttzte Serien
	// TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
	// so dass der Wert der Hardwareinfo eindeutig ist.
	// Werden beide Flags vorgefunden oder Protect alleine wird von TS alleine ausgegangen.
	if (hwInfo.typeExternalReader & 0x04) {

		hwInfo.typeExternalReader = 0x01;
	}


	return TRUE;
}

