#include "StdAfx.h"
#include "Channel4.h"
#include <stdio.h>
#include <math.h>
// ##################################################################
/**
 * @brief Initialisiert die Membervariablen und 
 *        benoetigte Umgebung.
 *
 * @param id Kanalnummer die fr Log usw. verwendet wird. Sie identifiziert 
 *           die Klasse.
 */
void CChannel4::InitClass(short channelID, short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);
#ifdef WIN32
/*	//#####################################################
	// SOCKETS
	// Benutzung der Socket - Funktionalitaet initialisieren
	WSADATA WindowsData;
	WORD    Version;

	Version = MAKEWORD(2, 2);
	WSAStartup(Version, &WindowsData);
	//#####################################################
*/
	// Status fuer ServiceStatus
	m_serviceState = 0;

	// Workerthread Ende - Signal
	m_endSignal = FALSE;

	// Sockets initialisieren
	m_listenSocket  = NULL;

	// Verbindungsthread
	m_communicationThread = NULL;

	// Callback fuer Nachricht auf Datensaetze
	m_CBNotifyRecord = NULL;

	// Anzahl der Suspendanforderungen halten
	m_suspendCounter = 0;

	// Hardwareinfo aufsetzen
	memset(&m_hwInfo, 0, sizeof(StructHardwareInfo));
	m_hwInfo.typeDevice           = IV_VALUE_TYPE_DEVICE_PZE;
	m_hwInfo.typeBoard            = IV_VALUE_TYPE_BOARD_2_1;
	m_hwInfo.componentRS485_0     = 0;
	m_hwInfo.typeTCPIP            = IV_VALUE_TYPE_TCPIP_XPORT;
	m_hwInfo.typeModemMC          = IV_VALUE_TYPE_MODEM_MC_NONE;
	m_hwInfo.componentRS485_1     = 0;
	m_hwInfo.typeTransponder      = IV_VALUE_TYPE_TRANSPONDER_TSR32;
	m_hwInfo.typeDisplay          = IV_VALUE_TYPE_DISPLAY_SMALL;
	m_hwInfo.typeKeypad           = IV_VALUE_TYPE_KEYPAD_9;
	m_hwInfo.typeMMC              = IV_VALUE_TYPE_MMC_NONE;
	m_hwInfo.typeGPS              = IV_VALUE_TYPE_GPS_NONE;
	m_hwInfo.componentUSB         = 0;
	m_hwInfo.typeRadio            = IV_VALUE_TYPE_RADIO_NONE;
	m_hwInfo.typeDCF77            = IV_VALUE_TYPE_DCF77_NONE;
	m_hwInfo.componentInAnalog    = IV_VALUE_COMPONENT_IN_ANALOG_NONE;
	m_hwInfo.componentInDigital   = IV_VALUE_COMPONENT_IN_DIGITAL_1;
	m_hwInfo.componentOutDigital  = IV_VALUE_COMPONENT_OUT_DIGITAL_1;
	m_hwInfo.typeFingerprint      = IV_VALUE_TYPE_FINGERPRINT_NONE;
	m_hwInfo.typeExternalReader   = IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED;
	m_hwInfo.optionIOModule       = 0;
	m_hwInfo.optionTBAdaption     = 0;

#endif
}

// ##################################################################
/**
 * @brief Destruktor bereinigt die verwendete Umgebung.
 */
CChannel4::~CChannel4()
{
	//#####################################################
	// SOCKETS

	// Socket - Funktionalitaet exit
//	WSACleanup();
	//#####################################################
}

#ifdef WIN32
// ##################################################################
/**
 * @brief Entgegennehmen der Verbindung und starten eines Verarbeitungsthreads.
 */
int CChannel4::Connect(SOCKET *listenSocket, short channelID, short deviceID, int timeout, int aliveTO, int infoFlags) 
{
	Trace("%s(%i):INFO(3): CChannel4::Connect [ 0x08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, listenSocket, channelID, deviceID, timeout, aliveTO);

	// Hauptthread starten
	if (m_communicationThread == NULL) {
	
		// Kanalobjekt nimmt Verbindung entgegen
		SetActiveState(is_accepting);
		Trace("%s(%i):INFO(1): CChannel4::Connect [ SetActiveState(%i) is_accepting ]\n", __FILE__, __LINE__, channelID);

		// Kanalnummer und DeviceID uebernehmen.
		SetID(channelID);
		SetDeviceID(deviceID);

		// Listensocket von dem die Verbindungsanforderung kommt wird 
		// ueber lokale Membervariable uebergeben.
		m_listenSocket = listenSocket;
		if (timeout < 3000) {

			Trace("%s(%i):WARNING(1): CChannel4::Connect(%i) [ Timeout von %i auf mindestens 3000ms gesetzt. ]\n", __FILE__, __LINE__, channelID, timeout);
			timeout = 3000;
		}
		SetTimeout(timeout);
		m_aliveTO = aliveTO;
		m_infoFlags = infoFlags;
		m_initSignal = -1; // Undefinierter Fehler
		m_communicationThread = ::CreateThread(NULL, 0, CommunicationThread, this, 0, &m_threadID);

		int i;
		// Hochlauf des Workers pruefen
		for (i=0; (m_communicationThread != NULL) && (m_initSignal == -1) && (i < ((timeout*6) / 100)); i++) {

			Sleep(100);
		}
		// Wurde ein Fehler ausgeloest.
		if (m_initSignal != IV_ERROR_SUCCESS) {

			// Grundlegender Initialiserungsfehler?
			if (m_initSignal == -1) {

			} else {

				// Fehler von Workerthread 1:1 weitergeben.
			}

			// Verbindung schliessen
			DirectClose();

			return 0;
		}
	}

	return (m_communicationThread != NULL);
}

// ##################################################################
/**
 * @brief Beendigungssignal im Workerthread setzen und auf dessen beendigung warten.
 */
void CChannel4::DirectClose()
{
	Trace("%s(%i):INFO(3): CChannel4::DirectClose\n", __FILE__, __LINE__);

	// Socket-Thread deaktivieren, wenn aktiv
	if (m_communicationThread != NULL) {

		// WorkerThread fuer Socket beenden.
		m_endSignal = TRUE;
		
		// Thread umschalten
		Sleep(10);
		
		// Thread Beendigung abwarten (CommunicationThread stellt m_endSignal auf FALSE)
		int i;
		for (i=0; i < (GetTimeout()*6 / 100) && m_endSignal; i++) {
				
			Sleep(100);
		}

		// Hartes Schliessen, wenn noch nicht selbst beendet.
		if (m_communicationThread) {

			::CloseHandle(m_communicationThread);
			m_communicationThread = NULL;
		}
	}
}

/*
int CChannel4::CallFunctionNotify(short module, short function, unsigned char *data, int &count)
{
	Trace("%s(%i):INFO(3): CChannel4::CallFunctionNotify [ %i, %i, 0x08X, %i ]\n", __FILE__, __LINE__, module, function, data, count);

	switch(module) {
	case IV_MODULE_APPLICATION:
		switch(function) {
		case IV_FUNCTION_NOTIFY: {

			switch(data[0]) {
			case IV_VALUE_NOTIFY_RECORD: {
				CNotifyRecord nr;

				if (m_CBNotifyRecord) {
				
					int pos = 0;

					// Meldungstypen uebernehmen
					nr.type = data[pos];
					pos += sizeof(unsigned char);

					// Meldungsprioritaet uebernehmen
					nr.priority = *(short*)&data[pos];
					pos += sizeof(unsigned short);

					// Zeitstempel der Meldungserzeugung uebernehmen
					pos += sprintf(nr.timestamp, "%s", (char*)&data[pos]);

					// Aktuellen Zeitstempel fuer Nachrichtenannahme halten
					nr.dt = time(NULL);

					// Meldung in Warteschlange stellen.
					m_CBNotifyRecord(nr);

					Trace("%s(%i):INFO(1): CChannel4::CallFunctionNotify [ %i, %i, %s ]\n", __FILE__, __LINE__, GetID(), nr.type, nr.timestamp);
				}
				break;}
			default:
				Trace("%s(%i):INFO(1): CChannel4::CallFunctionNotify [ Unkown type %i ]\n", __FILE__, __LINE__, data[0]);
				break;
			}


			break;}
		}
		break;
	}

	return IV_ERROR_SUCCESS;
}
*/
// ##################################################################
/**
 * @brief Workerthread fuer die Verarbeitung der Kommunikationsaufgaben.
 */
unsigned long WINAPI CChannel4::CommunicationThread(LPVOID lpParam)
{
	if (lpParam == NULL) {
	
		// Fehlerhafter Parameter
		return 0;
	}

	// Es wird davon ausgegangen, dass der Parameter ein CChannel4 - Objekt ist.
	CChannel4 &own = *(CChannel4*)lpParam;
	
	own.Trace("%s(%i):INFO(3): CChannel4::CommunicationThread [ 0x08X ]\n", __FILE__, __LINE__, lpParam);

	// Hauptschleife ausfuehren?
	int run = TRUE;
	own.m_threadCommand = idle;

	int res, failCount = 0, infoDevice = FALSE, resumeCount;
    SOCKADDR_IN sa;
	struct timeval seltime;
	fd_set readfds;
	int count;
	unsigned char data[IV_PACKET_LENGTH_DATA]; // 21 Byte Paketframe

    int len, err = IV_ERROR_SUCCESS;
	
	// Timeout fuer Abfrage auf Daten
	seltime.tv_sec  = 0;
	seltime.tv_usec = 10000;

	// Verbindungsgesuch annehmen.

	len = sizeof(sa);
	own.m_socket = accept(*own.m_listenSocket, (SOCKADDR *)&sa, &len);

	if(own.m_socket == INVALID_SOCKET) {

		own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ accept fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
		// Initialisierungsfehler setzen.
		own.m_initSignal = IV_ERROR_FAIL;
		// Fehlerfall abfangen
		goto fail;
	}

	// Verbindungsmeldung ausgeben
	own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ (%i) Connected to %s, port %d ]\n", __FILE__, __LINE__, own.GetID(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));

	// Mindesttimeout sicherstellen
	if (own.GetTimeout() < 3000) {

		own.SetTimeout(3000);
	}

	// Socket mit Basisklasse verbinden
//	own.AttachChannel(CChannelBase::Socket, (void*)own.m_communicationSocket, own.GetTimeout());
	own.m_type = CChannelBase::Socket;
		//m_socket = (SOCKET)value;
	own.m_to     = own.GetTimeout();

	// Grunddaten zur Verbindung abfragen
	for (failCount=0; failCount < 3; failCount++) {

		// Paketabarbeitung einleiten
		if ((res = own.Ping(254, 0, 0)) != IV_ERROR_SUCCESS) {

			if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2)) {
				
				// Wenn Daten gemeldet werden,
				// die Anzahl aber 0 ist, so wurde die Verbindung beendet.
				
				// Verbindung schliessen
				if (closesocket(own.m_socket) == SOCKET_ERROR) {
					
					own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
				} 
				
				own.m_socket = INVALID_SOCKET;
				own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect (%i) ]\n", __FILE__, __LINE__, own.GetID());
				
				// Hauptschleife verlassen
				run = FALSE;
				// Initialisierungsfehler setzen.
				own.m_initSignal = IV_ERROR_FAIL;
			}
		} else {

			// Verbindung OK...
			break;
		}
	}

	// Nur wenn Geraet antwortet, weitere Auswertungen.
	if (run && !own.m_endSignal) {

		// com.                  Praefix fuer Kommunikationsvariablen
		// active                Aktivieren der Aktiven Verbindung
		// notify                notify => Aktivieren der Datensatzmeldung
		// prio		             Prioritaet des Geraetes
		// host                  Hostname des Remoterechners
		// port                  Portnummer des Remoterechners
		// retry                 Anzahl Versuche fr Verbindungsaufbau
		// timeout               Auszeit zwischen den Versuchsbloecken
		// repeat                Auszeit zur wiederholten Datensatzmeldung, wenn diese nicht abgerufen wurden.
		// alive                 Pinganfrage wenn fuer diese Zeit keine Kommunikation zustande kam.
		// Alle Zeitangaben in Sekunden und 32bit

		char value[64];
		memset(value, 0, sizeof(value));
		sprintf(value, "%i", 3 + (int)ceil((double)own.m_aliveTO * 1.05)); // 5% fuer Synkronisation dazugeben
		// Aliveparameter berechnen und setzen.
		// Geraetetyp und Seriennummer als GeraeteID abfragen
		if ((res = own.SetSystemVariable(254, "COM.ALIVE", value)) != IV_ERROR_SUCCESS ||
			(res = own.GetTypeDevice(254, own.m_hwInfo.typeDevice)) != IV_ERROR_SUCCESS || 
			(res = own.GetSerial(254, own.m_serial)) != IV_ERROR_SUCCESS) {

			// Verbindung schliessen
			if (closesocket(own.m_socket) == SOCKET_ERROR) {
				
				own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
			} 
			
			own.m_socket = INVALID_SOCKET;
			own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect (%i) ]\n", __FILE__, __LINE__, own.GetID());
			
			// Hauptschleife verlassen
			run = FALSE;
			// Initialisierungsfehler setzen.
			own.m_initSignal = IV_ERROR_FAIL;
		} else {
		
			// Daten fuer das TCPIP-Modul ermitteln
			own.m_tcpipInfo.Reset();

			// Informationen zum TCP/IP abrufen
			// Erst nachtraeglich hinzugefuegt, deswegen separat ausfuehren.
			if ((res = own.GetInfo(254, "COM.TCPIP", "", (char*)own.m_tcpipInfo.data, own.m_tcpipInfo.length)) != IV_ERROR_SUCCESS) {
				
				// Nur der Fehler "Majorname unbekannt" ist hier gueltig. Dann wird der Befehl noch nicht unterstuetzt.
				if ((res & 0xFF) != 1) {
					
					// Wenn Daten gemeldet werden,
					// die Anzahl aber 0 ist, so wurde die Verbindung beendet.
					
					// Verbindung schliessen
					if (closesocket(own.m_socket) == SOCKET_ERROR) {
						
						own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
					} 
					
					own.m_socket = INVALID_SOCKET;
					own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect (%i) ]\n", __FILE__, __LINE__, own.GetID());
					
					// Hauptschleife verlassen
					run = FALSE;
					// Initialisierungsfehler setzen.
					own.m_initSignal = IV_ERROR_FAIL;
				}
			} else {

				// Werte aufschluesseln
				own.m_tcpipInfo.Parse();
			}

			// Gewuenschte Zusatzinfos abrufen
			if (own.m_infoFlags & 0x01) {
			}

		}

		// Ermittelbare Daten zur Aktiven Verbindung erfolgreich ermittelt.
		own.SetActiveState(is_usable);
		own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ SetActiveState(%i) is_usable ]\n", __FILE__, __LINE__, own.GetID());
		// Initialisierung erfolgreich.
		own.m_initSignal = IV_ERROR_SUCCESS;
	}

	// Hauptschleife des Worker
	while (run) {

		// Beendigungssignal abfragen
		if (own.m_endSignal) {

			// Verbindung herunterfahren
			if (own.m_socket != INVALID_SOCKET) {

				// Senden am Socket verhindern
				if (shutdown(own.m_socket, 0x01/*SD_SEND*/) == SOCKET_ERROR) {

					own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ shutdown fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
				} else {
			
					// Alle anstehenden Daten lesen
					do {
						FD_ZERO((fd_set FAR *) &readfds);
						FD_SET(own.m_socket, (fd_set FAR *) &readfds);

						if ((res = select(FD_SETSIZE,
							(fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
							(const struct timeval FAR *)&seltime)) == SOCKET_ERROR) {

							own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ select fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
						} else if (res) {

							// Anstehende Daten lesen
							if ((res = recv(own.m_socket, (char*)own.m_receive, sizeof(own.m_receive), 0 )) == SOCKET_ERROR) {

								own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ recv fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());							
							} else if (res == 0) {
								// Disconnect von Gegenstelle
							}
						}
					} while((res > 0) && (res != SOCKET_ERROR));

					// Verbindung schliessen
					if (closesocket(own.m_socket) == SOCKET_ERROR) {

						own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
					}
				}
				own.m_socket = INVALID_SOCKET;
			}

			// Hauptschleife verlassen
			run = FALSE;
			continue;
		}


		switch (own.m_threadCommand) {
		case idle: {
			
			// Socketdaten initialisieren
			FD_ZERO((fd_set FAR *) &readfds);
			FD_SET(own.m_socket, (fd_set FAR *) &readfds);
			
			// Auf eintreffende Daten warten
			if ((res = select(FD_SETSIZE,
				(fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
				(const struct timeval FAR *)&seltime)) != SOCKET_ERROR) {
				
				// Stehen Daten an? (ansonsten Timeout)
				if (res) {
					
					// Paketabarbeitung einleiten
					if ((res = own.Execute((short)IV_CONTEXT_SLAVE, (short)IV_MODULE_BASE, (short)IV_FUNCTION_BASE, data, count, 1)) != IV_ERROR_SUCCESS) {
						
						if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2)) {
							
							// Wenn Daten gemeldet werden,
							// die Anzahl aber 0 ist, so wurde die Verbindung beendet.
							
							// Verbindung schliessen
							if (closesocket(own.m_socket) == SOCKET_ERROR) {
								
								own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
							} 
							
							own.m_socket = INVALID_SOCKET;
							own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect ]\n", __FILE__, __LINE__, own.GetID(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));
							
							// Hauptschleife verlassen
							run = FALSE;
							continue;
						}
						
						// Fehlerzaehler erhoehen
						failCount++;
					} else {
						
						// Fehlerzaehler zuruecksetzen
						failCount = 0;
					}
				}
			} else {
				
				own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ select fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
				
				// Verbindung schliessen
				if (closesocket(own.m_socket) == SOCKET_ERROR) {
					
					own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
				} 
				
				own.m_socket = INVALID_SOCKET;
				own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect ]\n", __FILE__, __LINE__, own.GetID(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));
				
				// Hauptschleife verlassen
				run = FALSE;
				continue;
			}

			// Aliveping senden?
			double tmRP = (double)own.m_clockReceivePacket / (double)CLOCKS_PER_SEC,
				now = (double)clock() / (double)CLOCKS_PER_SEC;
			if ((tmRP + own.m_aliveTO) < now) {

				// Paketabarbeitung einleiten
				if ((res = own.Ping(254, 0, 0)) != IV_ERROR_SUCCESS) {
					
					if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2)) {
						
						// Wenn Daten gemeldet werden,
						// die Anzahl aber 0 ist, so wurde die Verbindung beendet.
						
						// Verbindung schliessen
						if (closesocket(own.m_socket) == SOCKET_ERROR) {
							
							own.Trace("%s(%i):ERROR(1): CChannel4::CommunicationThread [ closesocket fail (%i) ]\n", __FILE__, __LINE__, WSAGetLastError());
						} 
						
						own.m_socket = INVALID_SOCKET;
						own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Disconnect (%i) ]\n", __FILE__, __LINE__, own.GetID());
						
						// Hauptschleife verlassen
						run = FALSE;
						continue;
					}
					
					// Fehlerzaehler erhoehen
					failCount++;
				} else {
					
					// Fehlerzaehler zuruecksetzen
					failCount = 0;
				}

				own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ Alive - Ping (%i) %is ]\n", __FILE__, __LINE__, own.GetID(), int(now - tmRP));

			}
			break;}

		case suspending:
			own.m_threadCommand = suspend;
			break;

		case suspend: 
			// Kurze Auszeit.
			Sleep(10);
			break;

		case resuming:
			resumeCount = 0;
			own.m_threadCommand = resume;
			break;

		case resume:
			
			resumeCount++;

			if (resumeCount >= 10) {

				// Sicherstellen, dass keine Anforderung auf Suspending besteht.
				if (own.m_suspendCounter > 0) {

					own.m_threadCommand = suspend;
				} else {
				
					own.m_threadCommand = idle;
				}
			}
			Sleep(30);
			break;

		}
	}

fail:
	// Eventuellen Eintrag aus Benachrichtungsschlange entfernen.
	if (own.m_CBNotifyRecord) {

		CNotifyRecord nr;
	
		// Meldungen dieses Kanals loeschen
		nr.erase = 1;

		// Aktuelle Kanalnummer eintragen
		nr.channelID = own.GetID();

		// Meldung aus Warteschlange entfernen.
		own.m_CBNotifyRecord(nr);
	}

	// Connect-Socket entfernen
	own.ReleaseChannel();

	// Thread wird beendet
	own.m_communicationThread = NULL;
	// Thread Beendigung ruecksetzen
	own.m_endSignal = FALSE;

	// Kanalobjekt wieder frei
	own.SetActiveState(is_free);
	own.Trace("%s(%i):INFO(1): CChannel4::CommunicationThread [ SetActiveState(%i) is_free ]\n", __FILE__, __LINE__, own.GetID());

	return 0;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4::Execute(short context, short module, short function, unsigned char *buf, int &count, int contextValue)
// Modulbefehl senden und Antwort zurueckgeben
//  deviceIDRemote -> Adresse (Geraetenummer) des Empfaengers
//  module   -> Nummer des anzusprechenden Modules
//  function -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> module-/Funktionsfehler
{
	Trace("%s(%i):INFO(3): CChannel4::Execute [ %i, %i, %i, 0x%08X, %i, %i ]\n", __FILE__, __LINE__, context, module, function, buf, count, contextValue);

	int i;
	
	// Laeuft mit Kommunikationsworkerthread?
	if (m_communicationThread && m_threadID != GetCurrentThreadId()) {


		// Verschachtelung zulassen.
		////////////////////////////
		// Achtung: Diese Routine kann durch Aufruf unter bestimmten Umstnden nochmals aufgerufen werden.
		// Z. B. Aktive Antwort oder ein Befehl des Gertes kommt dazwischen.
		// Es muss mit einem Levelcounter das frhzeitiges Freischalten des Communikationsworkers verhindert werden.
		if (m_suspendCounter == 0) {
		
			m_threadCommand = suspending;
			Trace("%s(%i):INFO(1): CChannel4::Execute [ suspending ]\n", __FILE__, __LINE__);

			for (i=0; i < ((GetTimeout() + 1000)/10) && m_threadCommand != suspend; i++) {
				Sleep(10);
			}

			if (i == ((GetTimeout() + 1000)/10)) {

				Trace("%s(%i):ERROR(1): CChannel4::Execute [ Befehl nicht ausgefhrt ]\n", __FILE__, __LINE__);
				m_threadCommand = idle;
				return -IV_ERROR_FAIL;
			}
		}
		m_suspendCounter++;

		i = CChannel4ModuleSystem::Execute(context, module, function, buf, count, contextValue);
		// Verschachtelung zuruecknehmen
		m_suspendCounter--;
		if (m_suspendCounter == 0) {
		
			m_threadCommand = resuming;
			Trace("%s(%i):INFO(1): CChannel4::Execute [ resuming ]\n", __FILE__, __LINE__);
		}

		return i;
	}

	return CChannel4ModuleSystem::Execute(context, module, function, buf, count, contextValue);
}

/*
int CChannel4::PingICMP(char *remote)
{
	SOCKET sockRaw = (SOCKET)NULL;
	SOCKADDR_IN dest, from;

	int bread,datasize;
	int fromlen = sizeof(from);
	int timeout = 3000;
	//char *icmp_data;
	//char *recvbuf;
	unsigned int addr=0;
	USHORT seq_no = 0;

	if ((sockRaw = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) == INVALID_SOCKET) {

		Trace("CChannel4::PingICMP: Raw-Socket konnte nicht erstellt werden.\n");
		closesocket(sockRaw);
		return FALSE;
	}

	if ((bread = setsockopt(sockRaw, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout))) == SOCKET_ERROR) {
			
		Trace("CChannel4::PingICMP: Timeout fuer Raw-Socket, konnte nicht gesetzt werden.\n");
		closesocket(sockRaw);
		return FALSE;
	}

	memset(&dest,0,sizeof(dest));
	dest.sin_family = AF_INET;
	// Adresse von Punktnotation her umwandeln
	if ((dest.sin_addr.s_addr = inet_addr(remote)) == INADDR_NONE) {

		HOSTENT *hp;
		// Adresse versuchen ueber namen aufzuloesen
		if ((hp = gethostbyname(remote)) != NULL) {

			memcpy(&(dest.sin_addr), hp->h_addr, hp->h_length);
			dest.sin_family = hp->h_addrtype;
		} else {

			Trace("CChannel4::PingICMP: Uebergebene Adresse konnte nicht aufgeloest werden.\n");
			closesocket(sockRaw);
			return FALSE;
		}
	}

	// Zu versendendes Datenpaket aufsetzen
	datasize = sizeof(IcmpHeader);
	datasize += DEF_PACKET_SIZE; // Anzahl Daten

	memset(icmp_data, 0, IV_PACKET_LENGTH_MAX);

	// Paket aufbereiten
	PreparePacket(icmp_data, datasize);
	
	int retry=0;
	int bwrote;
	while(retry++ < 4) {

		((IcmpHeader*)icmp_data)->i_cksum = 0;
		((IcmpHeader*)icmp_data)->timestamp = GetTickCount();

		((IcmpHeader*)icmp_data)->i_seq = seq_no++;
		((IcmpHeader*)icmp_data)->i_cksum = Checksum((USHORT*)icmp_data, datasize);

		if ((bwrote = sendto(sockRaw, icmp_data, datasize, 0, (struct sockaddr*)&dest, sizeof(dest))) == SOCKET_ERROR) {

			if (WSAGetLastError() == WSAETIMEDOUT) {

				Trace("CChannel4::PingICMP: timed out\n");
				return FALSE;
			}

			Trace("CChannel4::PingICMP: Paket konnte nicht versendet werden.\n");
			return FALSE;
		} else if (bwrote < datasize ) {

			Trace("CChannel4::PingICMP: Es konnten nicht alles versendet werden (Wrote %d bytes)\n", bwrote);
		}

		if ((bread = recvfrom(sockRaw, recvbuf, IV_PACKET_LENGTH_MAX, 0, (struct sockaddr*)&from, &fromlen)) == SOCKET_ERROR) {

			if (WSAGetLastError() == WSAETIMEDOUT) {

				Trace("CChannel4::PingICMP: timed out\n");
				return FALSE;
			}

			Trace("CChannel4::PingICMP: Datenempfang gescheitert\n");
			return FALSE;
		}
	
		// Rueckgabepaket pruefen
		CheckPacket(recvbuf, bread, &from);

		Sleep(100);
	}

	// Socket schliessen.
	if (sockRaw != INVALID_SOCKET) {
		
		closesocket(sockRaw);
	}

	return TRUE;
}

int CChannel4::CmdXPort(char *remote, char *data, unsigned short &count, int needAnswer)
{
	unsigned char Buffer[128];
	
	char *server_name= remote;
	unsigned short port = 0x77FE;
	
	int retval, loopflag=0;
	int i, loopcount, maxloop = -1;
	unsigned int addr;
	struct sockaddr_in server;

	struct hostent *hp;
	SOCKET  conn_socket;

	//
	// Attempt to detect if we should call gethostbyname() or
	// gethostbyaddr()

	if (isalpha(server_name[0]))
	{   // server address is a name
		hp = gethostbyname(server_name);
	}
	else
	{ // Convert nnn.nnn address to a usable one
		addr = inet_addr(server_name);
		//hp = gethostbyaddr((char *)&addr,4,AF_INET);
		hp = gethostbyname(server_name);
	}
	
	if (hp == NULL )
	{
		Trace("Client: Cannot resolve address [%s]: Error %d\n",
			server_name,WSAGetLastError());

		return FALSE;
	}

	//
	// Copy the resolved information into the sockaddr_in structure
	//
	memset(&server,0,sizeof(server));
	memcpy(&(server.sin_addr),hp->h_addr,hp->h_length);
	server.sin_family = hp->h_addrtype;
	server.sin_port = htons(port);

	conn_socket = socket(AF_INET, SOCK_DGRAM , 0); // Open a socket
	if (conn_socket <0 )
	{
		Trace("Client: Error Opening socket: Error %d\n",
		        WSAGetLastError());

		return FALSE;
	}

	//
	// Notice that nothing in this code is specific to whether we
	// are using UDP or TCP.
	// We achieve this by using a simple trick.
	//    When connect() is called on a datagram socket, it does not
	//    actually establish the connection as a stream (TCP) socket
	//    would. Instead, TCP/IP establishes the remote half of the
	//    ( LocalIPAddress, LocalPort, RemoteIP, RemotePort) mapping.
	//    This enables us to use send() and recv() on datagram sockets,
	//    instead of recvfrom() and sendto()


	Trace("Client connecting to: %s\n",hp->h_name);
	if (connect(conn_socket,(struct sockaddr*)&server,sizeof(server))
	        == SOCKET_ERROR)
	{
		Trace("connect() failed: %d\n",WSAGetLastError());

		return FALSE;
	}

	// Befehl uebernehmen
	memcpy(Buffer, data, count);

	// set up a string to send
	//
	loopcount =0;
//	while(1)
	{
		retval = send(conn_socket, (char*)Buffer, 4, 0);

		if (retval == SOCKET_ERROR)
		{
			Trace("send() failed: error %d\n",WSAGetLastError());
			
			goto label_clean;
		}
		Trace("Sent Data ...\n");

		// Wird eine Antwort auf das Paket erwartet?
		if (needAnswer) {

			retval = recv(conn_socket, (char*)Buffer, sizeof(Buffer), 0);

			if (retval == SOCKET_ERROR)
			{
				Trace("recv() failed: error %d\n",WSAGetLastError());

				goto label_clean;
			}
			//
			// We are not likely to see this with UDP, since there is no
			// 'connection' established.
			//
			if (retval == 0)
			{
				Trace("Server closed connection\n");

				goto label_clean;
			}

			// Anzahl uebernehmen
			count = retval;

			Trace("Receive (%i): ", retval);
			for (i=0; i < retval; i++) {

				data[i] = (unsigned char)Buffer[i];
				Trace("[%02X]", (unsigned char)Buffer[i]);
			}
			Trace("\n");
		}
//
//		if (!loopflag)
//		{
//			Trace("Terminating connection\n");
//
//			goto label_clean;
//
//		} else {
//			if ( (loopcount >= maxloop) && (maxloop >0) )
//				goto label_clean;
//		}

	}

label_clean:
	closesocket(conn_socket);

	return TRUE;
}

int CChannel4::BroadcastXPort(char *address_net)
{
	int res = 0;

	char address_broadcast[20];
	if ((strlen(address_net) > sizeof(address_broadcast)) ||
		(address_net[0] < 0x30 || address_net[0] > 0x39) ) {

		// Addresse kann nicht entgegengenommen werden.
		return res;
	}
	// Adresse zur modifikation uebernehmen
	strcpy(address_broadcast, address_net);

	// Broadcastadresse aufbauen
	int i = strlen(address_broadcast);
	while (i && address_broadcast[i] != '.') i--;
	sprintf(&address_broadcast[i + 1], "255");

	unsigned int address = inet_addr(address_broadcast);
	struct hostent *hp;
	//hp = gethostbyaddr((char *)&address,4,AF_INET);
	if ((hp = gethostbyname(address_broadcast)) == NULL) {

		Trace("CChannel4::BroadcastXPort: address_broadcast [%s] konnte nicht aufgeloest werden.\n", address_broadcast);
		return res;
	}

	// Konfigurationsport
	unsigned short port = 0x77FE;
	// Datenpuffer
	unsigned char data[0xFF];
	unsigned char size = 0;

	// Paket aufsetzen
	data[size++] = 0x00;
	data[size++] = 0x01; // Kein Reboot
	data[size++] = 0x00;
	data[size++] = 0xF6; // Anfrage auf Firmwareversion

	int retval;
	SOCKET  socket_broadcast;

	//
	// Copy the resolved information into the sockaddr_in structure
	//
	struct sockaddr_in server;
	memset(&server, 0, sizeof(server));
	memcpy(&(server.sin_addr),hp->h_addr,hp->h_length);
	server.sin_family = hp->h_addrtype;
	server.sin_port = htons(port);

	// Zurueckgelieferte Anzahl
	struct sockaddr_in remote;
	int len = sizeof(remote);

	// Socket oeffnen
	if ((socket_broadcast = socket(AF_INET, SOCK_DGRAM , 0)) == ~0) {

		Trace("CChannel4::BroadcastXPort: Socket konnte nicht erstellt werden.\n");
		return res;
	}

	// Nonblocking einschalten
	unsigned long value = 1;
	if (ioctlsocket(socket_broadcast, FIONBIO, &value) == SOCKET_ERROR) {

		Trace("CChannel4::BroadcastXPort: Nonblocking konnte nicht aktiviert werden.\n");
		return res;
	}

	// Senden des Broadcast
	if (sendto(socket_broadcast, (char*)data, size, 0, (struct sockaddr*)&server, sizeof(server)) == SOCKET_ERROR) {

		Trace("CChannel4::BroadcastXPort: send() failed\n");
		goto label_clean;
	}

	Sleep(500);
	// set up a string to send
	//
	for (;;) {

		if ((retval = recvfrom(socket_broadcast, (char*)data, sizeof(data), 0, (struct sockaddr*)&remote, &len)) == SOCKET_ERROR) {

			Trace("CChannel4::BroadcastXPort: recv() failed(%d)\n", WSAGetLastError());
			goto label_clean;
		}

		//
		// We are not likely to see this with UDP, since there is no
		// 'connection' established.
		//
		if (retval == 0) {

			Trace("CChannel4::BroadcastXPort: Server closed connection\n");
			goto label_clean;
		}

		// Increment fuer Antwortzaehler
		res++;

		// Empfang ausgeben
		Trace("Receive (%i) from [%s:%i]: ", retval, inet_ntoa(remote.sin_addr), remote.sin_port);
		for (i=0; i < retval; i++) {

			Trace("[%02X]", (unsigned char)data[i]);
		}
		Trace("\n");


		Trace("Antwort von [%s:%i][%02X-%02X-%02X-%02X-%02X-%02X]\n", 
			inet_ntoa(remote.sin_addr), remote.sin_port,
			data[24], data[25], data[26], data[27], data[28], data[29]);

	}

label_clean:
	closesocket(socket_broadcast);

	return res;
}

// 16bit Pruefsumme erstellen
unsigned short CChannel4::Checksum(unsigned short *buffer, int size)
{

	unsigned long cksum=0;

	while(size > 1) {

		cksum += *buffer++;
		size -= sizeof(USHORT);
	}

	if(size) {

		cksum += *(UCHAR*)buffer;
	}

	cksum  = (cksum >> 16) + (cksum & 0xffff);
	cksum += (cksum >> 16);
	return (USHORT)(~cksum);
}

// Aubereiten eines ICPM request Packets
void CChannel4::PreparePacket(char * icmp_data, int datasize)
{
	IcmpHeader *icmp_hdr;
	char *datapart;

	icmp_hdr = (IcmpHeader*)icmp_data;

	icmp_hdr->i_type = ICMP_ECHO;
	icmp_hdr->i_code = 0;
	icmp_hdr->i_id = (USHORT)GetCurrentProcessId();
	icmp_hdr->i_cksum = 0;
	icmp_hdr->i_seq = 0;

	datapart = icmp_data + sizeof(IcmpHeader);
	//
	// Paketdaten aubereiten
	//
	memset(datapart,'E', datasize - sizeof(IcmpHeader));

}

// Das Antwortpaket ist ein IP - Paket.
// Der IP - Header muss zur auffindung der ICMP - Daten herangezogen werden.
void CChannel4::CheckPacket(char *buf, int bytes, struct sockaddr_in *from)
{
	IpHeader *iphdr;
	IcmpHeader *icmphdr;
	unsigned short iphdrlen;

	iphdr = (IpHeader *)buf;

	iphdrlen = iphdr->h_len * 4 ; // number of 32-bit words *4 = bytes

	if (bytes  < iphdrlen + ICMP_MIN) {

		Trace("CChannel4::CheckPacket: Too few bytes from %s\n", inet_ntoa(from->sin_addr));
	}

	icmphdr = (IcmpHeader*)(buf + iphdrlen);

	if (icmphdr->i_type != ICMP_ECHOREPLY) {

		Trace("CChannel4::CheckPacket: non-echo type %d recvd\n",icmphdr->i_type);
		return;
	}
	if (icmphdr->i_id != (USHORT)GetCurrentProcessId()) {
		
		Trace("CChannel4::CheckPacket: someone else's packet!\n");
		return ;
	}

	Trace("CChannel4::CheckPacket: %d bytes from %s:", bytes, inet_ntoa(from->sin_addr));
	Trace("CChannel4::CheckPacket: icmp_seq = %d. ", icmphdr->i_seq);
	Trace("CChannel4::CheckPacket: time: %d ms ", GetTickCount()-icmphdr->timestamp);
	Trace("\n");
}
*/
#endif

