#ifndef _CHANNEL4_MODULE_SYSTEM_H_
#define _CHANNEL4_MODULE_SYSTEM_H_

// Basisklasse einbinden
#include "Channel4ModuleHardware.h"

/** In diesem Module sind alle Systemspezifischen Funktionen enthalten.
 *
 */
#define IV_MODULE_SYSTEM                         0x0001


// Ermitteln der Firmwareversion
#define IV_FUNCTION_GET_VERSION                  0x0000

// Setzen/Ermitteln der Seriennummer
#define IV_FUNCTION_SET_SERIAL                   0x0001
#define IV_FUNCTION_GET_SERIAL                   0x0002
	
// Setzen/Ermitteln der Systemzeit. Format (YYYY-MM-DD HH:MM:SS\0)
#define IV_FUNCTION_SET_RTC                      0x0003
#define IV_FUNCTION_GET_RTC                      0x0004


// Setzen/Ermitteln einer Systemvariablen 
#define IV_FUNCTION_SET_SYSTEM_VARIABLE          0x0005
#define IV_FUNCTION_GET_SYSTEM_VARIABLE          0x0006

// Der Name einer Systemvariablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Fr GPRS ist der Majorname MODEM_MC35i.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.


// Setzen/Ermitteln eines Relaiszustandes
#define IV_FUNCTION_SET_RELAY                    0x0007
#define IV_FUNCTION_GET_RELAY                    0x0008

// Die Namen sind: INTERN, TMR (Trmodule), IOM (IOModule)
// Die Busnummer ist die Busnummer bei 0 beginnend. (Bei INTERN eigentlich immer 0)
// Die Relaisnummer ist die Relaisnummer bei 0 beginnend.
// Die Zeit ist eine Angabe in Sekunden (0 bedeutet eine Dauerschaltung in den gewhlten Status)
// Fehlernummer:
// 1 = Name undefiniert
// 2 = Busnummer undefiniert
// 3 = Relaisnummer undefiniert
// 4 = Status undefiniert  (0, 1) (Knnte irgendwann um andere Varianten erweitert werden)

// Schreiben/Lesen von: Externen Flash
#define IV_FUNCTION_SET_FLASH                    0x0009
#define IV_FUNCTION_GET_FLASH                    0x000A

// Schreiben/Lesen von: EEPROM
#define IV_FUNCTION_SET_EEPROM                   0x000B
#define IV_FUNCTION_GET_EEPROM                   0x000C

// Schreiben/Lesen von: Program-Flash schreiben/lesen
#define IV_FUNCTION_SET_PROGRAM                  0x000D // (wird nur aufgenommen)
#define IV_FUNCTION_GET_PROGRAM                  0x000E

// Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
#define IV_FUNCTION_BEGIN_SET_SEGMENT                0x000F
#define IV_FUNCTION_BEGIN_SET_SEGMENT_OK             0x0010
#define IV_FUNCTION_END_SET_SEGMENT                  0x0011
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_SEGMENT und _END_SEGMENT
#define IV_VALUE_SEGMENT_UPDATE         0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define IV_VALUE_SEGMENT_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_FIRMWARE       2
#define IV_VALUE_SEGMENT_FONTS          3
#define IV_VALUE_SEGMENT_GRAFICS        4
#define IV_VALUE_SEGMENT_SETUP          5
#define IV_VALUE_SEGMENT_LIST           6
#define IV_VALUE_SEGMENT_SETUP_EXTENDED 7
#define IV_VALUE_SEGMENT_LIST_ACCESS    8 // Listen fuer ZK II werden uebertragen, dadurch wird die Zutrittskontrolle blockiert.
#define IV_VALUE_SEGMENT_LIST_TIMEBOY   9 // Liste fuer Timeboy uebertragen
#define IV_VALUE_SEGMENT_FINGERPRINT    10
#define IV_VALUE_SEGMENT_TEXT           11
// 0  -> Global fuer Uebernahmerahmen von Datenbundel
// 1  -> Firmware
// 2  -> Fonts
// 3  -> Grafik
// 4  -> Setup
// 9  -> Timeboylisten uebertragen
// 10 -> Firmware des Fingerprintmoduls uebertragen
// 11 -> Segment fuer Textdaten der Firmware uebertragen

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfgung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_SET_SEGMENT_OK               0x0012
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

// Ermitteln von: Bootloaderversion
#define IV_FUNCTION_GET_VERSION_BOOTLOADER       0x0013
// Format: XX.XX.XX??????????????

// Durchfuehren eines Resets der Speicherbereiche
#define IV_FUNCTION_SYSTEM_RESET                 0x0014

// Resetmodus:
#define IV_VALUE_RESET_ALL             0 // Geraet in einen Zustand versetzen, wie nach dem Bootloader einspielen.
#define IV_VALUE_RESET_RESTART         1 // Warmstart (Watchdog ausfhren)
#define IV_VALUE_RESET_SETUP           2 // Setup, Listen und Datensatzzeiger Lschen/Zurcksetzen

// Fehler:
// 1 -> Resetmodus unbekannt.


// Ins Bios wechseln
#define IV_FUNCTION_ENTER_BIOS			         0x0015
// Bioswechsel erfolgt
#define IV_FUNCTION_ENTERED_BIOS		         0x0016

// Bios verlassen (aenderungen uebernehmen
#define IV_FUNCTION_EXIT_BIOS			         0x0017
// Biosverlassen erfolgt
#define IV_FUNCTION_EXITED_BIOS			         0x0018

// Setzt/liest das zu verwendende Interface
#define IV_FUNCTION_SET_INTERFACE                0x0019
// 0 -> RS232
// 1 -> TCP/IP
// 2 -> GSM
// 3 -> GPRS
// 4 -> RS485
#define IV_FUNCTION_GET_INTERFACE                0x001A

#define IV_VALUE_INTERFACE_RS232 0
#define IV_VALUE_INTERFACE_TCPIP 1
#define IV_VALUE_INTERFACE_GSM   2
#define IV_VALUE_INTERFACE_GPRS  3
#define IV_VALUE_INTERFACE_RS485 4


// Setzen/Ermitteln der zu verwendende BaudRate
#define IV_FUNCTION_SET_BAUD                     0x001B
#define IV_FUNCTION_GET_BAUD                     0x001C
// Setzen/Ermitteln der zu verwendenden DeviceID
#define IV_FUNCTION_SET_DEVICE_ID                0x001D
#define IV_FUNCTION_GET_DEVICE_ID                0x001E

// Befehl der beim Geraetestart den Bootloader des MasterIV wiederherstellt.
#define IV_FUNCTION_ENTER_BOOTLOADER             0x001F

// In Monitor Mode von XPort wechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0028 
#define IV_FUNCTION_ENTER_XPORT_MONITOR          0x0020
// Antwort in Monitor Mode gewechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x0029
#define IV_FUNCTION_ENTER_XPORT_MONITOR_OK       0x0021

// Monitorbefehl an XPort und Antwort erhalten.
#define IV_FUNCTION_CMD_XPORT                    0x0022

// In Monitor Mode von XPort wechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR           0x0023
// Antwort in Monitor Mode gewechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR_OK        0x0024


#define IV_FUNCTION_BEGIN_GET_SEGMENT            0x0025
// [in]-> [UINT16(SS_...)]
// <- 16bit 
#define IV_FUNCTION_BEGIN_GET_SEGMENT_OK         0x0026
// [out]-> 16bit Typ des Speichers
// [out]-> 32bit Adresse fuer Lesen
// [out]-> 32bit Groesse des Blocks
// [out]-> 32bit CRC des Blocks

#define IV_FUNCTION_END_GET_SEGMENT              0x0027
// -> [UINT16(SS_...)]
// <- [UINT16(SS_...)]
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// MatchPort in den Monitor Mode (z-Mode mit aktivem wLAN Interface) wechseln
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0039
// Antwort auf den Wechsel-Befehl BetriebsMode->MonitorMode
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x003A


// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_GET_SEGMENT und _END_GET_SEGMENT
#define IV_VALUE_SEGMENT_GET_UPDATE         0 // Es werden Daten gelesen ohne Bedienstop.
#define IV_VALUE_SEGMENT_GET_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_GET_SETUP          5
#define IV_VALUE_SEGMENT_GET_SETUP_EXTENDED 7

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfgung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_GET_SEGMENT_OK           0x0028
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

#define IV_FUNCTION_SYSTEM_RESET_OK              0x0029

// Transparentmodus aktivieren/deaktivieren.
#define IV_FUNCTION_OPEN_COM_SERVER_MODE         0x002A
#define IV_FUNCTION_CLOSE_COM_SERVER_MODE        0x002B

// Abfrage von Information zu einem Modultyp
#define IV_FUNCTION_GET_INFO                     0x002C
// param1: ACCESS.MODULE
// param2: bei ACCESS.MODULE UCHAR(BusNr) UCHAR(0=TM,1=L)

// Antwort in Com-Server-Mode gewechselt
#define IV_FUNCTION_OPEN_COM_SERVER_MODE_OK      0x002D

// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
#define IV_FUNCTION_SET_NOTIFY                   0x002E
// param1: Typ der Benachrichtigung
// param2: Zu setztender Zustand der Benachrichtigung
#define IV_FUNCTION_GET_NOTIFY                   0x002F
// param1: Typ der Benachrichtigung
// Werte siehe IV_MODULE_APPLICATION.IV_FUNCTION_NOTIFY

#define IV_FUNCTION_GET_SOFTWARE_INFO            0x0030

// Lesen einer Seite des Flash (Bereich: 0 - 4095, Seite hat 528Byte)
#define IV_FUNCTION_GET_FLASH_PAGE               0x0031

// Setzen/Lesen der Systemflags von EEPROM-Adresse: 0x0ECC
#define IV_FUNCTION_SET_SYSTEM_FLAGS             0x0032
#define IV_FUNCTION_GET_SYSTEM_FLAGS             0x0033

// Schreiben/Lesen von: RAM
#define IV_FUNCTION_SET_RAM                      0x0034
#define IV_FUNCTION_GET_RAM                      0x0035

// Schreiben/Lesen von MMC
#define IV_FUNCTION_SET_MMC                      0x0036
#define IV_FUNCTION_GET_MMC                      0x0037
#define IV_FUNCTION_GET_SFSMPB                   0x0038


#pragma  pack(1)
// Strucktur fuer dynamische Verwaltung des SFS (MS) 06.11.08
typedef struct struct_memory_partition_block {

	unsigned long  StorageStartAddress; ///< Startadresse des Filessystems im Speicher
	unsigned long  StartFATAddress;     ///< Startadresse FAT
	unsigned long  DirStartAddress;     ///< Startadresse File Directory
	unsigned long  FileStartAddress;    ///< Startadresse Datenbereich
	unsigned long  FirstRecordAddress;  ///< Startadresse des Ringspeichers
	unsigned long  StorageEndAddress;   ///< Letzte Adresse des Speicherbereiches

	unsigned short  DirStartCluster;     ///< Start Cluster in dem das Dateiverzeichnis liegt
	unsigned short  FileStartCluster;    ///< Start Cluster fr die Daten der Dateien
	unsigned short  FATClusterCount;     ///< Anzahl der Cluster die von der FAT belegt werden
	unsigned short  DirClusterCount;     ///< Anzahl der Cluster die vom Verzeichnis belegt werden.
	unsigned short  FileClusterCount;    ///< Anzahl der Cluster fr den Dateibereich
	unsigned short  TotalClusterCount;   ///< Anzahl aller Cluster des Dateisystems
	unsigned short  SectorsPerCluster;   ///< Anzahl der Sektoren pro Cluster

	unsigned long  ClusterSize;         ///< Gre eines Clusters in Byte
	unsigned long  RecordVolumeSize;      ///< Gre des Ringspeichers in Byte
	unsigned long  ListVolumeSize;      ///< Gre des Dateisystem in Byte
	unsigned long  StorageVolumeSize;   ///< Gre der Memory-Card in Byte

	unsigned char ClusterShiftBit;     ///< Schiebebit fr Clusteradressierung

} SFSMPB;
#pragma pack()

// System Device Log
#define IV_FUNCTION_GET_SYS_RECORD				 0x003B
#define IV_FUNCTION_SET_SYS_RECORD				 0x003C
#define IV_FUNCTION_RESTORE_ALLSYS_RECORDS		 0x003D

#define IV_ANSW_NO_SYS_RECORD               0xC0
#define IV_ANSW_ONE_SYS_RECORD				0xC1
#define IV_ANSW_ACK_ON_SYS_RECORD			0xC8
#define IV_SYS_RECORDS_MAX					0x200

// Passwort fuer Kommunikation
#define IV_FUNCTION_PASSWORD_CHECK              0x003E
#define IV_FUNCTION_PASSWORD_KEY                0x003F

#define IV_FUNCTION_PRESS_VIRTUAL_KEY           0x0040

class CChannel4ModuleSystem : public CChannel4ModuleHardware
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleSystem()  : CChannel4ModuleHardware(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID) 
	{ InitClass(); }
	CChannel4ModuleSystem(short channelID, short deviceID) : CChannel4ModuleHardware(channelID, deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleSystem();

public:
	unsigned long m_serial; // Seriennummer des Geraetes

	inline void SetPasswordCommunication(char *password)
	{
		memset(m_passwordCommunication, 0, sizeof(m_passwordCommunication));
		strncpy(m_passwordCommunication, password, sizeof(m_passwordCommunication) - 1);
	}
	inline const char *GetPasswordCommunication()
	{
		return m_passwordCommunication;
	}
	inline void SetFlagPasswordProtected(int bState)
	{
		m_isPasswordProtected = (bState == 0) ? 0 : 1;
	}
	// Ist Passwortschutz aktiv?
	inline int IsPasswordProtected()
	{
		return m_isPasswordProtected;
	}
	inline void SetFlagPasswordProtection(int bState)
	{
		m_isPasswordProtection = (bState == 0) ? 0 : 1;
	}
	// Wird Passwortschutz untersttzt?
	inline int IsPasswordProtection()
	{
		return m_isPasswordProtection;
	}

private:
	char m_passwordCommunication[17]; // Passwort fuer die Kommunikation
	int m_isPasswordProtected; // Hlt die Information, dass das Gert Passwortgeschtzt ist.
	int m_isPasswordProtection; // Hlt die Information, dass das Gert den Passwortschutz untersttzt.

public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short function, short error, char *text);

	// MODULE_HARDWARE
	// BUG:
	// Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
	// - TypeTransponder den Wert fuer das Display
	// - TypeDisplay den Wert fuer das Keypad
	// - TypeKeypad den Wert fuer den Transponder
	int SetTypeDisplay(short deviceID, unsigned char value, unsigned short value2 = 0xFFFF, int checked = FALSE);
	int GetTypeDisplay(short deviceID, unsigned char &value, int checked = FALSE);
	int SetTypeKeypad(short deviceID, unsigned char value, unsigned short value2 = 0xFFFF, int checked = FALSE);
	int GetTypeKeypad(short deviceID, unsigned char &value, int checked = FALSE);
	int SetTypeTransponder(short deviceID, unsigned char value, unsigned short value2 = 0xFFFF, int checked = FALSE);
	int GetTypeTransponder(short deviceID, unsigned char &value, int checked = FALSE);

	// Gesamte Hardwareinfo lesen
	int GetHardwareInfo(short deviceID, TDFHardwareInfo &hwInfo);
	// Gesamte Hardwareinfo in String umsetzen.
	int GetInfos(TDFHardwareInfo hwInfo, char *info);

	
	// MODULE_SYSTEM

	// Systemreset; Speicherbereiche loeschen
	int SystemReset(short deviceID, unsigned short modus);
	int SystemResetOK(short deviceID);

	// Firmwareversion lesen
	int GetVersion(short deviceID, char *version);
	int GetVersionBootloader(short deviceID, char *version);

	// Seriennummer lesen
	int SetSerial(short deviceID, unsigned long serial, unsigned long value2 = 0xFFFFFFFF);
	int GetSerial(short deviceID, unsigned long &serial);

	// System Device Log
	int SystemReadRecord(short deviceID, unsigned char *buf, int &count, unsigned char &status);
	int SystemQuitRecord(short deviceID, unsigned char &status);
	int SystemRestoreRecords(short deviceID);

	// Datum/Uhrzeit setzen/lesen
	int SetRTC(short deviceID, char *time);
	int GetRTC(short deviceID, char *time);

	// Helper fr Systemvariablen von Mobile
	void MobileMaskedSysvar(char *buf);
	void MobileUnmaskedSysvar(char *buf);
	unsigned char HexToUINT8(char *data);
	// Systemvariable setzen/lesen
	int SetSystemVariable(short deviceID, char *name, char *value);
	int GetSystemVariable(short deviceID, char *name, char *value);

	// Relais setzen/lesen
	int SetRelay(short deviceID, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time);
	int GetRelay(short deviceID, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time);

protected:
	unsigned long m_crcSetSegment, m_crcGetSegment;
public:
	// Datensegmentuebertragung beginnen/enden
	int BeginSetSegment(short deviceID, unsigned short art, unsigned long &address, unsigned long size, unsigned long crc);
	int BeginSetSegmentOK(short deviceID);
	int EndSetSegment(short deviceID, unsigned short art, unsigned long crc);
	int EndSetSegmentOK(short deviceID);

	// Datensegmentuebertragung beginnen/enden
	int BeginGetSegment(short deviceID, unsigned short art, unsigned long &address, unsigned long &size, unsigned long &crc);
	int BeginGetSegmentOK(short deviceID);
	int EndGetSegment(short deviceID, unsigned short art);
	int EndGetSegmentOK(short deviceID);

	// Flash setzen/lesen (die ersten 16kByte Bootloader koennen nur gelesen werden)
	int SetFlash(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetFlash(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetFlashPage(short deviceID, unsigned short pageNum, unsigned char *buf, int &count);

	// EEPROM setzen/lesen
	int SetEEPROM(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetEEPROM(short deviceID, unsigned long address, unsigned char *buf, int count);

	// PROGRAM setzen/(lesen unterbunden)
	int SetProgram(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetProgram(short deviceID, unsigned long address, unsigned char *buf, int count);

	// RAM setzen/lesen
	int SetRAM(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetRAM(short deviceID, unsigned long address, unsigned char *buf, int count);

	// MMC schreiben/lesen
	int SetMMC(short deviceID, unsigned long address, unsigned char *buf, int count);
	int GetMMC(short deviceID, unsigned long address, unsigned char *buf, int count);

	// Info zum Dateisystem
	int GetSFSMPB(short deviceID, unsigned char &type, SFSMPB &sfs);

	// Systembios aufrufen/verlassen
	int EnterBios(short deviceID);
	int EnteredBios(short deviceID);
	int ExitBios(short deviceID);
	int ExitedBios(short deviceID);

	// Interface setzen/lesen (RS232, TCPIP...)
	int SetInterface(short deviceID, unsigned char type);
	int GetInterface(short deviceID, unsigned char &type);
	
	// Baud-Rate setzen/lesen
	int SetBaud(short deviceID, unsigned long br);
	int GetBaud(short deviceID, unsigned long &br);

	// DeviceID setzen/lesen
	int SetDeviceIDBios(short deviceID, unsigned short id);
	int GetDeviceIDBios(short deviceID, unsigned short &id);

	// Bootloader aktivieren
	int EnterBootloader(short deviceID);

	// Monitor Mode vom XPort
	int EnterXPortMonitor(short deviceID, char *data);
	int EnterXPortMonitorExt(short deviceIDRemote, char *data);
	int EnterXPortMonitorOK(short deviceID);
	int EnterXPortMonitorExtOK(short deviceID);
	int CommandXPort(short deviceID, char *buf, int &count);
	int ExitXPortMonitor(short deviceID);
	int ExitXPortMonitorOK(short deviceID);

	// MatchPort Set Factory default
	int MatchPortSetFactoryDefault(short deviceID, char *buf, int &count);

	// Transparentsmodus fuer COM
	int OpenComServerMode(short deviceID, char *comm, unsigned long baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose);
	int OpenComServerModeOK(short deviceID);
	int CloseComServerMode(short deviceID);

	// Informationen zu einem Modultyp abfragen
	int GetInfo(short deviceID, char *name, char *param, char *info, int &infoLength);

	// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
	int SetNotify(short deviceID, unsigned char type, unsigned char state);
	int GetNotify(short deviceID, unsigned char type, unsigned char &state);

	// Softwareinformation der Anwendungsfirmware abfragen
	int GetSoftwareInfo(short deviceID, unsigned char *swi, int &infoLength);

	// Setzen/Lesen der Systemflags
	int SetSystemFlags(short deviceID, unsigned long flags);
	int GetSystemFlags(short deviceID, unsigned long &flags);
	
	// Passwort pruefen
	int PasswordCheck(short deviceID, char *password);
	// Passwortschlssel abrufen
	int PasswordKey(short deviceID, char *key);

	// Einen Tastencode ausfhren. F1-F15 drcken.
	int PressVirtualKey(short deviceID, unsigned char vk, unsigned char flags);

public:
	// Ausfuehren eines Protokollbefehls
	virtual int Execute(short context, short module, short function, unsigned char *buf, int &count, int contextValue = -1);

	//#################################
	// Call - Routinen

	// Ausfuehren von Befehlen
	virtual int Call(short module, short function, unsigned char *data, int &count);

};

#endif
