/**
 * @file  Channel4ModuleTable.cpp
 * @brief Implementierung der Klasse CChannel4ModuleTable.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Channel4ModuleTable.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleTable::InitClass(short channelID, short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleTable::~CChannel4ModuleTable() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleTable::GetErrorModule(short module, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_TABLE ?
	if (module == IV_MODULE_TABLE) {

	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleApplication::GetErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleTable::GetErrorFunction(short module, short function, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, function, error, text);

	if (module == IV_MODULE_TABLE) {

		// Sonderfehler fr Passwortschutz abhandeln.
		if (error == 28) {
			// Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
			// Es ist in diesem Fall zuerst das richtige Passwort fr die Kommunikation erforderlich.
			sprintf(text, "( %i ): Korrektes Passwort fr die Kommunikation erforderlich.", error);
			return;
		} else if (error == 29) {
			// Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
			// Die Befehlsausfhrung ist danach fr 60 Sekunden blockiert.
			sprintf(text, "( %i ): Gert ist aufgrund von fehlerhaften Passworteingaben fr 60 Sekunden gesperrt.", error);
			return;
		}

		switch(function) {
		case IV_FUNCTION_GET_TABLE:
		case IV_FUNCTION_GET_FIELD:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der Subminorbestandteil (Feldname) des Tabellenfeldnamens ist unbekannt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_OPEN:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt.", error); break;
				// Standardmaessig ist die Option mit 0 angegeben. Ein gesetztes Bit, dessen Bedeutung unklar ist, ergibt diesen Fehler.
			case 3:sprintf(text, "( %i ): Unbekannte Option gesetzt.", error); break;
				// Wenn ein reservierter Parameter angegeben wurde.
			case 4:sprintf(text, "( %i ): Der angegebene Reservierte Parameter wird nicht untersttzt.", error); break;
				// Die Tabelle ist bereits mit schreib - Recht geoffnet worden.
			//case 5:sprintf(text, "( %i ): Die Tabelle kann nur lesend geoeffnet werden.", error); break;
				// Die Tabelle ist bereits mit Exclusive - Recht geoffnet worden.
			//case 6:sprintf(text, "( %i ): Die Tabelle ist z. Z. gesperrt.", error); break;
				// Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt. (Zu oft geoeffnet worden usw..)
			//case 7:sprintf(text, "( %i ): Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_CLOSE:
			switch(error) {
				// Wenn versucht wird ein Handle zu schliessen der keiner offenen Tabelle zugehoerig ist.
			case 1:sprintf(text, "( %i ): Angegebener Handle war nicht geoffnet.", error); break;
				// Liste konnte nicht geschlossen werden.
			case 2:sprintf(text, "( %i ): Tabelle konnte nicht geschlossen werden.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_SET_FILTER:
		case IV_FUNCTION_TABLE_SET_FILTER_OK:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Ungueltiger Selektionsspaltenanzahl.
			case 2:sprintf(text, "( %i ): Falsche Anzahl von Selektionsspalten.", error); break;
				// Nach setzen des Filterkriteriums -> Kein Datensatz
			case 3:sprintf(text, "( %i ): Filterkriterium liefert kein Ergebnis.", error); break;
				// Spaltenname des Filter unbekannt.
			case 4:sprintf(text, "( %i ): Unbekannter Spaltenname.", error); break;
				// Unbekannter (nicht unterstuetzter) Filteroperator.
			case 5:sprintf(text, "( %i ): Unbekannter Filteroperator.", error); break;
				// Fehlender in Anfuehrungszeichenpaar des zu filterndem Spaltenwerts.
				// Keine vorhanden, unterschiedliches ein/aus '" oder fehlender ein oder Ausgang.
			case 6:sprintf(text, "( %i ): Fehler in Anfuehrungszeichenpaar des zu filterndem Spaltenwertes.", error); break;
				// Fehler im zu Filterndem Spaltenwert. z. B. Buchstaben obwohl nur Ziffern, zu lange Angabe etc.
			//case 7:sprintf(text, "( %i ): Fehler im zu filterndem Spaltenwert.", error); break;
				// Syntaxfehler im Filter. z. B. Abschluss des Filters mit Semikolon (reserviert fuer zukuenftige Erweiterung auf mehrer Spalten).
			//case 8:sprintf(text, "( %i ): Im zu setzendem Filter ist ein Syntaxfehler.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_GET_FILTER:
		case IV_FUNCTION_TABLE_CLEAR_FILTER:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Auf der angegebenen Tabelle ist kein Filter gesetzt.
			//case 2:sprintf(text, "( %i ): Kein Filter bei Tabelle angegeben.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_SKIP:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Wert fuer Origin ausserhalb des gueltigen Bereichs. (0, 1, 2)
			case 2:sprintf(text, "( %i ): Wert fuer Origin ungueltig.", error); break;
				// Sprungweite (Offset) fuehrte zu BOF (begin of file). Der Datensatzzeiger steht damit vor dem ersten Datensatz.
			case 3:sprintf(text, "( %i ): Sprung fuehrte zu BOF.", error); break;
				// Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
			case 4:sprintf(text, "( %i ): Sprung fuehrte zu EOF.", error); break;
				// Sprung konnte aufgrund von angegebenen offset mit origin nicht durchgefuehrt werden.
				// Z. B. Filter gesetzt und Sprung auf aktuellen der nicht dem Filterkriterium entspraeche.
			case 5:sprintf(text, "( %i ): Sprung konnte nicht durchgefuehrt werden.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_SET_FIELD_VALUE:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Spaltenname wurde nicht gefunden.
			case 2:sprintf(text, "( %i ): Spaltenname unbekannt.", error); break;
				// Angegebener Wert ist zu lang
			case 3:sprintf(text, "( %i ): Zu setzender Wert ist zu lang.", error); break;
				// Formatfehler in zu setzendem Spaltenwert.
			case 4:sprintf(text, "( %i ): Formatfehler in zu setzendem Spaltenwert.", error); break;
				// Schreiben des Spaltenwerts fehlgeschlagen
			case 5:sprintf(text, "( %i ): Schreiben des Spaltenwerts fehlgeschalgen.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_GET_FIELD_VALUE:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Spaltenname wurde nicht gefunden.
			case 2:sprintf(text, "( %i ): Spaltenname unbekannt.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
		case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
		case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
		case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
		case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
		case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
			switch(error) {
			case 1: sprintf(text, "( %i ): Das gewhlte Templateformat ist unbekannt.", error); break;
			case 2: sprintf(text, "( %i ): Das Modul antwortet nicht auf die Anfrage.", error); break;
			case 3: sprintf(text, "( %i ): Das gewhlte Templateformat entspricht nicht dem verwendeten.", error); break;
			case 7: sprintf(text, "( %i ): Das Modul kann zur Zeit nicht angesprochen werden.", error); break;
			case 129: sprintf(text, "( %i ): Das Modul bringt einen nicht nher spezifizierten Fehler.", error); break;
			case 132: sprintf(text, "( %i ): Die Datenbank des Moduls ist voll.", error); break;
			case 133: sprintf(text, "( %i ): Das Modul enthlt keine Fingertemplates.", error); break;
			case 134: sprintf(text, "( %i ): Das Fingertemplate existiert bereits.", error); break;
			case 135: sprintf(text, "( %i ): Das angeforderte Fingertemplate existiert nicht.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_GENERATE:
		case IV_FUCNTION_TABLE_GENERATE_OK:
			switch(error) {
			case 7: sprintf(text, "( %i ): Das Modul kann zur Zeit nicht angesprochen werden.", error); break;
			case 129: sprintf(text, "( %i ): Das Modul bringt einen nicht nher spezifizierten Fehler.", error); break;
			case 133: sprintf(text, "( %i ): Das Modul enthlt keine Daten zum erzeugen einer Liste.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleApplication::GetErrorFunction(module, function, error, text);
}

//#########################################################
//####################### TABLE ###########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::Call(short module, short function, unsigned char *data, int &count)
{
	if (module == IV_MODULE_TABLE) {

		switch(function) {

		// Ermitteln der Tabellendefinition
		case IV_FUNCTION_GET_TABLE: return GetTable(GetDeviceID(), (char*)data, (STable2&)data[strlen((char*)data) + 1]);
		// Ermitteln der Felddefinition
		case IV_FUNCTION_GET_FIELD: return GetField(GetDeviceID(), (char*)data, (STable2Field&)data[strlen((char*)data) + 1]); 
		case IV_FUNCTION_TABLE_OPEN:
		case IV_FUNCTION_TABLE_CLOSE:
		case IV_FUNCTION_TABLE_SET_FILTER:
		case IV_FUNCTION_TABLE_SET_FILTER_OK:
		case IV_FUNCTION_TABLE_GET_FILTER:
		case IV_FUNCTION_TABLE_CLEAR_FILTER:
		case IV_FUNCTION_TABLE_SKIP:
		case IV_FUNCTION_TABLE_SET_FIELD_VALUE:
		case IV_FUNCTION_TABLE_GET_FIELD_VALUE:
		case IV_FUNCTION_TABLE_SET_RECORD:
		case IV_FUNCTION_TABLE_GET_RECORD:
		case IV_FUNCTION_TABLE_GENERATE:
			// Funktion nicht definiert
			return 0x0200; 
		case IV_FUCNTION_TABLE_GENERATE_OK:
			return TableGenerateOK(GetDeviceID());
		case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
			// Funktion nicht definiert
			return 0x0200; 
		case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK:
			return FingerprintTableAppendRecordOK(GetDeviceID());
		case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
			// Funktion nicht definiert
			return 0x0200; 
		case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK:
			return FingerprintTableGetRecordOK(GetDeviceID());
		case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
			// Funktion nicht definiert
			return 0x0200; 
		case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK:
			return FingerprintTableDeleteRecordOK(GetDeviceID());
		case IV_FUNCTION_TABLE_GET_RECORD_COUNT:
		case IV_FUNCTION_TABLE_GET_RECORD_NUM:
		// Funktion nicht definiert
		default: return 0x0200; 
		}

	}

	return CChannel4ModuleApplication::Call(module, function, data, count);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::GetTable(short deviceID, char *name, STable2 &table)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::GetTable [ %i, 0x%08X, STable2 ]\n", __FILE__, __LINE__, deviceID, name);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		count = sprintf((char*)m_data, "%s", name) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE);
			return m_result;
		}

		// OK
		if (count != sizeof(STable2)) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::GetTable [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Struktur uebernehmen
		table = *(STable2*)m_data;
	} else {

		unsigned char data[256];
		STable2 t;
		count = 0;
		short err = IV_ERROR_SUCCESS;
		
		strcpy((char*)data, name);
		memset(&t, 0, sizeof(STable2));

		char *major, *minor;

		major = strtok((char*)data, ".");
		minor = strtok(NULL, ".");

		if (strcmp(major, "RECORD") == 0) {
			switch(atoi(minor)) {
			case 0:
				t.num = 0;
				strcpy(t.name, "PZE-Meldung");
				t.count = 5;
				break;
			case 1:
				t.num = 1;
				strcpy(t.name, "Status");
				t.count = 8;
				break;
			default:
				// Minorbestandteil undefiniert
				err = 0x0002;
			}

		} else if (strstr((char*)data, "LIST") == (char*)data) {

				// Minorbestandteil undefiniert
				err = 0x0002;
		} else if (strstr((char*)data, "ACCESS") == (char*)data) {

				// Minorbestandteil undefiniert
				err = 0x0002;
		} else {
			
			// Majorbestandteil undefiniert.
			err = 0x0001;
		}

		if (err == IV_ERROR_SUCCESS) {

			memcpy(&data[count], &t, sizeof(STable2));
			count = sizeof(STable2);
		} else {

			*(short*)data = err;
			count = sizeof(short);
		}
		// Antwort des Befehls senden.
		if ((m_result = Execute((err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE, data, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::GetTable [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::GetField(short deviceID, char *name, STable2Field &field)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::GetField [ %i, 0x%08X, STable2Field ]\n", __FILE__, __LINE__, deviceID, name);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		count = sprintf((char*)m_data, "%s", name) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD);
			return m_result;
		}

		// OK
		if (count != sizeof(STable2Field)) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::GetField [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Sturktur uebernehmen
		field = *(STable2Field*)m_data;
	} else {

		unsigned char data[256];
		STable2Field f;
		count = 0;
		short err = IV_ERROR_SUCCESS;

		strcpy((char*)data, name);
		memset(&field, 0, sizeof(field));
		char *major, *minor, *subminor;

		major = strtok((char*)data, ".");
		minor = strtok(NULL, ".");
		subminor = strtok(NULL, ".");

		if (strcmp(major, "RECORD") == 0) {

			switch(atoi(minor)) {
			case 0: {
				switch(atoi(subminor)) {
				case 0:
					f.num = 0;
					strcpy(f.name, "DU");
					f.type = TF_DT;
					f.size = 7;
					break;
				case 1:
					f.num = 1;
					strcpy(f.name, "Kennung");
					f.type = TF_A_NUM;
					f.size = 5;
					break;
				case 2:
					f.num = 2;
					strcpy(f.name, "PID");
					f.type = TF_A_NUM;
					f.size = 9;
					break;
				case 3:
					f.num = 3;
					strcpy(f.name, "IP");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 4:
					f.num = 4;
					strcpy(f.name, "Port");
					f.type = TF_A_NUM;
					f.size = 9;
					break;
				default:
					err = 0x0003;
				}
				break;}
			case 1: {
				switch(atoi(subminor)) {
				case 0:
					f.num = 0;
					strcpy(f.name, "DU");
					f.type = TF_DT;
					f.size = 7;
					break;
				case 1:
					f.num = 1;
					strcpy(f.name, "Feld1");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 2:
					f.num = 2;
					strcpy(f.name, "Feld2");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 3:
					f.num = 3;
					strcpy(f.name, "Feld3");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 4:
					f.num = 4;
					strcpy(f.name, "Feld4");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 5:
					f.num = 5;
					strcpy(f.name, "Feld5");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 6:
					f.num = 6;
					strcpy(f.name, "Feld6");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				case 7:
					f.num = 7;
					strcpy(f.name, "Feld7");
					f.type = TF_A_NUM;
					f.size = 21;
					break;
				default:
					err = 0x0003;
				}
				break;}
			default:
				err = 0x0002;
			}

		} else if (strcmp(major, "LIST") == 0) {

				err = 0x0002;
		} else if (strcmp((char*)data, "ACCESS") == 0) {

				err = 0x0002;
		} else {
			
			// Majorbestandteil undefiniert.
			err = 0x0001;
		}

		if (err == IV_ERROR_SUCCESS) {
			
			memcpy(&data[count], &f, sizeof(STable2Field));
			count = sizeof(STable2Field);
		} else {

			*(short*)data = err;
			count = sizeof(short);
		}

		// Antwort des Befehls senden.
		if ((m_result = Execute((err == IV_ERROR_SUCCESS) ? IV_CONTEXT_TRANSMIT_ANSWER : IV_CONTEXT_TRANSMIT_ANSWER_ERROR, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD, data, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::GetField [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableOpen(short deviceID, char *name, int options, int &handle, char *reserved)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableOpen [ %i, 0x%08X, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, deviceID, name, options, handle, reserved);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		count = sprintf((char*)m_data, "%s", name) + 1;

		*(int*)&m_data[count] = options;
		count += sizeof(int);

		count += sprintf((char*)&m_data[count], "%s", reserved) + 1;


		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN);
			return m_result;
		}

		// OK
		// Ein leerbyte fuer reservierten String in Antwort
		if (count != (sizeof(int) + sizeof(unsigned char))) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableOpen [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Handle uebernehmen
		handle = *(int*)m_data;
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableOpen [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableClose(short deviceID, int handle)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableClose [ %i, %i ]\n", __FILE__, __LINE__, deviceID, handle);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableClose [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableClose [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetFilter(short deviceID, int handle, char *filter)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSetFilter [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceID, filter);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);
		count += sprintf((char*)&m_data[count], "%s", filter) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableSetFilter[ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}
	/*
		short module, function, error = IV_ERROR_SUCCESS;
		// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
		// dieses musz hier seperat abgegriffen werden.
		if ((m_result = Pending(deviceID, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {

			Trace("%s(%i):ERROR: BeginGetSegment.Pending [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		if ((module == IV_MODULE_TABLE) && (function == IV_FUNCTION_TABLE_SET_FILTER_OK)) {

			Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSetFilter.Pending [ Antwort OK ]\n", __FILE__, __LINE__);
		}

		// evntl. aktiven Fehler auswerten
		if (error != IV_ERROR_SUCCESS) {

			m_result = error;
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK);
			return m_result;
		}
	*/
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, m_data, count, (IV_PENDING_TIMEOUT / GetTimeout()))) != IV_ERROR_SUCCESS ) {

			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK);
			return m_result;
		}

		// Keine Aktive - Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableSetFilter.Pending [ Aktive - Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}	
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSetFilter [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetFilterOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSetFilterOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSetFilterOK\n", __FILE__, __LINE__);
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSetFilterOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetFilter(short deviceID, int handle, char *filter, short &len)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGetFilter [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, filter, len);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter uebernehmen
		*(int*)m_data = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER);
			return m_result;
		}

		// OK
		if (count > 80) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetFilter [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Wert uebernehmen
		strcpy(filter, (char*)m_data);
		len = strlen(filter);
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGetFilter [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableClearFilter(short deviceID, int handle)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableClearFilter [ %i, %i ]\n", __FILE__, __LINE__, deviceID, handle);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableClearFilter [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableClearFilter [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSkip(short deviceID, int handle, int offset, short origin)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSkip [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceID, offset, origin);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);
		*(int*)&m_data[count] = offset;
		count += sizeof(int);
		*(short*)&m_data[count] = origin;
		count += sizeof(short);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableSkip [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSkip [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetField(short deviceID, int handle, char *name, char *value)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSetField [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceID, name, value);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);
		count += sprintf((char*)&m_data[count], "%s", name) + 1;
		count += sprintf((char*)&m_data[count], "%s", value) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableSetField [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSetField [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetField(short deviceID, int handle, char *name, char *value, short &len)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGetField [ %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, name, value, len);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)m_data = handle;
		count = sizeof(int);
		count += sprintf((char*)&m_data[count], "%s", name) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE);
			return m_result;
		}

		// OK
		if (count > 80) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetField [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Wert uebernehmen
		strcpy(value, (char*)m_data);
		len = strlen(value);

	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGetField [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetRecord(short deviceID, int handle, unsigned char *data, short len)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableSetRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, handle, data, len);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)&m_data[count] = handle;
		count = sizeof(int);
		memcpy(&m_data[count], data, len);
		count += len;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableSetRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableSetRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecord(short deviceID, int handle, unsigned char *data, short &len)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGetRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, handle, data, len);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)&m_data[count] = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD);
			return m_result;
		}

		// OK
		if (count > 256) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		if (count > len) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetRecord [ Puffer fr Antwortdaten zu klein gewhlt. ]\n", __FILE__, __LINE__, count);
			// Wert uebernehmen
			memcpy(data, (char*)m_data, len);
		} else {

			// Wert uebernehmen
			memcpy(data, (char*)m_data, count);
			len = count;
		}

	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGetRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}
	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecordCount(short deviceID, int handle, int &recordCount)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGetRecordCount [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceID, recordCount);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)&m_data[count] = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT);
			return m_result;
		}

		// OK
		if (count > sizeof(int)) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetRecordCount [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Anzahl uebernehmen
		recordCount = *(int*)m_data;
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGetRecordCount [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecordNum(short deviceID, int handle, int &recordNum)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGetRecordNum [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceID, recordNum);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Name als Parameter eintragen
		*(int*)&m_data[count] = handle;
		count = sizeof(int);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM);
			return m_result;
		}

		// OK
		if (count > sizeof(int)) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGetRecordCount [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Anzahl uebernehmen
		recordNum = *(int*)m_data;
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGetRecordNum [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableAppendRecord(short deviceID, unsigned char type, unsigned char *data, int size)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableAppendRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, type, data, size);

	int count = 0, i;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		m_data[count] = type;
		count = sizeof(unsigned char);

		for (i=0; i < size; i++) {
		
			m_data[count + i] = data[i];
		}
		count += size;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD);
			return m_result;
		}

		// OK
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableAppendRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Aktive Antwort abwarten
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, m_data, count, (IV_PENDING_TIMEOUT / GetTimeout()))) != IV_ERROR_SUCCESS ) {

			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK);
			return m_result;
		}
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableAppendRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableAppendRecordOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableAppendRecordOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableAppendRecordOK\n", __FILE__, __LINE__);
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableAppendRecordOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableGetRecord(short deviceID, unsigned char &type, unsigned long pid, unsigned char fid, unsigned char *data, int &size)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableGetRecord [ %i, %i, %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceID, type, pid, fid, data, size);

	int count = 0, i;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		m_data[count] = type;
		count = sizeof(unsigned char);
		*(unsigned long*)&m_data[count] = pid;
		count += sizeof(unsigned long);
		m_data[count] = fid;
		count += sizeof(unsigned char);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Wenn die Funktion unbekannt ist,
			// wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
			if (((m_result >> 8) & 0xFF) == 2) {

				if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1, m_data, count)) != IV_ERROR_SUCCESS) {

					// Fehler ausgeben
					TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1);
					return m_result;
				}

				// Templatetype uebernehmen
				type = m_data[0];
				// Templatedaten uebernehmen
				size = count - 1;
				for (i=1; i < count; i++) {
				
					data[i - 1] = m_data[i];
				}
				count += size;

				// OK zurueckmelden
				m_result = IV_ERROR_SUCCESS;
				return m_result;
			}


			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD);
			return m_result;
		}

		// OK
		if (count) {
			
			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableGetRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Aktive Antwort abwarten
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, m_data, count, (IV_PENDING_TIMEOUT / GetTimeout()))) != IV_ERROR_SUCCESS ) {
			
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK);
			return m_result;
		}

		// Templatetype uebernehmen
		type = m_data[0];
		// Templatedaten uebernehmen
		size = count - 1;
		for (i=1; i < count; i++) {
		
			data[i - 1] = m_data[i];
		}
		count += size;
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableGetRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableGetRecordOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableGetRecordOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableGetRecordOK\n", __FILE__, __LINE__);
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableGetRecordOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableDeleteRecord(short deviceID, unsigned long pid, unsigned char fid)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableDeleteRecord [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceID, pid, fid);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Parameter eintragen
		*(unsigned long*)&m_data[count] = pid;
		count = sizeof(unsigned long);
		m_data[count] = fid;
		count += sizeof(unsigned char);


		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

			// Wenn die Funktion unbekannt ist,
			// wird die Vorgaengerversion ohne "Aktive Antwort" geprueft.
			if (((m_result >> 8) & 0xFF) == 2) {

				// Vorgaengerfunktion ausfuehren.
				if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1, m_data, count)) != IV_ERROR_SUCCESS) {

					// Fehler ausgeben
					TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1);
					return m_result;
				}

				// OK
				if (count) {

					Trace("%s(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableDeleteRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
				}

				// OK zurueckmelden
				m_result = IV_ERROR_SUCCESS;
				return m_result;
			}

			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD);
			return m_result;

		}

		// OK
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableDeleteRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		}

		// Aktive Antwort abwarten
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, m_data, count, (IV_PENDING_TIMEOUT / GetTimeout()))) != IV_ERROR_SUCCESS ) {
			
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK);
			return m_result;
		}

	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableDeleteRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableDeleteRecordOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::FingerprintTableDeleteRecordOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableDeleteRecordOK\n", __FILE__, __LINE__);
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::FingerprintTableDeleteRecordOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGenerate(short deviceID, char *name, char *reserved)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGenerate [ %i, %s, %s ]\n", __FILE__, __LINE__, deviceID, name, reserved);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Alle Zusatzparameter uebernehmen
		count += sprintf((char*)&m_data[count], "%s", name) + 1;
		count += sprintf((char*)&m_data[count], "%s", reserved) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, m_data, count)) != IV_ERROR_SUCCESS) {

			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) {

			Trace("%s(%i):WARNING(1): CChannel4ModuleTable::TableGenerate [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
		} else {

			count = 0;
		}

		// Aktive Antwort abwarten
		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, m_data, count, (IV_PENDING_TIMEOUT / GetTimeout()))) != IV_ERROR_SUCCESS ) {

			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK);
			return m_result;
		}
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGenerate [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGenerateOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleTable::TableGenerateOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) {

		Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGenerateOK\n", __FILE__, __LINE__);
	
	} else {

		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, tmp, count)) != IV_ERROR_SUCCESS) {
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleTable::TableGenerateOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
		
	}


	return m_result;
}
