#ifndef _CHANNEL4_MODULE_TABLE_H_
#define _CHANNEL4_MODULE_TABLE_H_

// Basisklasse einbinden.
#include "Channel4ModuleApplication.h"

// MODULE_TABLE( 3 )

#define IV_MODULE_TABLE                          0x0003

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Get: Datensatz-/Listenbeschreibung. GET_TABLE( 0 )
#define IV_FUNCTION_GET_TABLE                    0x0000
// (Majorname.Minorname\0)
// (STable2)

// Die Tabellenbeschreibung kann auf folgende Arten ermittelt werden.
// Zum einen ist es noetig die Datensatz-/Listenbeschreibungen des Setup sowie der
// Zutrittskontrolle auseinanderzuhalten. Dann wird entweder der Index oder der Name,
// der jeweilingen Tabellendefinition benoetigt.

// Majornamen:
// RECORD - Sind die Datensatzbeschreibungen
// LIST   - Sind die Listenbeschreibungen
// ACCESS - Sind die Listenbeschreibungen der Zutrittskontrolle

// darauf folgt nach einem Punkt entweder die Nummer oder der Name der Tabelle.

// Get: Datensatz-/Listenbeschreibungsfeld. GET_FIELD( 1 )
#define IV_FUNCTION_GET_FIELD                    0x0001
// (Majorname.Minorname.Subminorname\0)
// (STableField)

// Majorname.Minorname bauen sich wie bei den Tabellen auf. Der Subminorname entspricht
// in seinem Aufbau dem Minornamen, ist logischerweise jedoch Feldbezogen.

// Fehler:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = SUBMINOR_NAME ist unbekannt

#define IV_FUNCTION_TABLE_OPEN                   0x0002
#define IV_FUNCTION_TABLE_CLOSE                  0x0003
#define IV_FUNCTION_TABLE_SET_FILTER             0x0004
#define IV_FUNCTION_TABLE_SET_FILTER_OK          0x0005
#define IV_FUNCTION_TABLE_GET_FILTER             0x0006
#define IV_FUNCTION_TABLE_CLEAR_FILTER           0x0007
#define IV_FUNCTION_TABLE_SKIP                   0x0008
#define IV_FUNCTION_TABLE_SET_FIELD_VALUE        0x0009
#define IV_FUNCTION_TABLE_GET_FIELD_VALUE        0x000A
#define IV_FUNCTION_TABLE_SET_RECORD             0x000B
#define IV_FUNCTION_TABLE_GET_RECORD             0x000C
#define IV_FUNCTION_TABLE_GENERATE               0x000D
#define IV_FUCNTION_TABLE_GENERATE_OK            0x000E
#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD    0x000F
#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK 0x0010
#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1      0x0011
#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1   0x0012
#define IV_FUNCTION_TABLE_GET_RECORD_COUNT             0x0013
#define IV_FUNCTION_TABLE_GET_RECORD_NUM               0x0014
#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD       0x0015
#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK    0x0016
#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD    0x0017
#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK 0x0018


// Insert RECORD_INSERT(  )
// (Name\0...\0)
// -> (Name\0)

// Delete RECORD_DELETE(  )
// (Name\0KeyFeld\0)
// -> (Name\0)
// Name ist der Tabellenname
// KeyFeld ist der Inhalt der gewnschten Datenzeile

/*
#pragma pack(1)

typedef struct StructTable2 {
	unsigned char num;     // Nummer der Tabelle im System
	char name[32];         // Name der Datensatzbeschreibung
	unsigned char count;   // Anzahl Felder im Datensatz 
	unsigned char key;     // Schluesselspalte
	unsigned char sort;    // Sortierspalte
	unsigned short flags;  // Flags
} STable2;

typedef struct StructTable2Field {
	unsigned char num;     // Nummer des Tabellenfeldes
	char name[32];         // Name des Feldes
	unsigned char type;    // Aufzaehlung Typ des Feldes 
	unsigned char size;    // Laenge des Feldes inc. anbschlieszende '\0' bei Strings 
	unsigned short flags;  // Flags
	char format[32];       // Formatstring fuer Feldwertmoeglichkeit
} STable2Field;

#pragma pack()
*/

class CChannel4ModuleTable : public CChannel4ModuleApplication
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleTable()  : CChannel4ModuleApplication(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID)
	{ InitClass(); }
	CChannel4ModuleTable(short channelID, short deviceID) : CChannel4ModuleApplication(channelID, deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleTable();

public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short function, short error, char *text);

	// MODULE_TABLE

	// Tabellendefinition ueber Index oder Namen ermitteln.
	int GetTable(short deviceID, char *name, STable2 &table);
	// Tabellenfelddefinition ueber Index oder Namen ermitteln.
	int GetField(short deviceID, char *name, STable2Field &field);

	// Oeffnen einer Tabelle
	int TableOpen(short deviceID, char *name, int options, int &handle, char *reserved);
	// Schliessen der Tabelle
	int TableClose(short deviceID, int handle);

	// Setzen eines Filters auf die Tabelle
	int TableSetFilter(short deviceID, int handle, char *filter);
	int TableSetFilterOK(short deviceID);
	// Lesen des Filters auf die Tabelle
	int TableGetFilter(short deviceID, int handle, char *filter, short &len);
	// Loeschen des Filters der Tabelle
	int TableClearFilter(short deviceID, int handle);
	
	// Ueberspringen von n Datensaetzen (offset kann negativ sein), ab top/aktuell/bottom
	int TableSkip(short deviceID, int handle, int offset, short origin);
	
	// Aendern eines Spaltenwerts, des aktuellen Datensatzes
	int TableSetField(short deviceID, int handle, char *name, char *value);
	// Abfragen eines Spaltenwerts, des aktuellen Datensatzes
	int TableGetField(short deviceID, int handle, char *name, char *value, short &len);

	// Aendern eines aktuellen Datensatzes
	int TableSetRecord(short deviceID, int handle, unsigned char *data, short len);
	// Abfragen eines aktuellen Datensatzes
	int TableGetRecord(short deviceID, int handle, unsigned char *data, short &len);

	// Tabelle erzeugen
	int TableGenerate(short deviceID, char *name, char *reserved);
	int TableGenerateOK(short deviceID);

	// Abfragen der Datensatzanzahl
	int TableGetRecordCount(short deviceID, int handle, int &recordCount);
	// Abfragen der aktuellen Datensatznummer
	int TableGetRecordNum(short deviceID, int handle, int &recordNum);
	
	// Datensatz an Fingerprint-Tabelle anfuegen.
	int FingerprintTableAppendRecord(short deviceID, unsigned char type, unsigned char *data, int size);
	int FingerprintTableAppendRecordOK(short deviceID);
	// Datensatz aus Fingerprint-Tabelle lesen.
	int FingerprintTableGetRecord(short deviceID, unsigned char &type, unsigned long pid, unsigned char fid, unsigned char *data, int &size);
	int FingerprintTableGetRecordOK(short deviceID);
	// Datensatz aus Fingerprint-Tabelle loeschen.
	int FingerprintTableDeleteRecord(short deviceID, unsigned long pid, unsigned char fid);
	int FingerprintTableDeleteRecordOK(short deviceID);


	//#################################
	// CallRoutinen

	// Ausfuehren von Befehlen
	virtual int Call(short module, short function, unsigned char *data, int &count);

};

#endif
