/** 
 * @file ChannelBase.h
 * @brief Schnittstelle fuer die Klasse CChannelBase.
 */
//////////////////////////////////////////////////////////////////////

#ifndef _CHANNELBASE_H_
#define _CHANNELBASE_H_

#pragma warning(disable : 4996)

#ifdef _WIN32
#include <winsock.h>
#else
#include "../Common/StdDef.h"
#endif

/////////////////////////////
// Error's
#include "../Common/AEErrorDef.h"

// Standardwerte als Vorgabe
#define IV_DEFAULT_CHANNELID        0

// Wiederholungsanzahlen
#define IV_RECEIVE_TRIALS   3 // Maximale Anzahl Leseversuche.
#define IV_TRANSMIT_TRIALS  3 // Maximale Anzahl Schreibversuche.

/// <summary>Basisklasse fuer pure Kommunikationsaufgaben.</summary>
class CChannelBase  
{
public:
	/// Konstruktor fuer Initialisierung der Kanalklasse.
	CChannelBase() { Init(); }
	CChannelBase(short id) { Init(id); };
	virtual void Init(short channelID = IV_DEFAULT_CHANNELID);
	/// Destruktor
	virtual ~CChannelBase();

	/// Ermitteln der zugewiesenen Kanalnummer.
	inline short GetID() { return m_id; };
	/// Zuweisen der Kanalnummer.
	inline short SetID(short id) { short old = m_id; m_id = id; return old; };

	/// Typ des Kommunikationskanals
	enum enumType { Undefined, Serial, Socket };
	// Typ des Kanals ermitteln
	inline enumType GetType() { return m_type; }

	/// Oeffnen der Schnittstelle
	int Open(enumType type, char *string, int value, int to);
	/// Schliessen der Schnittstelle.
	int Close(void);
	// Zuweisen von HANDLE oder Socket ohne Schnittstelle selbst zu oeffnen.
	int AttachChannel(enumType type, HANDLE value, int timeOut);
	// Freigeben der anderweitig zugewiesenen Resourcen
	int ReleaseChannel(void);

	/// Ermitteln ob Schnittstelle geoeffnet.
	int IsOpen(void);

	/// Loeschen anstehender Daten.
	void Flush();

	/// Setzen des Logdateinamen.
	inline int SetTraceFileName(const char *fileName)
	{ strcpy(m_traceFileName, (const char*)fileName); return IV_ERROR_SUCCESS; }
	
	/// Trace an-/ausschalten.
	inline void SetTrace(int state = TRUE)
	{ m_trace = state; }

	/// Tracezustand pruefen.
	inline int IsTrace(void)
	{ return m_trace; }

	/// Ermitteln der uebergebenen Parameter an Open.
	inline void GetOpenParams(enumType &type, char *string, int &value, int &to) 
	{type = m_type; strcpy(string, m_commString); value = m_value; to = m_to; };

	// Gibt die Grundklasse fuer pure Kommunikationsaufgaben zurueck
	CChannelBase& GetChannelBase() { return (CChannelBase&)*this; }

	HANDLE m_comm;   // Handle serielle   Schnittstelle
	SOCKET m_socket; // Handle TCP_Socket Schnittstelle
	char m_commString[64]; // Haelt letzten Kommunikationsstring
protected:
	enumType m_type; // Typ der Schnittstelle
	int m_value;     // Comm-Value halten
	int m_to;        // Timeout fuer Kommunikationsfunktionen halten

protected:
	int  ChangeStateSerial(HANDLE handle, int baud, int to);

	// Zugriff auf private member: m_to (Schnittstellentimeout)
	inline void SetTimeout(int to) { m_to = to; }
	inline int GetTimeout() { return m_to; }

	/// Oeffnen der Schnittstelle fuer RS232 / RS485 Verbindung.
	int  OpenSerial(char *comm, int baud, int to);
	int CloseSerial(void);

	/// Oeffnen der Schnittstelle fuer TCP/IP.
	int  OpenSocket(char *remote, int port, int to);
	int CloseSocket(void);


protected:
	// Datenempfang
	int  ReceiveSerial(void *buffer, int receiveCount, int &receivedCount, int receiveTrials = IV_RECEIVE_TRIALS);
	int  ReceiveSocket(void *buffer, int receiveCount, int &receivedCount, int receiveTrials = IV_RECEIVE_TRIALS);

public:
	int  ReceiveData(void *buffer, int receiveCount, int &receivedCount, int receiveTrials = IV_RECEIVE_TRIALS);

protected:
	// Datenversand
	int  TransmitSerial(const void *buffer, int transmitCount, int &transmittedCount, int transmitTrials = IV_TRANSMIT_TRIALS);
	int  TransmitSocket(const void *buffer, int transmitCount, int &transmittedCount, int transmitTrials = IV_TRANSMIT_TRIALS);

public:
	int TransmitData(const void *buffer, int transmitCount, int &transmittedCount, int transmitTrials = IV_TRANSMIT_TRIALS);

protected: // Statistik
	int m_transmittedSize;     // Anzahl der versendeten Bytes
	int m_receivedSize;        // Anzahl der gelesenen Bytes

protected: // Logging
	int m_trace;
	char m_traceFileName[MAX_PATH];

public:
	virtual void Trace(const char *format, ...);

protected:
	// zugewiesene Kanalnummer
	short m_id;
#ifndef _WIN32
	struct termios m_termio;
	struct flock  m_flock;
#endif
};

#endif
