#ifndef _DATAFLASH_H_
#define _DATAFLASH_H_

// Maximale Anzahl der Flashseiten
#define DATAFLASH_MAX_PAGE  8192
// Gre einer Flashseite in Byte (davon sind die hinteren 16 Byte fr interne Zwecke verwendet)
#define DATAFLASH_MAX_PAGE_SIZE 528
// Gre der Erweiterten Kontrolldaten einer Flashseite
#define DATAFLASH_PAGE_SIZE_EXTENDED 16
// Gre des Dateninhalts in einer Flashseite
#define DATAFLASH_PAGE_SIZE_DATA (DATAFLASH_MAX_PAGE_SIZE - DATAFLASH_PAGE_SIZE_EXTENDED)
// 4MByte Gesamtspeicher
#define DATAFLASH_SIZE		(DATAFLASH_MAX_PAGE * DATAFLASH_MAX_PAGE_SIZE)
 // Entspricht: 4325376 Byte = 4,125 MByte

class CDataFlash 
{
public:
	CDataFlash();
	virtual ~CDataFlash();

	// Ruecksetzen
	void Reset();

	// EEPROM
	inline unsigned char *GetMemDirect(unsigned long address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE) 
		{ ASSERT(0); return 0; }
		return &m_mem[address]; 
	}
	inline unsigned char GetMem(unsigned long address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE) 
		{ ASSERT(0); return 0; }
		return m_mem[address]; 
	}
	inline unsigned long GetMemRange(unsigned long address, unsigned char *value, unsigned long size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_SIZE)
		{ ASSERT(0); return 0; }
		unsigned long i;
		for (i=address; i < (address + size); i++) {
		
			value[i - address] = m_mem[i];
		}
		return (i - address);
	}
	inline void SetMem(unsigned long address, unsigned char value) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_SIZE)
		{ ASSERT(0); return; }
		m_mem[address] = value; 
		SetMemUsebit(address + m_baseAddress);
	}
	inline void SetMemRange(unsigned long address, unsigned char *value, unsigned long size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_SIZE)
		{ ASSERT(0); return; }
		for (unsigned long i=address; i < (address + size); i++) {
		
			m_mem[i] = value[i - address];
			SetMemUsebit(i + m_baseAddress);
		}
	}

	// Helper
	void SetBaseAddress(unsigned long base) { m_baseAddress = base; };
	unsigned long GetBaseAddress() { return m_baseAddress; };
	void SetMemUsebit(unsigned long address);
	unsigned char GetMemUsebit(unsigned long address);
	unsigned long GetMemFirstAddress();
	unsigned long GetMemLastAddress();

protected:
	unsigned long m_baseAddress;
	unsigned char *m_mem;
	unsigned long *m_memUse;
};

#endif
