// DFItemDevice.cpp: Implementierung der Klasse CDFItemDevice.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DFItemDevice.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDFItemDevice::CDFItemDevice(NodeTypeID _type) : CDFItemBase(_type)
{

}

CDFItemDevice::~CDFItemDevice()
{

}

int CDFItemDevice::Modify(CModifyData &modifyData)
{
	TConfig &config = modifyData.file->m_config;
	
	// Gertetyp ersetzen?
	if (modifyData.flags & SMDF_DEVICE_TYPE) {
	
		modifyData.file->m_eep[P_DEVICE_TYPE] = modifyData.deviceType + 1;
	}

	// Systempasswort ersetzen?
	if (modifyData.flags & SMDF_PASSWORD_SYSTEM) {
	
		config.SYSTEMMENU_PASSWORD = (unsigned short)modifyData.passwordSystem;
		ChangeOrderInt((unsigned char*)&config.SYSTEMMENU_PASSWORD);
	}
	// Benutzerpasswort ersetzen?
	if (modifyData.flags & SMDF_PASSWORD_USER) {
		
		config.TRANSPONDER_USER_PASSWORD = (unsigned short)modifyData.passwordUser;
		ChangeOrderInt((unsigned char*)&config.TRANSPONDER_USER_PASSWORD);
	}

	// Erweiterten Struktur lesen
	TExtendedFlash &extBegin = *(TExtendedFlash*)modifyData.file->m_flashExtended;
	TExtended extended;

	// Lesen der erweiterten Parameter - Struktur
	// Diese Struktur enthaelt informationen auf die beim Baumaufbau von anderen Eintraegen zurueckgegriffen wird,
	// somit muss sie vor dem Aufbau geladen sein.
	if (extBegin.pulExtended) {
		
		extended = *(TExtended*)&modifyData.file->m_flashExtended[extBegin.pulExtended];
	}

	// Transpondertype und Zusatzparameter setzen
	if (modifyData.flags & SMDF_TRANSPONDER_TECHNIC) {

		switch(modifyData.technic) {
		case 0: { // Unique
			// Transpondertyp: Unique
			config.TRANSPONDER_TYPE = 0;

			// 32bit oder 40bit
			if (modifyData.flagsTranponder & TMDF_UNIQUE) {
			
				config.UNIQUE = modifyData.unique;
			}
			// Datenformat
			if (modifyData.flagsTranponder & TMDF_UNIQUE_MODE) {
			
				config.UNIQUE_Mode = modifyData.uniqueMode;
			}
			break;}
		case 1: { // Hitag2

			// Transpondertyp: Hitag2
			config.TRANSPONDER_TYPE = 0x81;

			// Passwort
			if (modifyData.flagsTranponder & TMDF_PASSWORD) {

				strcpy(extended.transponderPassword32, modifyData.hitagPassword);
			}

			// Wert fuer Increment
			if (modifyData.flagsTranponder & TMDF_AUTOINC_VALUE) {

				config.HITAG_AUTOINC_VALUE = modifyData.hitagAutoincValue;
			}

			// Block 1
			// Segment
			if (modifyData.flagsTranponder & TMDF_SEGMENT1) {

				config.TP_GLOBAL.tpSegment1 = modifyData.transponder.tpSegment1;
			}
			// Format oder Laenge
			if (modifyData.flagsTranponder & TMDF_MODE1_FORMAT || modifyData.flagsTranponder & TMDF_MODE1_LENGTH) {

				config.TP_GLOBAL.tpMode1 = modifyData.transponder.tpMode1;
			}
			// Initialisierungswert
			if (modifyData.flagsTranponder & TMDF_INIT_VALUE1) {

				config.HITAG_INITIAL_VALUE_1 = modifyData.hitagInitValue1;
			}
			// Flag fuer Autoinc
			if (modifyData.flagsTranponder & TMDF_AUTOINC_FLAG1) {

				if (modifyData.hitagAutoincFlags & 0x01) {
				
					config.HITAG_AUTOINC_FLAGS |= 0x01;
				} else {

					config.HITAG_AUTOINC_FLAGS &= ~0x01;
				}
			}
			// Block 2
			// Segment
			if (modifyData.flagsTranponder & TMDF_SEGMENT2) {

				config.TP_GLOBAL.tpSegment2 = modifyData.transponder.tpSegment2;
			}
			// Format oder Laenge
			if (modifyData.flagsTranponder & TMDF_MODE2_FORMAT || modifyData.flagsTranponder & TMDF_MODE2_LENGTH) {

				config.TP_GLOBAL.tpMode2 = modifyData.transponder.tpMode2;
			}
			// Initialisierungswert
			if (modifyData.flagsTranponder & TMDF_INIT_VALUE2) {

				config.HITAG_INITIAL_VALUE_2 = modifyData.hitagInitValue2;
			}
			// Flag fuer Autoinc
			if (modifyData.flagsTranponder & TMDF_AUTOINC_FLAG2) {

				if (modifyData.hitagAutoincFlags & 0x02) {
				
					config.HITAG_AUTOINC_FLAGS |= 0x02;
				} else {

					config.HITAG_AUTOINC_FLAGS &= ~0x02;
				}
			}
			// Block 3
			// Segment
			if (modifyData.flagsTranponder & TMDF_SEGMENT3) {

				config.TP_GLOBAL.tpSegment3 = modifyData.transponder.tpSegment3;
			}
			// Format oder Laenge
			if (modifyData.flagsTranponder & TMDF_MODE3_FORMAT || modifyData.flagsTranponder & TMDF_MODE3_LENGTH) {

				config.TP_GLOBAL.tpMode3 = modifyData.transponder.tpMode3;
			}
			// Initialisierungswert
			if (modifyData.flagsTranponder & TMDF_INIT_VALUE3) {

				config.HITAG_INITIAL_VALUE_3 = modifyData.hitagInitValue3;
			}
			// Flag fuer Autoinc
			if (modifyData.flagsTranponder & TMDF_AUTOINC_FLAG3) {

				if (modifyData.hitagAutoincFlags & 0x04) {
				
					config.HITAG_AUTOINC_FLAGS |= 0x04;
				} else {

					config.HITAG_AUTOINC_FLAGS &= ~0x04;
				}
			}

			break;}
		case 5: { // Mifare

			// Transpondertyp: Mifare
			if (config.TRANSPONDER_TYPE != 0xC5 && config.TRANSPONDER_TYPE != 0x85) {
			
				config.TRANSPONDER_TYPE = 0x85;
			}
			// Referenz auf Struktur des Configbereichs
			STransponderMifare &tpGlobal = *(STransponderMifare*)&config.TP_GLOBAL;

			// Wert fuer Seriennummer lesen
			if (modifyData.flagsTranponder & TMDF_SERIAL_NUMBER) {

				if (modifyData.serial_number == 1) {
				
					config.TRANSPONDER_TYPE |= 0x40;
				} else {

					config.TRANSPONDER_TYPE &= ~0x40;
				}
			}

			// Wert fuer Increment
			if (modifyData.flagsTranponder & TMDF_AUTOINC_VALUE) {

				config.HITAG_AUTOINC_VALUE = modifyData.hitagAutoincValue;
			}

			// Segment angegeben
			if (modifyData.flagsTranponder & TMDF_SECTOR) {
			
				tpGlobal.sectorBlock &= ~0xF0;
				tpGlobal.sectorBlock |= (modifyData.transponderMifare.sectorBlock & 0xF0);
			}
			

			// Block angegeben
			if (modifyData.flagsTranponder & TMDF_BLOCK) {
			
				tpGlobal.sectorBlock &= ~0x0F;
				tpGlobal.sectorBlock |= (modifyData.transponderMifare.sectorBlock & 0x0F);
			}

			// Modusvariable ggf. initialisieren
			if (modifyData.flagsTranponder & TMDF_MODE_FORMAT || modifyData.flagsTranponder & TMDF_MODE_LENGTH) {

				tpGlobal.mode |= BDTP_ACTIVE;
			}

			// Format in Modusvariable uebernehmen
			if (modifyData.flagsTranponder & TMDF_MODE_FORMAT) {
			
				tpGlobal.mode &= ~0x30;
				tpGlobal.mode |= (modifyData.transponderMifare.mode & 0x30);
			}

			// Format in Modusvariable uebernehmen
			if (modifyData.flagsTranponder & TMDF_MODE_LENGTH) {

				tpGlobal.mode &= ~0x0F;
				tpGlobal.mode |= (modifyData.transponderMifare.mode & 0x0F);
			}

			// Startposition uebernehmen
			if (modifyData.flagsTranponder & TMDF_POSITION) {
			
				tpGlobal.cut &= ~0xF0;
				tpGlobal.cut = (modifyData.transponderMifare.cut & 0xF0);
			}

			int i, len = strlen(modifyData.mifareInitalValue);
			// Initialisierungswert angegeben
			if (modifyData.flagsTranponder & TMDF_INIT_VALUE1) {
				switch (tpGlobal.mode & 0x30) {
			case 0: // Dez
				*(unsigned long*)&extended.transponderInitialValue = (unsigned long)atoi(modifyData.mifareInitalValue);
				break;
			case 1: // Hex
			case 3: // Hex gedreht
				for(i=0; i < len; i++) {

					if (!isxdigit((int)modifyData.mifareInitalValue[i])) {
						break;
					}
				}
				if (i == len) {
					*(unsigned long*)&extended.transponderInitialValue = HexToUINT32(modifyData.mifareInitalValue);
				} else {
					*(unsigned long*)&extended.transponderInitialValue = 0;
				}
				break;
			case 2: // ASCII
				memset(extended.transponderInitialValue, 0, sizeof(extended.transponderInitialValue));
				strcpy((char*)extended.transponderInitialValue, modifyData.mifareInitalValue);
				break;
				}
			}

			// Flag fuer Autoinc
			if (modifyData.flagsTranponder & TMDF_AUTOINC_FLAG1) {

				if (modifyData.hitagAutoincFlags & 0x01) {

					config.HITAG_AUTOINC_FLAGS |= 0x01;
				} else {

					config.HITAG_AUTOINC_FLAGS &= ~0x01;
				}
			}


			// Passwortgruppe 1
			// Passwortmodus angegeben
			if (modifyData.flagsPassword & PMDF_MODUS1) {
			
				extended.transponderPasswordFlag &= ~0x03;
				extended.transponderPasswordFlag |= (modifyData.passwordFlags & 0x03);
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYA0) {

				strcpy(extended.transponderPasswordA0, modifyData.mifarePasswordA0);
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYB0) {

				strcpy(extended.transponderPasswordB0, modifyData.mifarePasswordB0);
			}

			// Passwortgruppe 2
			// Passwortmodus angegeben
			if (modifyData.flagsPassword & PMDF_MODUS2) {
			
				extended.transponderPasswordFlag &= ~(0x03 << 2);
				extended.transponderPasswordFlag |= (modifyData.passwordFlags & (0x03 << 2));
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYA1) {

				strcpy(extended.transponderPasswordA1, modifyData.mifarePasswordA1);
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYB1) {

				strcpy(extended.transponderPasswordB1, modifyData.mifarePasswordB1);
			}

			// Passwortgruppe 3
			// Passwortmodus angegeben
			if (modifyData.flagsPassword & PMDF_MODUS3) {
			
				extended.transponderPasswordFlag &= ~(0x03 << 4);
				extended.transponderPasswordFlag |= (modifyData.passwordFlags & (0x03 << 4));
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYA2) {
				
				strcpy(extended.transponderPasswordA2, modifyData.mifarePasswordA2);
			}
			// Passwort angegeben
			if (modifyData.flagsPassword & PMDF_KEYB2) {

				strcpy(extended.transponderPasswordB2, modifyData.mifarePasswordB2);
			}
			break;}
		}
	}

	// Sicherheitslevel ersetzen?
	if (modifyData.flags & SMDF_FINGERPRINT_SECURITY_LEVEL) {
	
		extended.fingerprint.securityLevel = modifyData.securityLevel;
		extended.fingerprint.securityFRR = modifyData.securityFRR;
		extended.fingerprint.securityFAR = modifyData.securityFAR;
	}
	// Mindestqualitaet beim scannnen ersetzen?
	if (modifyData.flags & SMDF_FINGERPRINT_QUALITY) {

		extended.fingerprint.quality = modifyData.quality;
	}
	// Mindestanzahl der Minutiae beim scannen ersetzen?
	if (modifyData.flags & SMDF_FINGERPRINT_MINUTIAE_COUNT) {
	
		extended.fingerprint.minutiaeCount = modifyData.minutiaeCount;
	}

	// Schreiben der erweiterten Parameter - Struktur
	if (extBegin.pulExtended) {

		 *(TExtended*)&modifyData.file->m_flashExtended[extBegin.pulExtended] = extended;
	}

	unsigned long offs;
	// Eintraege hinzufuegen
	//#####################################################
	// Eintrag fuer Datensatzbeschreibung einbinden
	CDFItemBase *item = new CDFItemTableRecord();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	// Datensatzbeschreibungen lesen
	offs = *(unsigned long*)&modifyData.file->m_eep[P_DAT_STRUCT];
	if (offs == 0) {	// Alte Setup-Datei
		offs = *(unsigned long*)&modifyData.file->m_eep[P_VERSION]; 
	}
	ChangeOrderLong((unsigned char*)&offs);
	modifyData.offsFlash = offs;
	if (!item->Modify(modifyData)) return FALSE;
	
	// Eintrag fuer Listenbeschreibung einbinden
	item = new CDFItemTableList();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	// Listenbeschreibungen lesen
	offs = *(unsigned long*)&modifyData.file->m_eep[P_LIST_STRUCT];
	ChangeOrderLong((unsigned char*)&offs);
	modifyData.offsFlash = offs;
	if (!item->Modify(modifyData)) return FALSE;

	// Eintrag fuer Bedienung einbinden
	item = new CDFItemHandling();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	SGlobVar globVar = *(SGlobVar*)modifyData.file->m_flash;
	ChangeOrderLong((unsigned char*)&globVar.P);
	modifyData.offsFlash = globVar.P;
	if (!item->Modify(modifyData)) return FALSE;

	// Eintrag fuer Signalverarbeitung einbinden
	item = new CDFItemMDE();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	modifyData.offsFlash = 0;
	if (!item->Modify(modifyData)) return FALSE;

	// Eintrag fuer Zutrittskontrolle einbinden
	item = new CDFItemAccessControl();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	modifyData.offsFlash = 0;
	if (!item->Modify(modifyData)) return FALSE;

	// Eintrag fuer Zutrittskontrolle 2 einbinden
	item = new CDFItemAccessControl2();
	if (item == NULL) return FALSE;
	LinkEndChild(item);
	modifyData.offsFlash = 0;
	if (!item->Modify(modifyData)) return FALSE;

	return TRUE;
}
