/**
 * @file  DFFileDeviceZip.cpp
 * @brief Implementierung der Klasse CDFFileDeviceZip.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFFileDeviceZip.h"
#include <sys/types.h>
#include <sys/stat.h>

CDFFileDeviceZip::CDFFileDeviceZip()
{
	// Initialisieren
	Reset();
	// Fehlerobjekt zuruecksetzen
	ResetError();
}

CDFFileDeviceZip::~CDFFileDeviceZip()
{
}

void CDFFileDeviceZip::Reset()
{
	// Basis Reset
	CDFFileDevice::Reset();
}

void CDFFileDeviceZip::ResetError()
{
	// Basis Reset
	CDFFileDevice::ResetError();
	// Fehlertext loeschen
	memset(m_errorExtended, 0, sizeof(m_errorExtended));
}

int CDFFileDeviceZip::CreateZipFromDirectory(char *path, char *fileZIP, char *defaultDFL)
{
	int loaded = 0;

#ifdef _WIN32
	char szZipFile[260], szFile[260], szIniFile[260];
	char szSearchFile[260];
	long fp;
	struct _finddata_t finddata;
	char version[32], timestamp[20], comment[256];
	int fileCount = 0;
	char *md5 = NULL;

	// Fehlerobjekt zuruecksetzen
	ResetError();
	
	// Speicher initialisieren
	memset(szZipFile, 0, sizeof(szZipFile));
	memset(szFile, 0, sizeof(szFile));
	memset(szIniFile, 0, sizeof(szIniFile));
	memset(szSearchFile, 0, sizeof(szSearchFile));
	memset(version, 0, sizeof(version));
	memset(timestamp, 0, sizeof(timestamp));
	memset(comment, 0, sizeof(comment));

	// Je nach Angabe Basispfad mit bercksichtigen
	if (fileZIP[1] == ':') {

		strcpy(szZipFile, fileZIP);
	} else {

		sprintf(szZipFile, "%s\\%s", path, fileZIP);
	}

	sprintf(szIniFile, "%s\\%s", path, "SWI.ini");
	::remove(szIniFile); // Vorhandene Initialisierungsdatei loeschen

	sprintf(szSearchFile, "%s\\*.hex", path);
	// Passende Dateien fuer Muster?
	if ((fp = _findfirst(szSearchFile, &finddata)) != -1) {

		// Vorhandenes Archiv loeschen
		::remove(szZipFile); 
		// Zipdatei erzeugen
		HZIP hZip = NULL;
		if ((hZip = CreateZip(szZipFile, 0)) == NULL) {

			// Fehlertext erzeugen.
			m_errorState = ES_AS_TEXT;
			FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
			goto error_label;
		}

		// Builddatum, wenn nicht vorhanden, eintragen
		if (strlen(timestamp) != 19) {

			time_t ltime;
			struct tm *ltm;

			time(&ltime);
			ltm = localtime(&ltime);
			strftime(timestamp, sizeof(timestamp), "%Y-%m-%d %H:%M:%S", ltm);
		}

		// Alle Dateien durchlaufen.
		do {

			// Dateiname zusammenstellen
			sprintf(szFile, "%s\\%s", path, finddata.name);

			// Dateidaten laden
			Reset();
			if (!Load(szFile) || IsCrypt()) {
				
				Reset();
				m_errorState = ES_AS_TEXT;
				sprintf(m_errorText, "Fehler (%i) beim Ladeversuch der Datei [ %s ]", GetLastErrorState(), szFile);
				goto error_label;
			}

			// Ist die Softwareinformation in der Geraetedatei verfuegbar?
			if (GetHeaderSoftwareInfo().size) {

				// MD5 der Datei in Initialisierungsdatei uebernehmen
				char *md5 = NULL;
				if ((md5 = MD5File(szFile)) == NULL) {

					// Datei kann nicht uebernommen werden.
					m_errorState = ES_AS_TEXT;
					strcpy(m_errorText, "Erstellen des MD5(1) schlug fehl.");
					goto error_label;
				}
				
				m_iniFile.WriteString(finddata.name, "MD5", md5);
				free(md5);

				// Version zur Datei in Initialisierungsdatei uebernehmen.
				char tmp[0x100];
				memset(tmp, 0, sizeof(tmp));
				GetHeaderVersionFirmware(tmp);
				m_iniFile.WriteString(finddata.name, "Version", tmp);

				// Erste Version fuer Archiv halten.
				if (!strlen(version)) {

					strncpy(version, tmp, 11);
				}

				// Softwareinfo zur Datei in Initialisierungsdatei uebernehmen.
				memset(tmp, 0, sizeof(tmp));
				SWInfoStructToString(tmp, GetHeaderSoftwareInfo());
				m_iniFile.WriteString(finddata.name, "SWI", tmp);

				// Datei zu Archiv hinzufuegen
				if (ZipAdd(hZip, finddata.name, szFile) != ZR_OK) {
					// Fehler anfuegen
					m_errorState = ES_AS_TEXT;
					FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
					goto error_label;
				}

				fileCount++;
			}

		} while (_findnext(fp, &finddata) != -1);

		// Textdaten fuer Firmware verfuegbar?
		if (defaultDFL != NULL) {

			// MD5 der Datei in Initialisierungsdatei uebernehmen
			char *md5 = NULL;
			if ((md5 = MD5String(defaultDFL)) == NULL) {

				// Datei kann nicht uebernommen werden.
				m_errorState = ES_AS_TEXT;
				strcpy(m_errorText, "Erstellen des MD5(1) schlug fehl.");
				goto error_label;
			}

			m_iniFile.WriteString("Default.dfl", "MD5", md5);
			free(md5);

			// Datei mit Textdaten schreiben
			if (ZipAdd(hZip, "Default.dfl", defaultDFL, strlen(defaultDFL)) != ZR_OK) {
				// Fehler anfuegen
				m_errorState = ES_AS_TEXT;
				FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
				goto error_label;
			}
		}

		// Initialisierungsdatei schreiben
		m_iniFile.WriteIniFile(szIniFile);
		m_iniFile.CloseIniFile();
		// Datei zu Archiv hinzufuegen
		if (ZipAdd(hZip, "SWI.ini", szIniFile) != ZR_OK) {
			// Fehler anfuegen
			m_errorState = ES_AS_TEXT;
			FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
			goto error_label;
		}

		// MD5 der Initialisierungsdatei als Name fuer Sicherheitsinitialisierungsdatei erstellen.
		if ((md5 = MD5File(szIniFile)) == NULL) {

			// Datei kann nicht uebernommen werden.
			m_errorState = ES_AS_TEXT;
			strcpy(m_errorText, "Erstellen des MD5(2) schlug fehl.");
			goto error_label;
		}
		// Aufgenommene Initialisierungsdatei loeschen
		::remove(szIniFile);
		
		sprintf(szFile, "%s.ini", md5);
		// Speicher freigeben.
		free(md5);
		md5 = NULL;

		sprintf(szIniFile, "%s\\%s", path, szFile);
		::remove(szIniFile); // Vorhandene Initialisierungsdatei loeschen

		// Archivdaten eintragen
		m_iniFile.WriteInt("Archive", "FileDeviceVersion", sizeof(SDFFileDeviceHeader));
		m_iniFile.WriteString("Archive", "Timestamp", timestamp);
		m_iniFile.WriteString("Archive", "Version", version);
		m_iniFile.WriteInt("Archive", "Count", fileCount);

		// Initialisierungsdatei schreiben
		m_iniFile.WriteIniFile(szIniFile);
		m_iniFile.CloseIniFile();

		// Datei zu Archiv hinzufuegen
		if (ZipAdd(hZip, szFile, szIniFile) != ZR_OK) {
			// Fehlertext erzeugen.
			m_errorState = ES_AS_TEXT;
			FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
			goto error_label;
		}
		// Aufgenommene Initialisierungsdatei loeschen
		::remove(szIniFile);

		// Erstellen erfolgreich
		loaded = 1;
		// Kommentar des Archives setzen
		sprintf(comment, "Company:\tDatafox GmbH\nContent:\tFiles for MasterIV - devices.\nVersion:\t%s\nTime stamp:\t%s", version, timestamp);
		ZipSetComment(hZip, comment);
error_label:
		_findclose(fp);
		// Zipdatei schliessen
		CloseZip(hZip);
		if (md5) free(md5);
		m_iniFile.CloseIniFile();
	} else {

		// Fehler zuweisen
		m_errorState = ES_AS_TEXT;
		sprintf(m_errorText, "Keine Hex-Dateien im Pfad [ %s ] gefunden.", path);
	}

	// Reset der Daten durchfuehren
	Reset();
#endif

	return loaded;
}

int CDFFileDeviceZip::SearchInfosDir(char *path, const TDFHardwareInfo &hwInfo)
{
	int loaded = 0;
	char szFile[260];
	char szSearchFile[260];
#ifdef _WIN32
	long fp;
	struct _finddata_t finddata;

	sprintf(szSearchFile, "%s\\*.hex", path);
	// Passende Dateien fuer Muster?
	if ((fp = _findfirst(szSearchFile, &finddata)) != -1) {
		
		// Alle Dateien durchlaufen.
		do {

			// Dateiname zusammenstellen
			sprintf(szFile, "%s\\%s", path, finddata.name);
#else
	DIR *d;
	struct stat st;
	struct dirent *de;

	if ((d = opendir(path)) != NULL) {
		
		// Alle Dateien durchlaufen
		while((de = readdir(d)) != NULL) {
	
			// Dateiname zusammenstellen
			sprintf(szFile, "%s/%s", path, de->d_name);

			if (stat(szFile, &st) == -1) {
				break;
			}
			
			if (S_ISREG(st.st_mode)) {
				
				// Nur ".hex" gueltig.
				int pos = strlen(de->d_name) - 4;
				if (de->d_name[pos] != '.' || 
					!(de->d_name[pos + 1] == 'h' || de->d_name[pos + 1] == 'H') ||
					!(de->d_name[pos + 2] == 'e' || de->d_name[pos + 2] == 'E') ||
					!(de->d_name[pos + 3] == 'x' || de->d_name[pos + 3] == 'X') ) {
				
					continue;
				}

#endif	
			// Dateidaten laden
			Reset();
			if (!Load(szFile) || IsCrypt()) {
				
				Reset();
				continue;
			}

			// Ist die Softwareinformation in der Geraetedatei verfuegbar?
			if (GetHeaderSoftwareInfo().size) {
				// Pruefen ob die Datei in die vorliegende Hardware eingespielt werden kann.
				if (CompareInfos(hwInfo, GetHeaderSoftwareInfo())) {
					
					loaded = 1;
					break;
				}
			} else {
				continue;
			}
#ifdef _WIN32
		} while (_findnext(fp, &finddata) != -1);
		_findclose(fp);
	}
#else
				
			}
		} // while

		closedir(d);
	}
#endif

	// Reset bei fehlerhaftem laden
	if (!loaded) {

		Reset();

		m_errorState = ES_NO_FILE_MATCH;
		strcpy(m_errorText, ES_NO_FILE_MATCH_TEXT);
	}

	return loaded;
}

int CDFFileDeviceZip::CheckZip(char *fileZIP)
{
	int loaded = 0;
#ifdef _WIN32
	char path[260], szFile[260], szIniFile[260];
	char version[32], timestamp[20], tmp[256];
	int index, fileCount, fileDeviceVersion;
	ZIPENTRY ze;
	char *md5 = NULL;

	// Speicher initialisieren
	memset(path, 0, sizeof(path));
	memset(szFile, 0, sizeof(szFile));
	memset(szIniFile, 0, sizeof(szIniFile));
	memset(version, 0, sizeof(version));
	memset(timestamp, 0, sizeof(timestamp));
	memset(tmp, 0, sizeof(tmp));
	memset(m_errorText, 0, sizeof(m_errorText));

	HZIP hZip = NULL;
	if ((hZip = OpenZip(fileZIP, 0)) == NULL) {

		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		return 0;
	}

	// Suchen der Initialisierungsdatei der Softwareinfo
	if (ZR_OK != FindZipItem(hZip, "SWI.ini", false, &index, &ze)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// Basispfad ermitteln
	memset(path, 0, sizeof(path));
	GetTempPath(sizeof(path), path);

	// Dateiname zusammenstellen
	sprintf(szIniFile, "%s\\%s", path, ze.name);
	// Entzippen der Initialisierungsdatei
	if (ZR_OK != UnzipItem(hZip, index, szIniFile)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}
	// MD5 ueber die Initialisierungsdatei erstellen
	if ((md5 = MD5File(szIniFile)) == NULL) { 
		// Fehlertext zuweisen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Erzeugen des MD5 gescheitert.");
		goto error_label; 
	}

	// Dateiname zusammenstellen
	sprintf(szFile, "%s.ini", md5);
	// Suchen der Initialisierungsdatei der Sicherheit
	if (ZR_OK != FindZipItem(hZip, szFile, false, &index, &ze)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// Dateiname zusammenstellen
	sprintf(szFile, "%s\\%s.ini", path, md5);
	// Entzippen der Sicherheitsinitialisierungsdatei
	if (ZR_OK != UnzipItem(hZip, index, szFile)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}
	// Speicher freigeben.
	free(md5);
	md5 = NULL;

	// Informationen der Sicherheitsini laden
	if (!m_iniFile.OpenIniFile(szFile)) {
		// Fehlertext zuweisen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Oeffnen der Initialisierungsdatei ist gescheitert.");
		goto error_label;
	}

	// Version der enthaltenen Geraetedateien pruefen
	fileDeviceVersion = m_iniFile.ReadInt("Archive", "FileDeviceVersion", 0);
	if (fileDeviceVersion > sizeof(SDFFileDeviceHeader)) {

		// Fehlertext zuweisen. Die Software ist nicht aktuell genug fuer diese Geraetedateien.
		m_errorState = ES_VERSION;
		strcpy(m_errorText, ES_VERSION_TEXT);
		goto error_label;
	}

	// Anzahl der Dateien laden
	fileCount = m_iniFile.ReadInt("Archive", "Count", 0);
	// Version der Dateien laden
	strcpy(version, m_iniFile.ReadString("Archive", "Version", ""));
	// Zeitstempel des Archives laden
	strcpy(timestamp, m_iniFile.ReadString("Archive", "Timestamp", ""));
	if (!strlen(timestamp)) {
		// Zeitstempel anhand des Erstellungsdatums setzen
		struct stat st;
		if (stat((char*)(LPCTSTR)fileZIP, &st) == 0) {
			
			struct tm *ptm = localtime(&st.st_ctime);
			
			sprintf(timestamp, "%02i%02i-%02i-%02i %02i:%02i:%02i", 
				(ptm->tm_year + 1900) / 100, (ptm->tm_year + 1900) % 100, ptm->tm_mon + 1, ptm->tm_mday,
				ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
		}
	}

	// Version halten
	strcpy(m_version, version);
	// Zeitstempel halten
	strcpy(m_timestamp, timestamp);
	// Als geladen markieren
	loaded = 1;

error_label:
	// Schliessen der Initialisierungsdatei
	m_iniFile.CloseIniFile();
	// Dateien loeschen
	::remove(szFile);
	::remove(szIniFile);
	memset(szFile, 0, sizeof(szFile));
	memset(szIniFile, 0, sizeof(szIniFile));
	// Ziparchiv schliessen
	CloseZip(hZip);	
	if (md5) free(md5);
#endif
	return loaded;
}

int CDFFileDeviceZip::UnzipDefaultDFL(char *fileZIP, char *fileDefaultDFL)
{
	int loaded = 0;
#ifdef _WIN32
	char path[260], szFile[260], szIniFile[260], szFileDefaultDFL[260];
	char tmp[256];
	int index;
	ZIPENTRY ze;
	char *md5 = NULL;

	// Speicher initialisieren
	memset(path, 0, sizeof(path));
	memset(szFile, 0, sizeof(szFile));
	memset(szIniFile, 0, sizeof(szIniFile));
	memset(szFileDefaultDFL, 0, sizeof(szFileDefaultDFL));
	memset(tmp, 0, sizeof(tmp));
	memset(m_errorText, 0, sizeof(m_errorText));

	HZIP hZip = NULL;
	if ((hZip = OpenZip(fileZIP, 0)) == NULL) {

		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		return 0;
	}

	// Suchen der Initialisierungsdatei der Softwareinfo
	if (ZR_OK != FindZipItem(hZip, "SWI.ini", false, &index, &ze)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// Basispfad ermitteln
	memset(path, 0, sizeof(path));
	GetTempPath(sizeof(path), path);

	// Dateiname zusammenstellen
	sprintf(szIniFile, "%s\\%s", path, ze.name);
	// Entzippen der Initialisierungsdatei
	if (ZR_OK != UnzipItem(hZip, index, szIniFile)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// MD5 ueber die Initialisierungsdatei erstellen
	if ((md5 = MD5File(szIniFile)) == NULL) { 
		// Fehlertext zuweisen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Erzeugen des MD5 gescheitert.");
		goto error_label; 
	}

	// Dateiname der Sicherheitsini zusammenstellen
	sprintf(szFile, "%s.ini", md5);
	// Suchen der Initialisierungsdatei der Sicherheit
	if (ZR_OK != FindZipItem(hZip, szFile, false, &index, &ze)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}
	// Speicher freigeben.
	free(md5);
	md5 = NULL;

	// Oeffnen der Initialisierungsdatei fuer die Softwareinfo
	if (!m_iniFile.OpenIniFile(szIniFile)) {
		// Fehlertext setzen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Oeffnen der Initialisierungsdatei(2) fehlgeschlagen.");
		goto error_label;
	}

	// MD5 der Textdatedatei pruefen
	// Version der Dateien laden
	strcpy(tmp, m_iniFile.ReadString("Default.dfl", "MD5", ""));
	if (!strlen(tmp)) {
		// Fehlertext setzen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Ermittlung des MD5 zur Default.dfl fehlgeschlagen.");
		goto error_label;
	}

	// Suchen der Textdatendatei Default.dfl
	if (ZR_OK != FindZipItem(hZip, "Default.dfl", false, &index, &ze)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// Dateiname zusammenstellen
	sprintf(szFileDefaultDFL, "%s\\tmp%s", path, ze.name);
	// Entzippen der Textdatendatei
	if (ZR_OK != UnzipItem(hZip, index, szFileDefaultDFL)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// MD5 ueber die Textdatendatei erstellen
	if ((md5 = MD5File(szFileDefaultDFL)) == NULL) { 
		// Fehlertext zuweisen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Erzeugen des MD5 gescheitert.");
		goto error_label; 
	}

	if (strcmp(tmp, md5) != 0) {
		// Fehlertext zuweisen.
		m_errorState = ES_AS_TEXT;
		strcpy(m_errorText, "Prfung des MD5 der Default.dfl gescheitert.");
		goto error_label; 
	}

	// Kopie der Datei an gewuenschte lokation vornehmen
	if (ZR_OK != UnzipItem(hZip, index, fileDefaultDFL)) {
		// Fehlertext erzeugen.
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, m_errorText, sizeof(m_errorText));
		goto error_label;
	}

	// Speicher freigeben.
	free(md5);
	md5 = NULL;

error_label:
	// Inidatei schliessen
	m_iniFile.CloseIniFile();
	// Dateien loeschen
	::remove(szIniFile);
	::remove(szFileDefaultDFL);
	// Ziparchiv schliessen
	CloseZip(hZip);	
	if (md5) free(md5);
#endif
	return loaded;
}

int CDFFileDeviceZip::SearchInfosZip(char *fileZIP, const TDFHardwareInfo &hwInfo)
{
	int loaded = 0;
#ifdef _WIN32
	char path[260], szFile[260], szIniFile[260];
	char version[32], timestamp[20], tmp[256], 
		errorText[256], errorExtended[sizeof(m_errorExtended)];
	int i, index, fileCount, fileDeviceVersion;
	ZIPENTRY zeZIP, ze;
	char *md5 = NULL;

	// Speicher initialisieren
	memset(path, 0, sizeof(path));
	memset(szFile, 0, sizeof(szFile));
	memset(szIniFile, 0, sizeof(szIniFile));
	memset(version, 0, sizeof(version));
	memset(timestamp, 0, sizeof(timestamp));
	memset(tmp, 0, sizeof(tmp));
//	memset(m_errorText, 0, sizeof(m_errorText));
	memset(errorText, 0, sizeof(errorText));
	memset(errorExtended, 0, sizeof(errorExtended));

	// Fehler zuruecksetzen
	ResetError();

	HZIP hZip = NULL;
	if ((hZip = OpenZip(fileZIP, 0)) == NULL) {

		// Fehler anfuegen
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		strcpy(m_errorText, errorText);
		strcat(fileZIP, ":[");
		strcat(fileZIP, errorText);
		strcat(fileZIP, "]");
		return 0;
	}

	// Suchen der Initialisierungsdatei der Softwareinfo
	if (ZR_OK != FindZipItem(hZip, "SWI.ini", false, &index, &ze)) {
		// Fehlertext ermitteln
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		goto error_label;
	}

	// Basispfad ermitteln
	memset(path, 0, sizeof(path));
	GetTempPath(sizeof(path), path);

	// Dateiname zusammenstellen
	sprintf(szIniFile, "%s\\%s", path, ze.name);
	// Entzippen der Initialisierungsdatei
	if (ZR_OK != UnzipItem(hZip, index, szIniFile)) {
		// Fehlertext ermitteln
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		goto error_label;
	}
	// MD5 ueber die Initialisierungsdatei erstellen
	if ((md5 = MD5File(szIniFile)) == NULL) { 
		// Fehlertext setzen.
		m_errorState = ES_AS_TEXT;
		strcpy(errorText, "Erzeugen des MD5(1) fehlgeschlagen.");
		goto error_label; 
	}

	// Dateiname zusammenstellen
	sprintf(szFile, "%s.ini", md5);
	// Suchen der Initialisierungsdatei der Sicherheit
	if (ZR_OK != FindZipItem(hZip, szFile, false, &index, &ze)) {
		// Fehlertext ermitteln
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		goto error_label;
	}

	// Dateiname zusammenstellen
	sprintf(szFile, "%s\\%s.ini", path, md5);
	// Entzippen der Sicherheitsinitialisierungsdatei
	if (ZR_OK != UnzipItem(hZip, index, szFile)) {
		// Fehlertext ermitteln
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		goto error_label;
	}
	// Speicher freigeben.
	free(md5);
	md5 = NULL;

	// Informationen der Sicherheitsini laden
	if (!m_iniFile.OpenIniFile(szFile)) {
		// Fehlertext setzen.
		m_errorState = ES_AS_TEXT;
		strcpy(errorText, "Oeffnen der Initialisierungsdatei(1) fehlgeschlagen.");
		goto error_label;
	}

	// Version der enthaltenen Geraetedateien pruefen
	fileDeviceVersion = m_iniFile.ReadInt("Archive", "FileDeviceVersion", 0);
	if (fileDeviceVersion > sizeof(SDFFileDeviceHeader)) {

		// Fehlertext zuweisen. Die Software ist nicht aktuell genug fuer diese Geraetedateien.
		m_errorState = ES_VERSION;
		strcpy(m_errorText, ES_VERSION_TEXT);
		goto error_label;
	}

	// Anzahl der Dateien laden
	fileCount = m_iniFile.ReadInt("Archive", "Count", 0);
	// Version der Dateien laden
	strcpy(version, m_iniFile.ReadString("Archive", "Version", ""));
	// Zeitstempel des Archives laden
	strcpy(timestamp, m_iniFile.ReadString("Archive", "Timestamp", ""));
	if (!strlen(timestamp)) {
		// Zeitstempel anhand des Erstellungsdatums setzen
		struct stat st;
		if (stat((char*)(LPCTSTR)fileZIP, &st) == 0) {
			
			struct tm *ptm = localtime(&st.st_ctime);
			
			sprintf(timestamp, "%02i%02i-%02i-%02i %02i:%02i:%02i", 
				(ptm->tm_year + 1900) / 100, (ptm->tm_year + 1900) % 100, ptm->tm_mon + 1, ptm->tm_mday,
				ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
		}
	}
	// Schliessen der Initialisierungsdatei
	m_iniFile.CloseIniFile();
	// Datei loeschen
	::remove(szFile);
	memset(szFile, 0, sizeof(szFile));

	// Informationen zum Archiv ermitteln
	if (ZR_OK != GetZipItem(hZip, -1, &zeZIP)) {
		// Fehlertext ermitteln
		m_errorState = ES_AS_TEXT;
		FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
		goto error_label;
	}

	// Oeffnen der Initialisierungsdatei fuer die Softwareinfo
	if (!m_iniFile.OpenIniFile(szIniFile)) {
		// Fehlertext setzen.
		m_errorState = ES_AS_TEXT;
		strcpy(errorText, "Oeffnen der Initialisierungsdatei(2) fehlgeschlagen.");
		goto error_label;
	}

	// Passende Dateien suchen
	for (i=0; i < zeZIP.index; i++) {

		if (ZR_OK != GetZipItem(hZip, i, &ze)) {
			// Fehlertext ermitteln
			m_errorState = ES_AS_TEXT;
			FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
			break;
		}

		// Ist kein Ordner?
		if (ze.attr & FILE_ATTRIBUTE_DIRECTORY) {

			continue;
		}
		// Nur ".hex" gueltig.
		int pos = strlen(ze.name) - 4;
		if (ze.name[pos] != '.' || 
			!(ze.name[pos + 1] == 'h' || ze.name[pos + 1] == 'H') ||
			!(ze.name[pos + 2] == 'e' || ze.name[pos + 2] == 'E') ||
			!(ze.name[pos + 3] == 'x' || ze.name[pos + 3] == 'X') ) {
		
			continue;
		}

		// Laden der Softwareinfo
		strcpy(tmp, m_iniFile.ReadString(ze.name, "SWI", ""));
		if (!strlen(tmp) || tmp[0] != '#') {
			// Fehlertext setzen.
			m_errorState = ES_AS_TEXT;
			strcpy(errorText, "Lesen der Softwareinfo(1) fehlgeschlagen.");
			goto error_label;
		}
		
		// Die Softwareinfo in eine Struktur wandeln
		SDFSoftwareInfo swInfo = SWInfoStringToStruct(tmp);
		if (!swInfo.size) {
			// Fehlertext setzen.
			m_errorState = ES_AS_TEXT;
			strcpy(errorText, "Lesen der Softwareinfo(2) fehlgeschlagen.");
			goto error_label;
		}

		// Pruefen ob die Datei in die vorliegende Hardware eingespielt werden kann.
		if (!CompareInfos(hwInfo, swInfo)) {

			if ((10 + strlen(errorExtended) + strlen(ze.name) + strlen(m_errorText)) < sizeof(errorExtended)) {

				strcat(errorExtended, ze.name);
				strcat(errorExtended, ":[");
				strcat(errorExtended, m_errorText);
				strcat(errorExtended, "]\n");
			}

			// Naechste Datei pruefen
			continue;
		}

		// Dateiname zusammenstellen
		sprintf(szFile, "%s\\%s", path, ze.name);
		// Entzippen der einzuspielenden Datei
		if (ZR_OK != UnzipItem(hZip, i, szFile)) {
			// Fehlertext ermitteln
			m_errorState = ES_AS_TEXT;
			FormatZipMessage(ZR_RECENT, errorText, sizeof(errorText));
			goto error_label;
		}
		// MD5 ueber die Initialisierungsdatei erstellen
		if ((md5 = MD5File(szFile)) == NULL) { 
			// Fehlertext setzen.
			m_errorState = ES_AS_TEXT;
			strcpy(errorText, "Erzeugen des MD5(2) fehlgeschlagen.");
			goto error_label; 
		}

		// MD5 der Datei aus Ini lesen		
		strcpy(tmp, m_iniFile.ReadString(ze.name, "MD5", ""));
		if (!strlen(tmp)) {
			// Keine Angabe vorhanden.
			continue;
		}
		if (strcmp(tmp, md5) != 0) {
			// Datei ist korrupt.
			continue;
		}
		// Speicher freigeben.
		free(md5);
		md5 = NULL;

		// Dateidaten laden
		Reset();
		if (!Load(szFile) || IsCrypt()) {
			// Fehlertext setzen.
			m_errorState = ES_AS_TEXT;
			sprintf(errorText, "Laden der Geraetedatei fehlgeschlagen (%i).", GetLastErrorState());
			goto error_label;
		} else if (GetHeaderSoftwareInfo().size) {

			// Den Namen der gewaehlten Datei fuer Log anfuegen
			strcat(fileZIP, "\\");
			strcat(fileZIP, ze.name);
			// Version halten
			strcpy(m_version, version);
			// Zeitstempel halten
			strcpy(m_timestamp, timestamp);
			// Ist die Softwareinformation in der Geraetedatei verfuegbar?
			loaded = 1;
			break;
		} 
	}

error_label:
	// Ziparchiv schliessen
	CloseZip(hZip);	
	if (md5) free(md5);
	// Inidatei schliessen
	m_iniFile.CloseIniFile();
	// Entzippte Dateien loeschen
	if (strlen(szIniFile)) ::remove(szIniFile);
	if (strlen(szFile)) ::remove(szFile);

	// Reset bei fehlerhaftem laden
	if (m_errorState == ES_NO && !loaded) {

		Reset();

		// Es wurde keine passende Datei gefunden.
		m_errorState = ES_NO_FILE_MATCH;
		strcpy(errorText, ES_NO_FILE_MATCH_TEXT);
		// Fehlertext der einzelnen Dateien uebernehmen.
		strcpy(m_errorExtended, errorExtended);
	}

	// Fehlertext uebernehmen
	if (strlen(errorText)) {

		strcpy(m_errorText, errorText);

		strcat(fileZIP, ":[");
		strcat(fileZIP, errorText);
		strcat(fileZIP, "]");
	}
#endif

	return loaded;
}

