/**
 * @file  DFFileDeviceZip.h
 * @brief Schnittstelle fuer die Klasse CDFFileDeviceZip.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFFILEDEVICEZIP_H_
#define _DFFILEDEVICEZIP_H_

#pragma warning(disable:4786)

// Basisklasse einbinden
#include "DFFileDevice.h"

// Fehlerzustaende
#include "../Common/ErrorState.h"
// Fuer die Verwendung von TDFHardwareInfo
#include "../Common/AES.h"

#ifdef _WIN32
#include <direct.h>
#include <io.h>

#include <time.h>
#include <string>
#include <set>
#include <functional>

using namespace std;
typedef greater<string> sf;
typedef multiset<string, sf> ms;
typedef ms::iterator msIterator;

// Initialisierungsdatei des Zip-Archiv
#include "IniFile.h"
// MD5 Berechnung 
#include "../Common/md5.h"
// Routinen fuer die zip Unterstuetzung
#include "../zip_utils/zip.h"
using namespace Zip;

#else

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#endif

class CDFFileDeviceZip : public CDFFileDevice
{
public:
	CDFFileDeviceZip();
	virtual ~CDFFileDeviceZip();

	void Reset();
	
	inline int GetLastErrorState() { return m_errorState; };

	int CreateZipFromDirectory(char *path, char *fileZIP, char *defaultLng);

	int CheckZip(char *fileZIP);
	int SearchInfosDir(char *path, const TDFHardwareInfo &hwInfo);
	int SearchInfosZip(char *fileZIP, const TDFHardwareInfo &hwInfo);

	int UnzipDefaultDFL(char *fileZIP, char *fileDefaultDFL);

	void ResetError();

protected:
#ifdef _WIN32
	CIniFile m_iniFile;
public:
	char m_version[32];
	char m_timestamp[20];
#endif
public:
	char m_errorExtended[4096];
};

#endif
