/**
 * @file  DFFileDisplay.cpp
 * @brief Implementierung der Klasse CDFFileDisplay.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFFileDisplay.h"
#include <memory.h>
#include <stdio.h>

CDFFileDisplay::CDFFileDisplay()
{
	// CRC32 Initialisierung
	InitCRC32Table();

	// Ausgangseinstellung
	ResetError();
	Reset();
}

CDFFileDisplay::~CDFFileDisplay()
{
}

void CDFFileDisplay::Reset()
{
	// Basis Reset
	CDFFileIntelHex::Reset();

	// Member initialisieren
	
	// Groessen setzen
	m_sizeData   = 0;
	m_sizeFont   = 0;
	m_sizeGrafic = 0;

	// Speicher fuer Zeichensatz- und Grafikdaten loeschen
	memset(m_memFont, 0x00, MAX_MEM_FONT);
	memset(m_memGrafic, 0x00, MAX_MEM_GRAFIC);

	// Hilfsstrukturen aufsetzen
	m_pFontNode   = (FONTNODE*)m_memFont;
	m_pFontNode->version = 1;
	m_pFontNode->cnt     = 0;
	m_pFontNode->temp[0] = 0;
	m_pFontNode->temp[1] = 0;
	m_pFontNode->ptrFontArray = sizeof(FONTNODE);

	m_pGraficNode = (GRAFICNODE*)m_memGrafic;
	m_pGraficNode->version = 1;
	m_pGraficNode->cnt     = 0;
	m_pGraficNode->temp[0] = 0;
	m_pGraficNode->temp[1] = 0;
	m_pGraficNode->ptrGraficArray = sizeof(GRAFICNODE);
}

void CDFFileDisplay::Default(int iFontCount, int iGraficCount)
{
	// Variablen fuer Groessenangabe
	m_sizeData   = 0;
	
	DefaultFont(iFontCount);
	DefaultGrafic(iGraficCount);
}

void CDFFileDisplay::DefaultFont(int iCount)
{
	int i, j;

	// Variablen fuer Groessenangabe
	m_sizeFont   = 0;

	// Speicher fuer Zeichensatz- und Grafikdaten loeschen
	memset(m_memFont, 0x00, MAX_MEM_FONT);

	// 1.
	// Zeichensaetze init
	// Grundstruktur aufsetzen
	m_pFontNode->version = 1;
	m_pFontNode->cnt     = iCount;
	m_pFontNode->temp[0] = 0;
	m_pFontNode->temp[1] = 0;
	m_pFontNode->ptrFontArray = sizeof(FONTNODE);

	// Zeigerarray und Zeichensatzstrukturzeiger initialisieren
	i = m_pFontNode->cnt;
	for (j=0; j < i; j++) {
		
		// Zeigereintrag auf Zeichensatzstruktur
		*(unsigned long*)&m_memFont[m_pFontNode->ptrFontArray + j * sizeof(unsigned long)] = 
			m_pFontNode->ptrFontArray + i * sizeof(unsigned long) + j * sizeof(FONT);

		m_pFont[j] = (FONT*)&m_memFont[m_pFontNode->ptrFontArray + i * sizeof(unsigned long) + j * sizeof(FONT)];
	}

}

void CDFFileDisplay::DefaultGrafic(int iCount)
{
	int i, j;

	// Variablen fuer Groessenangabe
	m_sizeGrafic = 0;

	// Speicher fuer Grafikdaten loeschen
	memset(m_memGrafic, 0x00, MAX_MEM_GRAFIC);

	// 2.
	// Grafiken init
	// Grundstruktur aufsetzen
	m_pGraficNode->version = 1;
	m_pGraficNode->cnt     = iCount;
	m_pGraficNode->temp[0] = 0;
	m_pGraficNode->temp[1] = 0;
	m_pGraficNode->ptrGraficArray = sizeof(GRAFICNODE);

	// Zeigerarray und Grafikstrukturzeiger initialisieren
	i = m_pGraficNode->cnt;
	for (j=0; j < i; j++) {
		
		// Zeigereintrag auf Grafikstruktur
		*(unsigned long*)&m_memGrafic[m_pGraficNode->ptrGraficArray + j * sizeof(unsigned long)] = 
			m_pGraficNode->ptrGraficArray + i * sizeof(unsigned long) + j * sizeof(GRAFIC);

		m_pGrafic[j] = (GRAFIC*)&m_memGrafic[m_pGraficNode->ptrGraficArray + i * sizeof(unsigned long) + j * sizeof(GRAFIC)];
	}

}

void CDFFileDisplay::FontCorrect(int iNum) 
{
	int j = iNum;

	// Leerzeichen ist so breit wie Doppelpunkt
	m_memFont[m_pFont[j]->ptrCharacter + (m_memFont[m_pFont[j]->ptrTable + (unsigned char)' '] - 1) * m_pFont[j]->bytesPerChar] =
		m_memFont[m_pFont[j]->ptrCharacter + (m_memFont[m_pFont[j]->ptrTable + (unsigned char)':'] - 1) * m_pFont[j]->bytesPerChar];

	// 64 Pixel - Zeichensatz
	if (j == 2) {

		char q;
		int iWidth = 0;
		
		// Suche breitestes Zeichen
		for (q='0'; q <= '9'; q++) {
			if(m_memFont[m_pFont[j]->ptrCharacter + (m_memFont[m_pFont[j]->ptrTable + (unsigned char)q] - 1) * m_pFont[j]->bytesPerChar] > iWidth) {

				iWidth = m_memFont[m_pFont[j]->ptrCharacter + (m_memFont[m_pFont[j]->ptrTable + (unsigned char)q] - 1) * m_pFont[j]->bytesPerChar];
			}
		}
		// Alle Zeichen so breit wie breitestes
		for (q='0'; q <= '9'; q++) {
			
			m_memFont[m_pFont[j]->ptrCharacter + (m_memFont[m_pFont[j]->ptrTable + (unsigned char)q] - 1) * m_pFont[j]->bytesPerChar] = iWidth;
		}
	}

	// Neuberechnung des 32bit CRC ueber Tabelle und Zeichensatzdaten. (muessen immer direkt hintereinander im Speicher liegen)
	m_pFont[j]->crc  = GetCRC32(&m_memFont[m_pFont[j]->ptrTable], 0x100 + m_pFont[j]->size);
}

int CDFFileDisplay::Load(char *fileName) 
{
	// Fehlerobjekt zuruecksetzen
	ResetError();

	// Umgebung zuruecksetzen
	Reset();

	// Laden der Zeichensatzdaten aus Hexfile
	if (!LoadFont(fileName)) {

		return 0;
	}

	// Laden der Grafikdaten aus Hexfile
	if (!LoadGrafic(fileName)) {

		return 0;
	}

	// Gesamtdatenbyte
	m_sizeData = m_sizeFont + m_sizeGrafic;

	return 1;
}

int CDFFileDisplay::LoadFont(char *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			address = GetAddress(data);
			type    = GetType(data);

			switch (type) {

			case INTELHEX_TYPE_DISPLAY_HEADER:
				break;

			case INTELHEX_TYPE_DISPLAY_FONT:
				m_sizeFont += GetData(data, &m_memFont[address]);
				break;

			case INTELHEX_TYPE_DISPLAY_GRAFIC:
				break;

			case INTELHEX_TYPE_EOF:
				break;
			}
		}
	}
	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeFont) {
		
		int i, j;

		// 1.
		// Zeichensaetze pruefen

		// Zeigerarray und Zeichensatzstrukturzeiger initialisieren
		i = m_pFontNode->cnt;
		if (i > MAX_FONT) {

			m_errorState = ES_FORMAT;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FORMAT_TEXT);
			return 0;
		}

		for (j=0; j < i; j++) {
			
			m_pFont[j] = (FONT*)&m_memFont[m_pFontNode->ptrFontArray + i * sizeof(unsigned long) + j * sizeof(FONT)];
		}

		// CRC32 pruefen
		for (j=0; j < i; j++) {
			
			if (m_pFont[j]->crc != GetCRC32(&m_memFont[m_pFont[j]->ptrTable], 0x100 + m_pFont[j]->size)) {
				
				m_errorState = ES_CRC_FAIL;
				sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_CRC_FAIL_TEXT);
				return 0;
			}
		}

	} else {

		m_errorState = ES_NO_DATA;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_NO_DATA_TEXT);
		return 0;
	}

	return 1;
}

int CDFFileDisplay::LoadGrafic(char *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			address = GetAddress(data);
			type    = GetType(data);

			switch (type) {

			case INTELHEX_TYPE_DISPLAY_HEADER:
				break;

			case INTELHEX_TYPE_DISPLAY_FONT:
				break;

			case INTELHEX_TYPE_DISPLAY_GRAFIC:
				m_sizeGrafic += GetData(data, &m_memGrafic[address]);
				break;

			case INTELHEX_TYPE_EOF:
				break;
			}
		}
	}
	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeGrafic) {
		
		int i, j;

		// 2.
		// Grafiken pruefen

		// Anzahl der Grafiken setzen
		i = m_pGraficNode->cnt; 

		if (i > MAX_GRAFIC) {

			m_errorState = ES_FORMAT;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FORMAT_TEXT);
			return 0;
		}

		// Zeigerarray und Grafikstrukturzeiger initialisieren
		for (j=0; j < i; j++) {
			
			m_pGrafic[j] = (GRAFIC*)&m_memGrafic[m_pGraficNode->ptrGraficArray + i * sizeof(unsigned long) + j * sizeof(GRAFIC)];
		}

		// CRC32 pruefen
		unsigned char ucByteCount;
		for (j=0; j < i; j++) {
	
			ucByteCount = ((m_pGrafic[j]->width - 1) / 8) + 1;

			if (m_pGrafic[j]->crc != GetCRC32(&m_memGrafic[m_pGrafic[j]->ptrGrafic], ucByteCount * m_pGrafic[j]->height)) {

				m_errorState = ES_CRC_FAIL;
				sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_CRC_FAIL_TEXT);
				return 0;
			}
		}

	} else {

		m_errorState = ES_NO_DATA;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_NO_DATA_TEXT);
		return 0;
	}

	return 1;
}

int CDFFileDisplay::Save(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	// vorhandene Datei anlegen
	if (!append) {
		if ((fp = fopen(fileName, "w")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}
		fclose(fp);
	}

	// Daten der Zeichensaetze sichern
	if (!SaveFont(fileName, 1)) {

		return 0;
	}

	// Daten der Grafiken sichern
	if (!SaveGrafic(fileName, 1)) {

		return 0;
	}

	if (!append) {
		// Endezeile in Datei eintragen
		if ((fp = fopen(fileName, "a")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
		fclose(fp);
	}

	return 1;
}

int CDFFileDisplay::SaveFont(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten der Zeichensaetze eintragen
	unsigned long ulPos;
	for (ulPos=0; ulPos < m_sizeFont; ulPos += INTELHEX_DATA_COUNT) {

		if (!MakeTypedDataLine((unsigned short)ulPos, INTELHEX_TYPE_DISPLAY_FONT, 
			&m_memFont[ulPos], (unsigned char)(((ulPos + INTELHEX_DATA_COUNT) < m_sizeFont) ? INTELHEX_DATA_COUNT : m_sizeFont - ulPos), szData)) {
			
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

int CDFFileDisplay::SaveGrafic(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten der Bilder eintragen
	unsigned long ulPos;
	for (ulPos=0; ulPos < m_sizeGrafic; ulPos += INTELHEX_DATA_COUNT) {

		if (!MakeTypedDataLine((unsigned short)ulPos, INTELHEX_TYPE_DISPLAY_GRAFIC, 
			&m_memGrafic[ulPos], (unsigned char)(((ulPos + INTELHEX_DATA_COUNT) < m_sizeGrafic) ? INTELHEX_DATA_COUNT : m_sizeGrafic - ulPos), szData)) {
			
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

// Textdaten als letzten Zeichensatz anfuegen.
int CDFFileDisplay::AppendSegmentText(unsigned char *buf, unsigned long size) 
{
	unsigned char *memFont;
	// Verwaltungsstrukturen fuer Zeichensatz
	FONTNODE *pFontNode;
	FONT     *pFont[MAX_FONT];
	TEXTDATA *pText = (TEXTDATA*)buf; // Ist TEXTDATA hat aber den gleichen Aufbau wie FONT
	unsigned long memFontSize;
	int i = 0;

	// Sind bereits Zeichensaetze angegeben?
	if (m_sizeFont) {

		//###################################
		// Als letzten Zeichensatz anfuegen

		// Speichergroesse von Zeichensaetzen berechnen
		memFontSize = m_pFont[m_pFontNode->cnt - 1]->ptrCharacter + m_pFont[m_pFontNode->cnt - 1]->size;
		// Kopie der Bereits vorhandenen Zeichensatzdaten erstellen
		memFont = new unsigned char[memFontSize];
		// Speicher allokation nicht erfolgreich?
		if (memFont == NULL) {
			m_errorState = ES_ALLOC_MEM;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_ALLOC_MEM_TEXT);
			return 0;
		}

		// Speicher kopieren
		memcpy(memFont, m_memFont, memFontSize);

		pFontNode = (FONTNODE*)memFont;
		// Kontrollstrukturen initialisieren
		for (i=0; i < pFontNode->cnt; i++) {

			pFont[i] = (FONT*)&memFont[pFontNode->ptrFontArray + pFontNode->cnt*sizeof(unsigned long) + i * sizeof(FONT)];
		}

		// Zeichensaetze initialisieren
		DefaultFont(pFontNode->cnt + 1);

		// Strukturen der Bestehende Zeichensaetze uebernehmen
		for (i=0; i < pFontNode->cnt; i++) {

			*m_pFont[i] = *pFont[i];
		}
		// Neuen Textblock anfuegen
		*m_pFont[m_pFontNode->cnt - 1] = *(FONT*)buf;

		// Erstes Freies Datenbyte
		m_pFont[0]->ptrTable = m_pFontNode->ptrFontArray + m_pFontNode->cnt*sizeof(unsigned long) + m_pFontNode->cnt*sizeof(FONT);
		m_pFont[0]->ptrCharacter = m_pFont[0]->ptrTable + 0x100;;

		// Daten der bestehenden Zeichensaetze uebernehmen
		for (i=0; i < pFontNode->cnt; i++) {

			memcpy(&m_memFont[m_pFont[i]->ptrTable], &memFont[pFont[i]->ptrTable], 0x100);
			memcpy(&m_memFont[m_pFont[i]->ptrCharacter], &memFont[pFont[i]->ptrCharacter], pFont[i]->size);
			
			// Naechsten freien Speicher zuweisen
			m_pFont[i + 1]->ptrTable = m_pFont[i]->ptrCharacter + m_pFont[i]->size;
			m_pFont[i + 1]->ptrCharacter = m_pFont[i + 1]->ptrTable + 0x100;;
		}

		// Neuen Block anfuegen
		memcpy(&m_memFont[m_pFont[i]->ptrTable], &buf[pText->ptrIDTable], pText->cnt*sizeof(unsigned short));
		m_pFont[i]->ptrCharacter = m_pFont[i]->ptrTable + pText->cnt*sizeof(unsigned short);
		memcpy(&m_memFont[m_pFont[i]->ptrCharacter], &buf[pText->ptrTextdata], pText->size);

		// Speicher wieder freigeben
		delete[] memFont;

		// Groessen anpassen
		m_sizeFont = m_pFont[m_pFontNode->cnt - 1]->ptrCharacter + m_pFont[m_pFontNode->cnt - 1]->size;
		m_sizeData = m_sizeFont + m_sizeGrafic;

	} else {

		//###################################
		// Als einzelnen Zeichensatz anfuegen

		// Zeichensaetze initialisieren
		DefaultFont(1);

		// Neuen Textblock anfuegen
		*m_pFont[0] = *(FONT*)buf;

		// Erstes Freies Datenbyte
		m_pFont[0]->ptrTable = m_pFontNode->ptrFontArray + m_pFontNode->cnt*sizeof(unsigned long) + m_pFontNode->cnt*sizeof(FONT);
		m_pFont[0]->ptrCharacter = m_pFont[0]->ptrTable + pText->cnt*sizeof(unsigned short);

		// Neuen Block anfuegen
		memcpy(&m_memFont[m_pFont[0]->ptrTable], &buf[pText->ptrIDTable], pText->cnt*sizeof(unsigned short));
		memcpy(&m_memFont[m_pFont[0]->ptrCharacter], &buf[pText->ptrTextdata], pText->size);

		// Groessen anpassen
		m_sizeFont = m_pFont[0]->ptrCharacter + m_pFont[0]->size;
		m_sizeData = m_sizeFont + m_sizeGrafic;
	}

	// BUGFIX: Anpassung der Groesse, damit die CRC-Berechnung von der Firmware
	// durchgefuehrt werden kann. Die Firmware geht von einer festen Tabellengroesse von 256 Byte aus.
	// Bei den Textdaten kann die Groesse dieser Tabelle abhaengig von der Maximal-ID variieren.
	m_pFont[i]->size += (((TEXTDATA*)m_pFont[i])->cnt * sizeof(unsigned short)) - 256;

	return 1;
}

