// DFFileStudio.cpp: Implementierung der Klasse CDFFileStudio.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFFileStudio.h"
#include <sys/stat.h>
/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/
//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDFFileStudio::CDFFileStudio()
{
	// Speicher fuer EEP reservieren
	m_eep = new unsigned char[EEP_SIZE_SU];
	// Speicher fuer Flash reservieren
	m_flash = new unsigned char[FLASH_SIZE_SU + FLASH_SIZE_SU2 + EXTENDED_SIZE_SU + 100];
	if (m_flash == NULL) {

//		AfxMessageBox("Nicht gengend Speicher verfgbar.", MB_ICONSTOP);
	}

	// Dokument aufsetzen
	SetDefault();
}

CDFFileStudio::~CDFFileStudio()
{
	delete[] m_eep;
	delete[] m_flash;
}

void CDFFileStudio::SetDefault()
{
	// Eeprom 
	memset(m_eep, 0xFF, sizeof(m_eep));
	
	// Dateiheader 
	memset(&m_fileHeader, 0, sizeof(TAESHeader));

	// Zusaetlicher Setupspeicher
	m_flashSizeSU2 = FLASH_SIZE_SU2;
	// Flashbereiche
	memset(m_flash, 0xFF, FLASH_SIZE_SU + FLASH_SIZE_SU2);

	// Konfigbereich
	memset(&m_config, 0, sizeof(TConfig));
	m_configMem = (unsigned char*)&m_config;

	// Extended Flash
	m_flashExtended = &m_flash[FLASH_SIZE_SU + FLASH_SIZE_SU2];
	memset(m_flashExtended, 0, EXTENDED_SIZE_SU + 100);

	// Der erweiterte Speicherbereich wird mit einer dynamischen Struktur begonnen,
	// deren erster Parameter ein Check - UINT32 - Wert enthaelt.
	TExtendedFlash extBegin;
	// Alles mit Null initialisieren.
	memset(&extBegin, 0, sizeof(TExtendedFlash));
	// Struktur fuer erweiterten Bereich aufsetzen
	extBegin.ulCheck = EXTENDED_CHECK;
	extBegin.ulSize  = sizeof(TExtendedFlash);
	extBegin.pulFreeScope = extBegin.ulSize;
	// Flag fuer Verarbeitung
	extBegin.ulFlags = EXTF_NEW;
	
	*(TExtendedFlash*)m_flashExtended = extBegin;

	// Listenpfade
	memset(m_listFileName, 0, sizeof(m_listFileName));
	memset(m_listEntranceFileName, 0, sizeof(m_listEntranceFileName));
	memset(m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName));
	memset(m_strExtendedFilePath, 0, sizeof(m_strExtendedFilePath));
}

// Mgliche Rckgabewerte:
// -1 -> Datei konnte nicht geffnet werden.
// 0 -> Unbekanntes Dateiformat. 
// 1 -> Erfolgreich geladen.
// 2 -> Dateiformat ist zu aktuell.
int CDFFileStudio::Load(char *fileName)
{
	int res = TRUE;
	struct stat st;
	char *fName;
	char mTimeStamp[20]; // 2009-05-19 09:16:22
	unsigned char buf[2]; // Laengenangabe bei Strings verarbeiten
	int length;
	int i;
	int listFileSize = 0;
	int entranceListFileSize = 0;
	int entrance2ListFileSize = 0;
	int extendedFilePathSize = 0;
	int sizeHeader = EEP_SIZE - EEP_SIZE_SU;
	unsigned long sizeSetup;

	if (stat(fileName, &st) != 0) {

		return FALSE;
	}

	FILE *fp = fopen(fileName, "rb");
	if (fp == NULL) {

		TRACE("Setupdatei konnte nicht geffnet werden.\n");
		return FALSE;
	}

	// EEPROM - lesen
	fread(m_eep, EEP_SIZE_SU, 1, fp);
	
	// Rest des Speicherblocks uebergehen
	fseek(fp, sizeHeader, SEEK_CUR);

	// Setupgroesse
	sizeSetup = *(unsigned long*)&m_eep[P_FLASH_SIZE_SU];
	ChangeOrderLong((unsigned char*)&sizeSetup);
	// Ermittlung der Groesse des zu lesenden Speicherblocks fuer Flash.
	if ((*(unsigned short*)&m_eep[P_FILE_VERSION] == 0xDF02) && (sizeSetup > FLASH_SIZE_SU)) {

		m_flashSizeSU2 = FLASH_SIZE_SU2;
	} else {

		m_flashSizeSU2 = 0;
	}

	// Flashbereich - lesen
	fread(m_flash, FLASH_SIZE_SU + m_flashSizeSU2, 1, fp);

	// ListFile - lesen
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
		
		fread(buf, sizeof(unsigned char), 1, fp); // ein byte Laenge
		if (buf[0] == 0xFF) {

			fread(buf, sizeof(unsigned char), 2, fp); // zwei byte Laenge
			length = *(unsigned short*)buf;

			listFileSize += 3;
		} else {

			length = buf[0];
			listFileSize++;
		}
		listFileSize += length;

		fread(m_listFileName[i], sizeof(unsigned char), length, fp);
	}

	// Konfiguration - lesen
	fread(&m_config, sizeof(TConfig), 1, fp);


	// Sind ab aktueller Postition noch 16kByte erweiterter Speicher vorhanden?
	if ((unsigned long)st.st_size >= 
		(EEP_SIZE_SU + sizeHeader + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU)) {

		// Erweiterter Bereich - lesen: 16kByte Extended Flash
		fread(m_flashExtended, EXTENDED_SIZE_SU, 1, fp);
		// Pruefen der Extended - Struktur
		TExtendedFlash extBegin = *(TExtendedFlash*)m_flashExtended;

		if (extBegin.ulCheck != EXTENDED_CHECK) { // existiert?

//			*piError = IV_ERROR_FILE_FORMAT;
			return FALSE;
		}
	}
	
	// Sind noch ListFile fuer Zutrittskontrolle vorhanden?
	if ((unsigned long)st.st_size >= 
		(EEP_SIZE_SU + sizeHeader + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + 4)) {
		
		// ListFile - lesen
		for (i=0; i < 4; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				entranceListFileSize += 3;
			} else {

				length = buf[0];
				entranceListFileSize++;
			}
			entranceListFileSize += length;

			fread(m_listEntranceFileName[i], sizeof(unsigned char), length, fp);
		}

		// Loeschen, wenn ZK nicht aktiviert.
		if (m_config.ENABLE_ENTRANCE == 0) {

			memset(m_listEntranceFileName, 0, sizeof(m_listEntranceFileName));
		}
	}

	// Sind noch ListFile fuer Zutrittskontrolle 2 vorhanden?
	if ((unsigned long)st.st_size >= 
		(EEP_SIZE_SU + sizeHeader + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + entranceListFileSize + MAX_DATENSATZ_BESCHREIBUNG)) {

		// ListFile - lesen
		for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				entrance2ListFileSize += 3;
			} else {

				length = buf[0];
				entrance2ListFileSize++;
			}
			entrance2ListFileSize += length;

			fread(m_listEntrance2FileName[i], sizeof(unsigned char), length, fp);
		}

		// Loeschen, wenn ZK nicht aktiviert.
		if (m_config.ENABLE_ENTRANCE2 == 0) {

			memset(m_listEntrance2FileName, 0, sizeof(m_listEntrance2FileName));
		}
	}

	// Sind noch ExtendedFilePath fuer Zusatzdateien vorhanden?
	if ((unsigned long)st.st_size >= 
		(EEP_SIZE_SU + sizeHeader + FLASH_SIZE_SU + m_flashSizeSU2 + listFileSize + sizeof(TConfig) + EXTENDED_SIZE_SU + entranceListFileSize + entrance2ListFileSize + MAX_DATENSATZ_BESCHREIBUNG)) {

		// ListFile - lesen
		for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
			
			fread(buf, sizeof(unsigned char), 1, fp); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, fp); // zwei byte laenge
				length = *(unsigned short*)buf;

				extendedFilePathSize += 3;
			} else {

				length = buf[0];
				extendedFilePathSize++;
			}
			extendedFilePathSize += length;

			fread(m_strExtendedFilePath[i], sizeof(unsigned char), length, fp);
		}
	}

	// Die Werte fr Dateinamen und Zeitstempel der Modifizierung eintragen.
	// Dateinamen extrahieren
	fName = &fileName[strlen(fileName)];
	while (fName > fileName) {

		// Maskierte Leerzeichen (unix)
		if (*(fName - 1) == '\\' && *fName == ' ') {

		} else if(*(fName - 1) == '\\') {

			// Pfadbestandteil vor Dateinamen (windows)
			break;

		} else if (*(fName - 1) == '/') {

			// Pfadbestandteil vor Dateinamen (unix)
			break;
		} 

		fName--;
	}

	struct tm *ptm = localtime(&st.st_mtime);
	sprintf(mTimeStamp, "%02i%02i-%02i-%02i %02i:%02i:%02i", 
		(ptm->tm_year + 1900) / 100, (ptm->tm_year + 1900) % 100, ptm->tm_mon + 1, ptm->tm_mday,
		ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
	
	// Eintragen in der Extended - Struktur
	TExtendedFlash extBegin = *(TExtendedFlash*)m_flashExtended;
	if (extBegin.ulCheck == EXTENDED_CHECK && extBegin.pulExtended) {

		TExtended *extended = (TExtended*)&m_flashExtended[extBegin.pulExtended];
		strncpy(extended->fileName, fName, sizeof(extended->fileName));
		// Nullterminierung sicherstellen
		extended->fileName[sizeof(extended->fileName) - 1] = 0;
		strcpy(extended->fileTimeStamp, mTimeStamp);
	}

	fclose(fp);
	return res;
}

int CDFFileStudio::Save(char *fileName)
{
	int res = TRUE;
	int i;
	unsigned long ulListFileSize = 0;
	unsigned long ulEntranceListFileSize = 0;
	int sizeHeader = EEP_SIZE - EEP_SIZE_SU;
	unsigned long sizeSetup;

	FILE *fp = fopen(fileName, "wb");
	if (fp == NULL) {
		
		TRACE("Setupdatei konnte nicht geffnet werden.\n");
		return FALSE;
	}

	// Groesse der Setupdaten eintragen
	sizeSetup = GetFlashSetupSize();
	*(unsigned long*)&m_eep[P_FLASH_SIZE_SU] = sizeSetup;
	ChangeOrderLong(&m_eep[P_FLASH_SIZE_SU]);

	// Ermittlung der Groesse des zu schreibenden Speicherblocks fuer Flash.
	if (sizeSetup > FLASH_SIZE_SU) {

		m_flashSizeSU2 = FLASH_SIZE_SU2;
		*(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF02;
	} else {

		m_flashSizeSU2 = 0;
		*(unsigned short*)&m_eep[P_FILE_VERSION] = 0xDF01;
	}

	// EEPROM - schreiben
	if (fwrite(m_eep, sizeof(unsigned char), EEP_SIZE_SU, fp) != EEP_SIZE_SU)
	{
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
		res = FALSE;
		goto error_label;
	}


	// Dateikopf schreiben
	unsigned char mem[EEP_SIZE - EEP_SIZE_SU];
	memset(mem, 0, sizeof(mem));
	if (fwrite(mem, sizeof(unsigned char), sizeof(mem), fp) != sizeof(mem))
	{
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
		res = FALSE;
		goto error_label;
	}
	//fwrite(&m_fileHeader, sizeof(TAESHeader), 1, fp);


	// Flashbereich - schreiben
	if (fwrite(m_flash, sizeof(unsigned char), FLASH_SIZE_SU + m_flashSizeSU2, fp) != (FLASH_SIZE_SU + m_flashSizeSU2))
	{
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
		res = FALSE;
		goto error_label;
	}


	// ListFile - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listFileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = strlen(m_listFileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		} else {

			unsigned char len = strlen(m_listFileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listFileName[i], sizeof(unsigned char), strlen(m_listFileName[i]), fp) != strlen(m_listFileName[i]))
		{
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
			res = FALSE;
			goto error_label;
		}
	}

	// Konfiguration - schreiben
	if (fwrite(&m_config, sizeof(unsigned char), sizeof(TConfig), fp) != sizeof(TConfig))
	{
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
		res = FALSE;
		goto error_label;
	}

	// Erweiterter Speicher - schreiben
	if (fwrite(m_flashExtended, sizeof(unsigned char), EXTENDED_SIZE_SU, fp) != EXTENDED_SIZE_SU)
	{
		TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
		res = FALSE;
		goto error_label;
	}

	// Zutrittskontrolle - ListFile - schreiben
	for (i=0; i < 4; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listEntranceFileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = strlen(m_listEntranceFileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}

		} else {

			unsigned char len = strlen(m_listEntranceFileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listEntranceFileName[i], sizeof(unsigned char), strlen(m_listEntranceFileName[i]), fp) != strlen(m_listEntranceFileName[i]))
		{
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
			res = FALSE;
			goto error_label;
		}
	}
	// Zutrittskontrolle - ListFile - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_listEntrance2FileName[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = strlen(m_listEntrance2FileName[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		} else {

			unsigned char len = strlen(m_listEntrance2FileName[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_listEntrance2FileName[i], sizeof(unsigned char), strlen(m_listEntrance2FileName[i]), fp) != strlen(m_listEntrance2FileName[i]))
		{
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
			res = FALSE;
			goto error_label;
		}
	}

	// Zusatzpfadangaben - schreiben
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {

		// Laenge des Strings berechnen und nach MFC - CString Konvention schreiben
		if (strlen(m_strExtendedFilePath[i]) > 0xFF) {

			unsigned char len[3];
			len[0] = 0xFF; // Markierung das eine 16bit Laengenangabe folgt.
			*(unsigned short*)&len[1] = strlen(m_strExtendedFilePath[i]);
			if (fwrite(len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		} else {

			unsigned char len = strlen(m_strExtendedFilePath[i]);
			if (fwrite(&len, sizeof(unsigned char), sizeof(len), fp) != sizeof(len))
			{
				TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
				res = FALSE;
				goto error_label;
			}
		}

		// String rausschreiben
		if (fwrite(m_strExtendedFilePath[i], sizeof(unsigned char), strlen(m_strExtendedFilePath[i]), fp) != strlen(m_strExtendedFilePath[i]))
		{
			TRACE("Es konnte nicht in die Datei geschrieben werden.\n");
			res = FALSE;
			goto error_label;
		}
	}

error_label:
	if (fp != NULL)
	{
		fclose(fp);
	}
	return res;
}

// Die zu verwendende Version der Datei
int CDFFileStudio::NeededVersion()
{
	if (GetFlashSetupSize() > FLASH_SIZE_SU) {

		return 2;
	}

	return 1;
}

void CDFFileStudio::ChangeOrderLong(unsigned char *Buf)
{
	unsigned char h;

	h = Buf[0];
	Buf[0] = Buf[3];
	Buf[3] = h;

	h = Buf[1];
	Buf[1] = Buf[2];
	Buf[2] = h;
}

// Groesse der Setupdaten im Flash ermitteln
unsigned long CDFFileStudio::GetFlashSetupSize()
{
	// ##############################################################
	// Pointer sind direkte Adressen die im AEIII+:
	//   1. Im Motorolla - Format vorliegen.
	//   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

	// Pointer auf ListenBeschreibungen ermitteln
	unsigned long pLB = *(unsigned long*)(&m_eep[P_LIST_STRUCT]);

	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pLB);
	//}
	pLB -= FLASH_KONFIG_START;

	// Pointerarray der Listenbeschreibungen durchschreiten
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
	
		// Nullpointer suchen fuer Listenanzahl
		if (*(unsigned long*)&m_flash[pLB + i * sizeof(unsigned long)] == 0ul) {
		
			break;
		}
	}

	// Liste gefunden?

	if (i > 0) {
	
		// Pointer auf letzte definierte Liste
		pLB = *(unsigned long*)(&m_flash[pLB + (i - 1) * sizeof(unsigned long)]);

		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pLB);
		//}
		pLB -= FLASH_KONFIG_START;

		TDatensatzBeschreibung *pLBStruct = (TDatensatzBeschreibung*)(&m_flash[pLB]);

		// Pointer auf Feldarray der Liste
		unsigned long pFeldArray = pLBStruct->P;
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((unsigned char*)&pFeldArray);
		//}
		pFeldArray -= FLASH_KONFIG_START;

		// Felder vorhanden?
		if (pLBStruct->Felder) {
		
			// Pointer auf letztes Feld der Listenbeschreibung ermitteln.
			unsigned long pFeld = *(unsigned long*)&m_flash[pFeldArray + (pLBStruct->Felder - 1) * sizeof(unsigned long)];
			//if (App->ByteOrder == BO_MOT) {
				
				ChangeOrderLong((unsigned char*)&pFeld);
			//}
			pFeld -= FLASH_KONFIG_START;
			
			// Feldadresse + eigene Groesse bildet FlashSetupSize
			return pFeld + sizeof(TDatensatzFeld);
		}


		// Keine Felder definiert!
		return pFeldArray;
	}
	
	// Keine Listenbeschreibung definiert!
	// Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
	// um bei einer Uebertragung diese zu loeschen.
	return pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned long);
}

