/**
 * @file  DFLog.cpp
 * @brief Implementierung der Klasse CDFLog.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFLog.h"

#ifdef _WIN32
#include <windows.h>
#include <direct.h>
#include <io.h>
#else
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>
#include <stdarg.h>
#endif

CDFLog::CDFLog(short channelID)
{
	// Zugewiesene Kanalnummer
	m_channelID = channelID;
	m_logCheck.m_channelID = channelID;
	// Initialisierungsdatei lesen
	logIniRead();
	logCheck();
}

CDFLog::~CDFLog()
{
}

void CDFLog::logIniRead()
{
	char szTmp[260];
	char szFileName[260];
	memset(szFileName, 0, sizeof(szFileName));
#ifdef _WIN32
	// Dateiname der Inidatei
	GetModuleFileName(NULL, szFileName, sizeof(szFileName));
	
	MakeFilePath(szFileName, "DFCom", "ini");
#else
	sprintf(szFileName, INI_FILENAME);
#endif
	
	// pruefen und evnt. anlegen der Inidatei
	struct stat lstat;
	if (stat(szFileName, &lstat) != 0) {
		FILE *fp;
		if ((fp = fopen(szFileName, "w")) == (FILE*)0) {
			// Fehler bei erzeugung
			m_bEnable = false;
			m_bEraseFiles = false;
			return;
		}

		// Sektion "Log" anlegen
		fprintf(fp, "[Log]\n");
		fprintf(fp, "Enable=1\n");
#ifdef _WIN32
		// Aktuelles Verzeichnis verwenden
		memset(szTmp, 0, sizeof(szTmp));
		//GetCurrentDirectory(MAX_PATH, szTmp);
		GetModuleFileName(NULL, szTmp, sizeof(szTmp));	
		MakeFilePath(szTmp, "", "");
		fprintf(fp, "Path=");
		for (int i=0; szTmp[i]; i++) {
			fprintf(fp, "%c", szTmp[i]);
			if (szTmp[i] == '\\') { // Maskierungszeichen einfuegen
				fprintf(fp, "\\");
			}
		}
		fprintf(fp, "\n");
#else
		fprintf(fp, "Path=./\n");
#endif
		fprintf(fp, "Folder=%s\n", SUB_FOLDER);
		fprintf(fp, "Trace=0\n");
		fprintf(fp, "Erase=1\n");
		fprintf(fp, "Keep=10\n");
		fprintf(fp, "Elapse=1\n");
		fprintf(fp, "List=1\n");

		fprintf(fp, "\n\n");
		fprintf(fp, "; Kurzbeschreibung der enthaltenen Parameter:\n");
		fprintf(fp, "; Enable: Aktivieren des Hintergrundloggens. Wertebereich 0 = Nicht loggen, 1 = Loggen aktiviert.\n");
		fprintf(fp, "; Path: Gibt das Basisverzeichnis an. Bei Verwendung von Backslash-Zeichen \\ mssen diese doppelt eingegeben werden.\n");
		fprintf(fp, "; Folder: Das zu verwendende Unterverzeichnis, dieses wird wenn nicht bereits vorhanden angelegt.\n");
		fprintf(fp, "; Trace: Gibt an ob das DLL eigene Detaillog aktiviert sein soll. Wertebereich ist 1 = Ja aktivieren, 0 = Nein.\n; \tBitte beachten Sie, dass hierbei grere Datenmengen anfallen.\n");
		fprintf(fp, "; Erase: Gibt an ob die Logdateien automatisch gelscht werden. Wertebreich ist 1 = Ja automatisch lschen, 0 = Nein manuell lschen.\n; \tBitte beachten  Sie dass die Dateiverwaltung nach Abschalten bernommen werden muss.\n");
		fprintf(fp, "; Keep: Gibt bei Erase=1 die Anzahl der zu haltenden Logtage an. Wertebereich ist 1 - 100 Logtage. Logtage bedeutet hier, Tage an denen geloggt wurde.\n");
		fprintf(fp, "; Elapse: Gibt an ob die Ausfhrungsdauer der jeweiligen Funktion in eckigen Klammern hinter jedem Logeintrag ausgegeben wird, dabei hat dieser folgende Aufteilung [Sekunden.Hundertstel].\n");
		fprintf(fp, "; List: Gibt an ob die bertragenen Listendaten vor jedem bertragen, in das Logverzeichnis exportiert werden.\n; \tWertebereich ist 0 = Nein, 1 = In Binrformat mitprotokollieren, 2 = In Binrform und Textformat (tab-Getrennt).\n");

		fclose(fp);
	}

	// Log aktiv?
	m_bEnable = (m_logIni.GetUInt("Log", "Enable", 1, szFileName)) ? true : false;
	if (!m_bEnable) {

		return;
	}
	// Trace aktiv?
	m_bTrace = (m_logIni.GetUInt("Log", "Trace", 0, szFileName)) ? true : false;
	if (!m_bEnable) {

		return;
	}

	// Basispfad fuer Log
	memset(m_logCheck.m_szBasePath, 0, sizeof(m_logCheck.m_szBasePath));
	m_logIni.GetString("Log", "Path", m_logCheck.m_szBasePath, szFileName);
	if (m_logCheck.m_szBasePath[0] == 0) {
		//fullpath(m_szBasePath, PATH_BASE, sizeof(m_szBasePath));
		getcwd(m_logCheck.m_szBasePath, sizeof(m_logCheck.m_szBasePath));
	} else {
#ifdef _WIN32
		if (m_logCheck.m_szBasePath[strlen(m_logCheck.m_szBasePath) - 1] != '\\') {
			strcat(m_logCheck.m_szBasePath, "\\");
		}
#else
		if (m_logCheck.m_szBasePath[strlen(m_logCheck.m_szBasePath) - 1] != '/') {
			strcat(m_logCheck.m_szBasePath, "/");
		}
#endif
	}
	// Unterordner fuer Logdateien
	memset(m_logCheck.m_szSubFolder, 0, sizeof(m_logCheck.m_szSubFolder));
	m_logIni.GetString("Log", "Folder", m_logCheck.m_szSubFolder, szFileName);
	if (m_logCheck.m_szSubFolder[0] == 0) {
		strcpy(m_logCheck.m_szSubFolder, SUB_FOLDER);
	}
	sprintf(szTmp, "%s%s", m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder);
	// pruefen und evnt. anlegen des Ordners
	if (stat(szTmp, &lstat) != 0) {
#ifdef _WIN32
		if (mkdir(szTmp) != 0) {
#else
		if (mkdir(szTmp, 18) != 0) {
#endif
			// Fehler bei Erzeugung
			m_bEnable = false;
			m_bEraseFiles = false;
			m_bWriteList = 0;
			return;
		}
	}
#ifdef _WIN32
	if (m_logCheck.m_szSubFolder[strlen(m_logCheck.m_szSubFolder) - 1] != '\\') {
		strcat(m_logCheck.m_szSubFolder, "\\");
	}
#else
	if (m_logCheck.m_szSubFolder[strlen(m_logCheck.m_szSubFolder) - 1] != '/') {
		strcat(m_logCheck.m_szSubFolder, "/");
	}
#endif
	// Dateiname der Logdatei
	time_t t = time(NULL);
	tm *ltm = localtime(&t);
	sprintf(m_szFileName, LOG_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, 
		ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_channelID);
	sprintf(m_szFileNameTrace, LOG_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, 
		ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_channelID + 1000);
	// Alte Dateien loeschen
	m_bEraseFiles = (m_logIni.GetUInt("Log", "Erase", 1, szFileName)) ? true : false;
	m_logCheck.m_eraseFileDays = m_logIni.GetUInt("Log", "Keep", 10, szFileName);
	if (m_logCheck.m_eraseFileDays < 1) m_logCheck.m_eraseFileDays = 1;
	if (m_logCheck.m_eraseFileDays > 100) m_logCheck.m_eraseFileDays = 100;
	// Ausfuehrungszeit angeben
	m_bElapse = (m_logIni.GetUInt("Log", "Elapse", 1, szFileName)) ? true : false;
	// Listen wegschreiben
	m_bWriteList = m_logIni.GetUInt("Log", "List", 1, szFileName);
}

void CDFLog::logCheck() 
{
	// Soll durchgefuehrt werden?
	if (!m_bEnable || !m_bEraseFiles) {
		
		return;
	}

#ifdef _WIN32
/*
	// Suche nach Dateien
	sprintf(szFile, LOG_SEARCH_FILENAME, m_szBasePath, m_szSubFolder);
	long fp;
	struct finddata_t finddata;
	if ((fp = findfirst(szFile, &finddata)) != -1) {

		do {

			for (i=0; i < 5; i++) {
				// Stimmt mit einem der zu erhaltenden Dateien ueberein?
				if (strstr(szFileName[i], finddata.name) != 0) {
					break;
				}
			}
			if (i == 5) {
				// Sammeln der zu loeschenden Dateien,
				// da erst nach findclose geloescht werden kann!
				strcpy(szDeleteFileName[j++], finddata.name);
			} 

		} while ((j < 5) && (findnext(fp, &finddata) != -1));
		findclose(fp);

		if (j) {
			// Dateien loeschen
			for (i=0; i < j; i++) {
			
				sprintf(szFile, LOG_DELETE_FILENAME, m_szBasePath, m_szSubFolder, szDeleteFileName[i]);
				if (remove(szFile) == -1) {
					// Konnte nicht geloescht werden!
				}
			}
		}
	}
*/
	m_logCheck.Check();
#else
	int i, j;
	char szFile[260];
	char szFileName[5][260];
	char szDeleteFileName[5][260];

	// Namen der Logdateien fuer die letzten fuenf Tage erstellen.
	for (i=0; i < 5; i++) {
		// Dateiname der Logdatei
		time_t t = time(NULL);
		t -= 3600 * 24 * i;
		tm *ltm = localtime(&t);
		sprintf(szFileName[i], LOG_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, 
			ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_channelID);
	}
	// Alle anderen Logdateien loeschen 

	j = 0;

	// Suche nach Dateien
	sprintf(szFile, "%s%s", m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder);

	DIR *dir;
	if ((dir = opendir(szFile)) != NULL) {
		
		struct dirent *ent;
		while((j < 5) && ((ent = readdir(dir)) != NULL)) {

			for (i=0; i < 5; i++) {
				// Stimmt mit einem der zu erhaltenden Dateien ueberein?
				if (strstr(szFileName[i], ent->d_name) != 0) {
					break;
				}
			}
			if (i == 5) {
				// Sammeln der zu loeschenden Dateien,
				// da erst nach closedir geloescht werden kann!
				strcpy(szDeleteFileName[j++], ent->d_name);
			} 
		}
		closedir(dir);

		if (j) {
			// Dateien loeschen
			for (i=0; i < j; i++) {
			
				sprintf(szFile, LOG_DELETE_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, szDeleteFileName[i]);
				if (remove(szFile) == -1) {
					// Konnte nicht geloescht werden!
				}
			}
		}
	}
#endif
	
}

void CDFLog::logBegin(const char *Format, ...)
{
	if (!m_bEnable) {
	
		return;
	}

	char szTime[9];
	char szFileName[260];
	char szLine[0x400];
	va_list ap;
	FILE *Logfp;

	// Uhrzeit des Logeintrages
	time_t t = time(NULL);
	m_clockBegin = clock();
	tm *ltm = localtime(&t);
	sprintf(szTime, "%02i:%02i:%02i", ltm->tm_hour, ltm->tm_min, ltm->tm_sec);
	// Dateiname der Logdatei
	sprintf(szFileName, LOG_FILENAME, m_logCheck.m_szBasePath, m_logCheck.m_szSubFolder, 
		ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday, m_channelID);
	
	if (strcmp(szFileName, m_szFileName) != 0) {
		// Dateiname hat sich geaendert.
		strcpy(m_szFileName, szFileName);
		logCheck(); // Eventuell die aelteste Logdatei loeschen.
	}

	sprintf(szLine, "%s\t", szTime);
	va_start(ap, Format);
	vsprintf(&szLine[strlen(szLine)], Format, ap);
	va_end(ap);
	
	if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

		return;
	}

	fprintf(Logfp, szLine);

	fclose(Logfp);
}

void CDFLog::logAppend(const char *Format, ...)
{
	if (!m_bEnable) {

		return;
	}

	va_list ap;
	FILE *Logfp;

	if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

		return;
	}

	va_start(ap, Format);
	vfprintf(Logfp, Format, ap);
	va_end(ap);

	fclose(Logfp);
}

void CDFLog::logEnd(const char *Format, ...)
{
	if (!m_bEnable) {
		
		return;
	}

	va_list ap;
	FILE *Logfp;

	if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

		return;
	}

	va_start(ap, Format);
	vfprintf(Logfp, Format, ap);
	va_end(ap);

	// Ausfuehrungszeit angeben?
	if (m_bElapse) {
		// Ausfuehrungszeit berechnen
		double clockDiff;
		clockDiff = (double)(clock() - m_clockBegin) / CLOCKS_PER_SEC;

		fprintf(Logfp, "\t[%2.1f]\n", clockDiff);
	} else {

		fprintf(Logfp, "\n");
	}

	fclose(Logfp);
}

#ifdef _WIN32
void CDFLog::MakeFilePath(char *path_buffer, char *szFileName, char *szFileExt)
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	
	// Speicher initialisieren
	memset(drive, 0, sizeof(drive));
	memset(dir, 0, sizeof(dir));
	memset(fname, 0, sizeof(fname));
	memset(ext, 0, sizeof(ext));
	
	_splitpath(path_buffer, drive, dir, fname, ext);
	// Wenn keine Extension, dann gehoert eventueller Dateiname noch zum Pfad
	if (!strlen(ext)) {
		
		if (strlen(fname)) {
			
			strcat(path_buffer, "\\");
			// Neu auswerten
			_splitpath(path_buffer, drive, dir, fname, ext);
		}
	}
	
	// Speicher loeschen
	memset(path_buffer, 0, _MAX_PATH);
	
	_makepath(path_buffer, drive, dir, (szFileName) ? szFileName : fname, (szFileExt) ? szFileExt : ext);
}
#endif


