/**
* @file  AECom.cpp
* @brief Definitionen der Exportroutinen.
*/
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DatafoxCommIV.h"
#include "AEComm.h"

// ########################################################
// #####################   EXPORTE    #####################
// ########################################################

#ifdef _WIN32
// ########################################################
/**
* @fn    CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
* @brief Ermitteln der DLL - Version. Kompatible zu Windows - Standard.
*
* @param pdvi       { Zeiger auf Struktur zur Informationsuebergabe. }
*/
extern "C" HRESULT PASCAL EXPORT CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
{
	char szModuleName[MAX_PATH];
	DWORD dwHandle;
	DWORD dwLen;
	LPVOID lpBuffer = NULL;
	UINT uiLen;

	unsigned char *pucData = NULL;

	// zulaessige Bereiche pruefen
	if (pdvi == NULL) {

		return E_ABORT;
	}

	if (pdvi->cbSize != sizeof(DLLVERSIONINFO)) {

		return E_ABORT;
	}
	// Modulname der DLL
	if (!GetModuleFileName(theApp.m_hInstance, szModuleName, MAX_PATH)) {

		return E_ABORT;
	}
	// Laenge der Version - Resource
	dwLen = GetFileVersionInfoSize(szModuleName, &dwHandle);
	if (!dwLen) {

		return E_ABORT;
	}

	pucData = new unsigned char[dwLen + 100]; // unicode
	// DLL - Versioninfo ermitteln
	if (!GetFileVersionInfo(szModuleName, dwHandle, dwLen, (LPVOID)pucData)) {

		delete pucData;
		return E_ABORT;
	}
	// Pointer auf allgemeine Infostruktur ermitteln
	if (!VerQueryValue((LPVOID)pucData, "\\\0", &lpBuffer, &uiLen)) {

		delete pucData;
		return E_ABORT;
	}
	// Daten zurueckgeben
	VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;
	pdvi->dwMajorVersion = pvs_ffi->dwFileVersionMS >> 16;
	pdvi->dwMinorVersion = pvs_ffi->dwFileVersionMS & 0x0000FFFF;
	pdvi->dwBuildNumber  = pvs_ffi->dwFileVersionLS >> 16;
	pdvi->dwPlatformID   = DLLVER_PLATFORM_WINDOWS;

	delete pucData;

	return NOERROR;
}
#endif

#undef FUNC_NAME
#define FUNC_NAME "DFCGetErrorText"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCGetErrorText(int iChannelNum, int iError, int iLanguage, char *szText, int iLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iError, iLanguage);

	// zulaessige Bereiche pruefen
	if (szText == NULL ) {

		pChannel->logEnd("\t\t%i", iLength);
		return;
	}

	switch(iError) {
	case IV_ERROR_CRC_PACKET		:
		snprintf(szText, iLength, "CRC Fehler.");
		break;
	case IV_ERROR_CHANNEL_OPEN	:
		snprintf(szText, iLength, "Oeffnen der Schnittstelle fehlgeschlagen.");
		break;
	case IV_ERROR_CHANNEL_CLOSED	:
		snprintf(szText, iLength, "Schnittstelle ist geschlossen.");
		break;
	case IV_ERROR_CHANNEL_COMMUNICATION		:
		snprintf(szText, iLength, "Allgemeiner Fehler waehrend Kommunikation.");
		break;
	case IV_ERROR_RECEIVE 	:
		snprintf(szText, iLength, "Geraet antwortet nicht.");
		break;
	case IV_ERROR_TRANSMIT	:
		snprintf(szText, iLength, "Fehler waehrend Senden eingetreten.");
		break;
		/*
		case ERROR_AT_RECIEVING	:
		snprintf(szText, iLength, "Fehler waehrend Empfang eingetreten.");
		break;
		*/
	case IV_ERROR_ADDRESS	:
		snprintf(szText, iLength, "Antwort von falschem Geraet oder Antwort nicht sinnvoll.");
		break;
	case IV_ERROR_DEVICE_ID_REMOTE	:
		snprintf(szText, iLength, "Keine gueltige Busnummer. (1 - 31, 254)");
		break;
	case IV_ERROR_CHANNEL	:
		snprintf(szText, iLength, "Kanal nicht vorhanden oder nicht erstellbar.");
		break;
	case IV_ERROR_FILE_OPEN		:
		snprintf(szText, iLength, "Oeffnen einer Datei gescheitert.");
		break;
	case IV_ERROR_FILE_FORMAT	:
		snprintf(szText, iLength, "z.B. falsches Format einer Setup-Datei.");
		break;
	case IV_ERROR_LIST_PROCESSING		:
		snprintf(szText, iLength, "Fehler bei Listendatenverarbeitung.");
		break;
	case IV_ERROR_VERSION_FIRMWARE	:
		snprintf(szText, iLength, "Firmwareversion nicht kompatible zu Routine.");
		break;
	case IV_ERROR_DATA_AVAILABLE	:
		snprintf(szText, iLength, "Es sind noch Daten im Geraet vorhanden.");
		break;
	case IV_ERROR_USER_CANCEL	:
		snprintf(szText, iLength, "Durchfuehrung wurde von Anwender abgebrochen.");
		break;
	case IV_ERROR_DESCRIPTION_RECORD_UNLOADED	:
		snprintf(szText, iLength, "Keine Datensatzbeschreibung geladen.");
		break;
	case IV_ERROR_DESCRIPTION_LIST_UNLOADED	:
		snprintf(szText, iLength, "Keine Listenbeschreibung geladen.");
		break;
		/*
		case ERROR_FLASH_PROG:
		snprintf(szText, iLength, "Fehler beim Programmieren des Flash, Firmware.");
		break;
		*/
	case IV_ERROR_FUNCTION_PARAMETER:
		snprintf(szText, iLength, "Parameter einer Funktion ausserhalb ihres Gueltigkeitsbereiches.");
		break;
	case IV_ERROR_VARIABLE_UNDEFINED:
		snprintf(szText, iLength, "Globale Variable nicht definiert.");
		break;
		/*
		case ERROR_NO_DATA:
		snprintf(szText, iLength, "Anfrage obwohl keine Daten.");
		break;
		*/
		/*
		case ERROR_MULTI:
		snprintf(szText, iLength, "Dublikat von Datensaetzen abgefangen.");
		break;
		*/
	case IV_ERROR_BOOTVERSION:
		snprintf(szText, iLength, "Geraetedatei nicht mit Bootloaderversion kompatible.");
		break;
		/*
		case ERROR_BOOTFILEVERSION:
		snprintf(szText, iLength, "Firmwaredatei nicht mit Bootloaderversion kompatible.");
		break;
		*/
	case IV_ERROR_VARIABLE_VALUE:
		snprintf(szText, iLength, "Wert einer Variablen konnte nicht gesetzt werden.");
		break;
	case IV_ERROR_FUNCTION_UNAVAILABLE:
		snprintf(szText, iLength, "Funktion steht im vorliegenden Fall nicht zur Verfuegung.");
		break;
	case IV_ERROR_PROCESSING:
		snprintf(szText, iLength, "Der Befehl wurde erfolgreich uebermittelt, konnte aber nicht durchgefuehrt werden.");
		break;
	case IV_ERROR_DEVICE_UNKNOWN:
		snprintf(szText, iLength, "Der Geraetetyp ist unbekannt. Update der Software erforderlich.");
		break;
	case IV_ERROR_DEVICEFILE_WRONG:
		snprintf(szText, iLength, "Die angegebene Geraetedatei stimmt nicht mit der vorliegenden Hardware ueberein.");
		break;
	case IV_ERROR_DEVICE_WRONG:
		snprintf(szText, iLength, "Geraetetyp stimmt nicht. z.B. Versuchtes Einspielen von Setupdatei fuer PZE-MasterIV in BDE-MasterIV.");
		break;
	case IV_ERROR_FILE_EXTENSION:
		snprintf(szText, iLength, "Die vorliegende Dateiendung wird nicht unterstuetzt.");
		break;
	case IV_ERROR_ENTRANCE2_OFF:
		snprintf(szText, iLength, "Die Zutrittskontrolle ist deaktiviert.");
		break;
	case IV_ERROR_NAME_UNKNOWN:
		snprintf(szText, iLength, "Der Tabellenname oder Spaltenname ist undefiniert.");
		break;
	case IV_ERROR_HANDLE:
		snprintf(szText, iLength, "Ein angegebener Handle ist nicht gueltig.");
		break;
	case IV_ERROR_NULL_RESULT:
		snprintf(szText, iLength, "Der anzuwendende Filter wuerde keine Datensaetze liefert.");
		break;
	case IV_ERROR_BOF:
		snprintf(szText, iLength, "Sprung fuehrte zum Anfang der Tabelle. (Base of File)");
		break;
	case IV_ERROR_EOF:
		snprintf(szText, iLength, "Sprung fuehrte zum Ende der Tabelle. (End of File)");
		break;
	case IV_ERROR_VALUE:
		snprintf(szText, iLength, "Der angegebene Feldwert ist Fehlerhaft.");
		break;
	case IV_ERROR_OVERFLOW:
		snprintf(szText, iLength, "Bei einer Datenuebertragung ist ein Speicherueberlauf eingetreten.");
		break;
	case IV_ERROR_NO_DATA_AVAILABLE:
		snprintf(szText, iLength, "Es liegen keine Daten vor.");
		break;
	case IV_ERROR_PROCESSING_DLL:
		snprintf(szText, iLength, "Der Befehl konnte zwar uebermittelt werden, bei den Daten liegt jedoch ein Fehler vor.");
		break;
	case IV_ERROR_MODULE_BUSY:
		snprintf(szText, iLength, "Das angesprochene Modul ist zur Zeit durch einen anderen Process blockiert.");
		break;
	case IV_ERROR_FILE_LOAD_XML:
		snprintf(szText, iLength, "Fehler beim Laden der XML-Datei zu ModifyStudioFile.");
		break;
	case IV_ERROR_FILE_UNKNOWN_XML:
		snprintf(szText, iLength, "Unbekannte Elemente der XML-Datei zu ModifyStudioFile.");
		break;
	case IV_ERROR_FILE_CONTENT_XML:
		snprintf(szText, iLength, "Fehler im Dateninhalt der XML-Datei zu ModifyStudioFile.");
		break;
	case IV_ERROR_FILE_MODIFY:
		snprintf(szText, iLength, "Fehler bei Uebernahme (Anwendung) der zu modifizierenden Daten");
		break;
	case IV_ERROR_ALLOC_MEM:
		snprintf(szText, iLength, "Speicherallokation konnte nicht durchgefuehrt werden.");
		break;
	case IV_ERROR_VERSION_DEVICEFILE:
		snprintf(szText, iLength, "Diese Version der Geraetedatei kann aufgrund ihrer Aktualitt nicht verarbeitet werden.");
		break;
	case IV_ERROR_FILE_PROCESSING:
		snprintf(szText, iLength, "Bei der Verarbeitung der Dateidaten trat ein Fehler auf.");
		break;
	case IV_ERROR_DEVICEFILE_MATCH:
		snprintf(szText, iLength, "Die vorliegende Geraetedatei passt nicht zur vorliegenden Hardware. Oder aber: Im Archiv wurde keine passende Gertedatei gefunden.");
		break;
	case IV_ERROR_DEVICEFILE_LOAD:
		snprintf(szText, iLength, "Fehler beim Laden der Gertedatei(archive) aufgetreten.");
		break;
	case IV_ERROR_AC_STARTED:
		snprintf(szText, iLength, "Die Aktive Verbindung ist gestartet.");
		break;
	case IV_ERROR_AC_STOPPED:
		snprintf(szText, iLength, "Die Aktive Verbindung ist nicht gestartet.");
		break;
	case IV_ERROR_AC_PORTINUSE:
		snprintf(szText, iLength, "Der angegebene Listensocket ist bereits in Verwendung.");
		break;
	case IV_ERROR_AC_NO_NOTIFY_RECORD:
		snprintf(szText, iLength, "Die Aktive Meldung von Datenstzen ist deaktiviert.");
		break;
	case IV_ERROR_AC_LISTENER:
		snprintf(szText, iLength, "Allgemeiner Fehler beim Starten der Aktiven Verbindung.");
		break;

	case IV_ERROR_NEED_PASSWORD:
		snprintf(szText, iLength, "Das angegebene Passwort stimmt nicht mit dem bentigten berein. Es wird ein Passwort gefordert.");
		break;
	case IV_ERROR_PASSWORD_LOCK:
		snprintf(szText, iLength, "Gert ist aufgrund von fehlerhaften Passworteingaben fr 60 Sekunden gesperrt.");
		break;

	case IV_ERROR_FILE_WRITE:
		snprintf(szText, iLength, "Das Schreiben in die Datei ist fehlgeschlagen.");
		break;
	case IV_ERROR_FILE_READ:
		snprintf(szText, iLength, "Das Lesen aus der Datei ist fehlgeschlagen.");
		break;
	case IV_ERROR_FILE_CRC:
		snprintf(szText, iLength, "Fehlerhafter Dateninhalt. Die Prfung auf die Datenkonsistenz per CRC schlug fehl.");
		break;
	case IV_ERROR_MEMORY_FULL:
		snprintf(szText, iLength, "Es steht kein weiterer Speicherplatz zur Verfgung.\nZ. B. kann die maximale aufnehmbare Anzahl Fingertemplates erreicht sein.");
		break;
	case IV_ERROR_FILE_DATA_FORMAT:
		snprintf(szText, iLength, "Die in der Datei vorliegenden Daten stimmen nicht mit den geforderten berein.\nZ. B. in der Datei liegen Fingertemplates vom Typ Idencom-Compact vor, es werden jedoch DIN Erfordert.");
		break;
	case IV_ERROR_NOT_ENOUGH_FLASH:
		snprintf(szText, iLength, "Es steht nicht gengend Flash-Speicher zur Verfgung, um die Daten aufzunehmen.");
		break;
	case IV_ERROR_NO_MODULE:
		snprintf(szText, iLength, "Das von Ihnen angesprochene Modul ist eine Optionale Hardware-Erweiterung und ist im vorliegenden Gert nicht verfgbar.");
		break;
	case IV_ERROR_VERSION_SETUPFILE:
		snprintf(szText, iLength, "In der Setupdatei werden Erweiterungen verwendet, die einen aktuelleren Firmwarestand voraussetzen.");
		break;

	default:
		snprintf(szText, iLength, "Unbekannter Fehler.");
		break;
	}

	pChannel->logEnd("\t%s\t%i", szText, iLength);
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogFileName"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogFileName(int iChannelNum, const unsigned char *szLogFileName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t%s", FUNC_NAME, iChannelNum,
		(szLogFileName == NULL) ? "" : (char*)szLogFileName);

	// zulaessige Bereiche pruefen
	if (szLogFileName == NULL ) {

		pChannel->logEnd("");
		return;
	}


	pChannel->SetLogFileName((const char*)szLogFileName);

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogOn"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogOn(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt?
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	// Pruefe ob Channel vorhanden
	pChannel->SetLogOn();

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogOff"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogOff(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	pChannel->SetLogOff();

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetComPort"
// ########################################################
extern "C" __declspec(dllexport)

HANDLE __stdcall DFCGetComPort(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return INVALID_HANDLE_VALUE;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	HANDLE hRes = pChannel->GetComPort();

	pChannel->logEnd("\t(%i)", hRes);

	return hRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetComPort"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetComPort(int iChannelNum, HANDLE hComPort, int iType, int iComValue, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	int iRes = (pChannel->SetComPort(hComPort, iType, iComValue, iTimeOut) == IV_ERROR_SUCCESS);

	pChannel->logEnd("\t%i\t%i\t%i\t%i\t(%i)", hComPort, iType, iComValue, iTimeOut, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenSerial"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComOpenSerial(int iChannelNum, char *szComPort, int iBaudRate, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum,
		(szComPort == NULL) ? "" : szComPort, iBaudRate, iTimeOut);

	int iRes = FALSE;

	// Is Channel not engaged?
	if (!pChannel->IsOpen()) {

		if (pChannel->OpenAEIII(1, szComPort, iBaudRate, iTimeOut) == IV_ERROR_SUCCESS) {

			iRes = TRUE;
		}
	}
	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenSocket"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComOpenSocket(int iChannelNum, char *szRemoteIP, int iPortNum, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum,
		(szRemoteIP == NULL) ? "" : szRemoteIP, iPortNum, iTimeOut);

	// zulaessige Bereiche pruefen
	if (szRemoteIP == NULL ) {

		pChannel->logEnd("\t(0)");
		return 0;
	}


	int iRes = FALSE;

	// Is Channel not engaged?
	if (!pChannel->IsOpen()) {

		if (pChannel->OpenAEIII(3, szRemoteIP, iPortNum, iTimeOut) == IV_ERROR_SUCCESS) {

			iRes = TRUE;
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenIV"
// ########################################################
// Multimaster

extern "C" __declspec(dllexport)

int __stdcall DFCComOpenIV(int iChannelNum, int iBusNum, int iType, char *szComString, int iComValue, int iComTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iType,
		(szComString == NULL) ? "" : szComString, iComValue, iComTimeOut);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 32)) || (iBusNum == 254)) ||
		(szComString == NULL) ) {

			pChannel->logEnd("\t(0)");
			return 0;
	}


	int iRes = FALSE;

	// Is Channel not engaged?
	if (!pChannel->IsOpen()) {

		//2009_09_02: Busnummer fix auf 32
		if (pChannel->Open(32, iType, szComString, iComValue, iComTimeOut) == IV_ERROR_SUCCESS) {

			iRes = TRUE;
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

// ________________________________________________________

#undef FUNC_NAME
#define FUNC_NAME "DFCComClose"
// ########################################################
extern "C" __declspec(dllexport)

void   __stdcall DFCComClose(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	pChannel->Close();

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckDevice"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCheckDevice(int iChannelNum, int iBusNum, int *piError, int iDevicePollRetry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) || (iDevicePollRetry < 1)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER, iDevicePollRetry);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t(0)", *piError, iDevicePollRetry);
		return 0;
	}

	int iRes = pChannel->CheckDevice(iBusNum, piError, iDevicePollRetry);

	pChannel->logEnd("\t%i\t%i\t(%i)", *piError, iDevicePollRetry, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetDevicePollRetry"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetDevicePollRetry(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);
	pChannel->logEnd("\t(%i)", pChannel->m_iDevicePollRetry);

	return pChannel->m_iDevicePollRetry;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComCollectData"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComCollectData(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
			return -1;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, -1);
		return -1;
	}

	int iRes = pChannel->CollectData(iBusNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == -1) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComGetDatensatz"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComGetDatensatz(int iChannelNum, unsigned char *pucBuf, int *piLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if ((pucBuf == NULL) || (piLength == NULL) || (piError == NULL) ) {

		if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
		return -1;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, -1);
		return -1;
	}

	int iRes = pChannel->GetDatensatz(pucBuf, piLength, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == -1) pChannel->ResetWithError(*piError);

	switch(iRes) {
	case 1:
	case 2: {

		// Daten in Puffer uebergeben
		unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
		int i;

		for (i=0; i < *piLength; i++) {

			sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)pucBuf[i]);
		}
		ucTmp[i * 3] = 0;

		pChannel->logAppend("\t%s\t%i", ucTmp, *piLength);
		break;}

	default:
		pChannel->logAppend("\t%s\t%i", "", 0);
	}

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCReadRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCReadRecord(int channelID, int deviceID, char *buf, int *length, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if ((buf == NULL) || (length == NULL) || (error == NULL) ) {

		if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
		return -1;
	}
	*error = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t\t\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->DFReadRecord(deviceID, buf, length, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) {
		pChannel->ResetWithError(*error);

		pChannel->logAppend("\t\t");
	} else if (res == 0) {

		pChannel->logAppend("\t\t");
	} else {

		// Daten fuer Log in Puffer uebergeben
		unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
		int i;
		for (i=0; i < *length; i++) {

			sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buf[i]);
		}
		ucTmp[i * 3] = 0;

		pChannel->logAppend("\t%s\t%i", ucTmp, *length);
	}
	pChannel->logEnd("\t%i\t(%i)", *error, res);
	return res;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCQuitRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCQuitRecord(int channelID, int deviceID, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (error == NULL ) {

		if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
		return -1;
	}
	*error = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->DFQuitRecord(deviceID, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) pChannel->ResetWithError(*error);

	pChannel->logEnd("\t%i\t(%i)", *error, res);
	return res;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCComSendMessage"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComSendMessage(int iChannelNum, int iBusNum,
								unsigned char ucDelayTime, unsigned char ucAction,
								unsigned char ucAudio, char *szMessage, int iLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i\t%s\t%i", FUNC_NAME,
		iChannelNum, iBusNum, ucDelayTime, ucAction, ucAudio,
		(szMessage == NULL) ? "" : szMessage, iLength);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!((iLength >= 0) && (iLength <= 242)) ||
		(szMessage == NULL) ) {

			pChannel->logEnd("\t(0)");
			return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->SendMessage(iBusNum, ucDelayTime, ucAction, ucAudio,
		szMessage, iLength, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	pChannel->logEnd("\t(%i)", iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComSendInfotext"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComSendInfotext(int iChannelNum, int iBusNum, char *szMessage, int iLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%s\t%i", FUNC_NAME,
		iChannelNum, iBusNum, (szMessage == NULL) ? "" : szMessage, iLength);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!((iLength >= 0) && (iLength <= 242)) ||
		(szMessage == NULL) ) {

			pChannel->logEnd("\t(0)");
			return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->SendInfotext(iBusNum, szMessage, iLength, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	pChannel->logEnd("\t(%i)", iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComClearData"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComClearData(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->ClearData(iBusNum, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	pChannel->logEnd("\t(%i)", iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCRestoreRecords"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCRestoreRecords(int channelID, int deviceID, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(0)", *piError);
		return 0;
	}

	*piError = IV_ERROR_SUCCESS;
	int iRes = pChannel->RestoreRecords(deviceID, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComSetTime"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComSetTime(int iChannelNum, int iBusNum, unsigned char *pucDateTime)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		pChannel->logEnd("\t\t(0)");
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->SetDateTime(iBusNum, pucDateTime, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	// Uhrzeit mitloggen
	if (pucDateTime != NULL) {
		pChannel->logEnd("\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i)",
			pucDateTime[0], pucDateTime[1], pucDateTime[2], pucDateTime[3],
			pucDateTime[4], pucDateTime[5], pucDateTime[6], iRes);
	} else {

		pChannel->logEnd("\t\t(%i)", iRes);
	}
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComGetTime"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComGetTime(int iChannelNum, int iBusNum, unsigned char *pucDateTime)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucDateTime == NULL) ) {

			pChannel->logEnd("\t\t(0)");
			return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->GetDateTime(iBusNum, pucDateTime, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	if (iRes) {
		pChannel->logEnd("\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i)",
			pucDateTime[0], pucDateTime[1], pucDateTime[2], pucDateTime[3],
			pucDateTime[4], pucDateTime[5], pucDateTime[6], iRes);
	} else {
		pChannel->logEnd("\t\t(%i)", iRes);
	}

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeListe"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCMakeListe(int iChannelNum, int iListNum, int iLineCount, int iListSize,
						   unsigned char *pucBuf, int iListMemSizeNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t\t%i", FUNC_NAME, iChannelNum,
		iListNum, iLineCount, iListSize, iListMemSizeNum);

	// zulaessige Bereiche pruefen
	if (!((iListNum >= 0) && (iListNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		(pucBuf == NULL) ) {

			pChannel->logEnd("\t(0)");
			return 0;
	}

	int iRes = pChannel->MakeListe(iListNum, iLineCount, iListSize, pucBuf);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCClrListenBuffer"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCClrListenBuffer(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	pChannel->ClearListBuffer();

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadListen"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadListen(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes;
	// Listendaten wegschreiben.
	if (pChannel->logIsWriteList()) {

		char fileNameBin[_MAX_PATH];
		char basePath[_MAX_PATH];
		int listNum;
		FILE *fp = NULL;
		time_t t;
		tm *ltm;

		t = time(NULL);
		ltm = localtime(&t);

		for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++) {

			// Listendaten vorhanden?
			if (pChannel->m_listSize[listNum] == 0) {
				continue;
			}

			unsigned char *pucBuf = &pChannel->m_pucListMem[listNum][sizeof(unsigned long)];
			unsigned long size = pChannel->m_listSize[listNum] - sizeof(unsigned long);
			unsigned long recordCount = *(unsigned long*)pChannel->m_pucListMem[listNum];

			// Dateiname der Logdatei
			sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
				ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
				ltm->tm_hour, ltm->tm_min, ltm->tm_sec, iChannelNum, 'L', listNum);

			// Daten Binaer rausschreiben
			fp = fopen(fileNameBin, "wb");
			if (!fp) {
				goto error_label_bin;
			}
			fwrite((const void*)pucBuf, 1, size, fp);
			fclose(fp);

error_label_bin:
			if (fp) fclose(fp);
		}
	}

	// Listendaten wegschreiben.
	if (pChannel->logIsWriteList() > 1) {

		char fileNameTxt[_MAX_PATH];
		char basePath[_MAX_PATH];
		unsigned char listName[32], fieldCount, varList;
		unsigned char fieldName[32], fieldNum, fieldType, fieldLength;
		int listOffs, listNum;
		FILE *fp = NULL;
		time_t t;
		tm *ltm;
		char line[512];

		// Laden der Listenbeschreibungen
		iRes = pChannel->GetTableDescription(iBusNum, TRUE, piError);
		// Nach Kommunikationsfehler, Kanal bereinigen
		if (iRes == 0) {

			pChannel->ResetWithError(*piError);
			pChannel->logEnd("\t%i\t(%i)", *piError, iRes);
			return iRes;
		}

		t = time(NULL);
		ltm = localtime(&t);

		for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++) {

			// Listendaten vorhanden?
			if (pChannel->m_listSize[listNum] == 0) {
				continue;
			}

			unsigned char *pucBuf = &pChannel->m_pucListMem[listNum][sizeof(unsigned long)];
			unsigned long size = pChannel->m_listSize[listNum] - sizeof(unsigned long);
			unsigned long recordCount = *(unsigned long*)pChannel->m_pucListMem[listNum];

			if (!pChannel->GetTableRecord(listNum, listName, &fieldCount, &varList, TRUE)) {
				goto error_label_txt;
			}

			// Dateiname der Logdatei
			sprintf(fileNameTxt, LOG_LIST_FILENAME_TXT, pChannel->logGetBasePath(basePath),
				ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
				ltm->tm_hour, ltm->tm_min, ltm->tm_sec, iChannelNum, 'L', listName);

			// Daten in Textform ablegen
			fp = fopen(fileNameTxt, "wb");
			if (!fp) {
				goto error_label_txt;
			}

			listOffs = 0;
			unsigned long recordNum;
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				memset(line, 0, sizeof(line));
				for (fieldNum=0; fieldNum < fieldCount; fieldNum++) {

					if (!pChannel->GetTableField(listNum, fieldNum, fieldName, &fieldType, &fieldLength, TRUE)) {
						fclose(fp);
						goto error_label_txt;
					}

					strcat(line, (char*)&pucBuf[listOffs]);
					if (fieldNum != (fieldCount-1)) strcat(line, "\t");
					listOffs += fieldLength;
				}

				fprintf(fp, "%s\x00D\x00A", line);
			}
error_label_txt:
			if (fp) fclose(fp);
		}
	}

	iRes = pChannel->LoadListData(iBusNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadDatensatzbeschreibung"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadDatensatzbeschreibung(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetTableDescription(iBusNum, FALSE, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadListenbeschreibung"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadListenbeschreibung(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetTableDescription(iBusNum, TRUE, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBCnt"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCDatBCnt(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	int iRes = pChannel->GetTableCount(FALSE);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBCnt"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCListBCnt(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	int iRes = pChannel->GetTableCount(TRUE);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBDatensatz"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCDatBDatensatz(int iChannelNum, int iDataNum, unsigned char *szName, int *piFieldCount)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum);

	// zulaessige Bereiche pruefen
	if (!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		(szName == NULL) || (piFieldCount == NULL) ) {

			pChannel->logEnd("\t\t\t(0)");
			return 0;
	}

	unsigned char szTableName[MAX_NAME];
	unsigned char ucFieldCount;

	int iRes = pChannel->GetTableRecord(iDataNum, szTableName, &ucFieldCount);

	if (iRes) {

		strcpy((char*)szName, (const char*)szTableName);
		*piFieldCount = ucFieldCount;
	}

	pChannel->logEnd("\t%s\t%i\t(%i)", szName, *piFieldCount, iRes);
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBDatensatz"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCListBDatensatz(int iChannelNum, int iDataNum, unsigned char *szName,
								int *piFieldCount, int *piVar)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum);

	// zulaessige Bereiche pruefen
	if (!( ((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		((iDataNum >= 100) && (iDataNum < 104)) ||
		((iDataNum >= 200) && (iDataNum < 207)) ) ||
		(szName == NULL) || (piFieldCount == NULL) /*|| (piVar == NULL)*/ ) {

			pChannel->logEnd("\t\t\t\t(0)");
			return 0;
	}

	unsigned char szTableName[MAX_NAME];
	unsigned char ucFieldCount, ucVariable;

	int iRes = pChannel->GetTableRecord(iDataNum, szTableName, &ucFieldCount, &ucVariable, TRUE);

	if (iRes) {

		strcpy((char*)szName, (const char*)szTableName);
		*piFieldCount = ucFieldCount;
		// Variable nicht verwendet, kann NULL uebergeben
	}

	pChannel->logEnd("\t%s\t%i\t0\t(%i)", szName, *piFieldCount, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBFeld"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCDatBFeld(int iChannelNum, int iDataNum, int iField, unsigned char *szName,
						  int *piType, int *piLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum, iField);

	// zulaessige Bereiche pruefen
	if (!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		!((iField >= 0) && (iField < MAX_FELD)) ||
		(szName == NULL) || (piType == NULL) || (piLength == NULL) ) {

			pChannel->logEnd("\t\t\t\t(0)");
			return 0;
	}

	unsigned char szFieldName[MAX_NAME];
	unsigned char ucType, ucLength;

	int iRes = pChannel->GetTableField(iDataNum, iField, szFieldName,
		&ucType, &ucLength, FALSE);

	if (iRes) {

		strcpy((char*)szName, (const char*)szFieldName);
		*piType = ucType;
		*piLength = ucLength;
	}

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", szName, *piType, *piLength, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBFeld"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCListBFeld(int iChannelNum, int iDataNum, int iField, unsigned char *szName,
						   int *piType, int *piLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum, iField);

	// zulaessige Bereiche pruefen
	if (!( ((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		((iDataNum >= 100) && (iDataNum < 104)) ||
		((iDataNum >= 200) && (iDataNum < 207)) ) ||
		!((iField >= 0) && (iField < MAX_FELD)) ||
		(szName == NULL) || (piType == NULL) || (piLength == NULL) ) {

			pChannel->logEnd("\t\t\t\t(0)");
			return 0;
	}

	unsigned char szFieldName[MAX_NAME];
	unsigned char ucType, ucLength;

	int iRes = pChannel->GetTableField(iDataNum, iField, szFieldName,
		&ucType, &ucLength, TRUE);

	if (iRes) {

		strcpy((char*)szName, (const char*)szFieldName);
		*piType   = ucType;
		*piLength = ucLength;
	}

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", szName, *piType, *piLength, iRes);
	return iRes;
}


// #################### +++++++++++++++++++ #########################
// #################### ZUTRITTSKONTROLLE 1 #########################

#undef FUNC_NAME
#define FUNC_NAME "DFCClearEntranceListBuffer"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCClearEntranceListBuffer(int iChannelNum, int iListNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!(((iListNum >= 0) && (iListNum <= 3)) || (iListNum == -1)) ) {

		pChannel->logEnd("");
		return;
	}

	pChannel->ClearEntranceListBuffer(iListNum);

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeEntranceList"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCMakeEntranceList(int iChannelNum, int iListNum, int iLineCount, int iListSize,
								  unsigned char *pucBuf, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iListNum, iLineCount, iListSize);

	// zulaessige Bereiche pruefen
	if (!((iListNum >= 0) && (iListNum <= 3)) ||
		(pucBuf == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// ListNum ist:
	// 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
	// 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
	// 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
	// 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich

	// Bereich der gueltigen Nummer pruefen.
	switch(iListNum) {
	case 0:
		if ((iLineCount < 1) || (iLineCount > 16)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_0)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 1:
		if ((iLineCount < 1) || (iLineCount > 32)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_1)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 2:
		if ((iLineCount < 1) || (iLineCount > 5120)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_2)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 3:
		if ((iLineCount < 1) || (iLineCount > 64)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_3)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;

	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
	}


	int iRes = pChannel->MakeEntranceList(iListNum, iLineCount, iListSize, pucBuf);

	pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_SUCCESS, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadEntranceList"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadEntranceList(int iChannelNum, int iBusNum, int iListNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!(((iListNum >= 0) && (iListNum <= 3)) || (iListNum == -1)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes;
	// Kommunikationsversionsnummer
	int commVersion = 0;

	if (pChannel->GetProtocolVersion(iBusNum) == 1) {
		// Bei AEIII+ zusaetzliche Prfung wegen Befehlsunterstuetzung notwendig
		commVersion = pChannel->GetProtocolVersion2(iBusNum, TRUE, piError);
		switch(commVersion) {
			//case 0x0400: Unterstuetzt kein Zutritt
		case 0x0401:
		case 0x0402:
			break;
		default:
			*piError = IV_ERROR_VERSION_FIRMWARE;
			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}
	}

	if (commVersion != 0x0400 && commVersion != 0x0401) {

		char version[32];
		memset(version, 0, sizeof(version));
		// Firmwareversion lesen
		if (!(iRes = pChannel->GetVersionFirmware(iBusNum, version, piError))) {

			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Firmwareversion pruefen
		if (!theApp.m_util.CheckVersionFirmwareZKOld(version)) {

			*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}
	}

	iRes = pChannel->LoadEntranceListData(iBusNum, iListNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

// #################### ZUTRITTSKONTROLLE 2 #########################

#undef FUNC_NAME
#define FUNC_NAME "DFCClearEntrance2ListBuffer"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCClearEntrance2ListBuffer(int iChannelNum, int iListNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!(((iListNum >= 0) && (iListNum <= 6)) || (iListNum == -1)) ) {

		pChannel->logEnd("");
		return;
	}

	pChannel->ClearEntrance2ListBuffer(iListNum);

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeEntrance2List"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCMakeEntrance2List(int iChannelNum, int iListNum, int iLineCount, int iListSize,
								   unsigned char *pucBuf, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iListNum, iLineCount, iListSize);

	// zulaessige Bereiche pruefen
	if (!((iListNum >= 0) && (iListNum <= 6)) ||
		(pucBuf == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// ListNum ist:
	// 0: Reader         (33 Byte)
	// 1: Identification (64 Byte)
	// 2: Location       (20 Byte)
	// 3: Time           (25 Byte)
	// 4: Holiday        (21 Byte)
	// 5: Events         (17 Byte)
	// 6: Action         (21 Byte)

	// Bereich der gueltigen Nummer pruefen.
	switch(iListNum) {
	case 0:
		// Anzahl moeglicher Datensaetze
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_0))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Berechnete Listengroesze pruefen
		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_0)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_0) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 1:
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_1))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_1)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_1) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 2:
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_2))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_2)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_2) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 3:
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_3))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_3)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_3) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 4:
		// Anzahl moeglicher Datensaetze
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_4))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Berechnete Listengroesze pruefen
		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_4)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_4) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 5:
		// Anzahl moeglicher Datensaetze
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_5))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Berechnete Listengroesze pruefen
		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_5)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_5) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;
	case 6:
		// Anzahl moeglicher Datensaetze
		if ((iLineCount < 1) /*|| (iLineCount > (FLASH_SIZE_LIST / ENTRANCE2_RECORD_SIZE_6))*/) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Berechnete Listengroesze pruefen
		if (iListSize != (iLineCount * ENTRANCE2_RECORD_SIZE_6)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		// Zu grosze Listen abfangen
		/*		if ((iLineCount * ENTRANCE2_RECORD_SIZE_6) > FLASH_SIZE_LIST) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
		}
		*/		break;

	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->MakeEntrance2List(iListNum, iLineCount, iListSize, pucBuf);

	pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_SUCCESS, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadEntrance2List"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadEntrance2List(int iChannelNum, int iBusNum, int iListNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!(((iListNum >= 0) && (iListNum <= 6)) || (iListNum == -1)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Zutrittskontrolle Version 2 wird erst ab 04.01.xx unterstuetzt.
	unsigned char protocolVersion = pChannel->GetProtocolVersion(iBusNum);
	if (protocolVersion < 3) {

		*piError = IV_ERROR_VERSION_FIRMWARE;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes;
	char version[32];
	memset(version, 0, sizeof(version));
	// Firmwareversion lesen
	if (!(iRes = pChannel->GetVersionFirmware(iBusNum, version, piError))) {

		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}
	/*
	// Firmwareversion pruefen
	if (!theApp.m_util.CheckVersionFirmwareZK(version)) {

	*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
	pChannel->logEnd("\t%i\t(%i)", *piError, 0);
	return 0;
	}
	*/
	// Listendaten wegschreiben. BIN
	if (pChannel->logIsWriteList()) {

		char fileNameBin[_MAX_PATH];
		char basePath[_MAX_PATH];
		int listNum;
		FILE *fp = NULL;
		time_t t;
		tm *ltm;

		t = time(NULL);
		ltm = localtime(&t);

		for (listNum=0; listNum < 7; listNum++) {

			// Listendaten vorhanden?
			if (pChannel->m_ulEntrance2ListSize[listNum] == 0) {
				continue;
			}

			unsigned char *pucBuf = &pChannel->m_pucEntrance2ListMem[listNum][sizeof(unsigned long)];
			unsigned long size = pChannel->m_ulEntrance2ListSize[listNum] - sizeof(unsigned long);
			unsigned long recordCount = *(unsigned long*)pChannel->m_pucEntrance2ListMem[listNum];

			// Dateiname der Logdatei
			sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
				ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
				ltm->tm_hour, ltm->tm_min, ltm->tm_sec, iChannelNum, 'E', listNum);

			// Daten Binaer rausschreiben
			fp = fopen(fileNameBin, "wb");
			if (!fp) {
				goto error_label_bin;
			}
			fwrite((const void*)pucBuf, 1, size, fp);
			fclose(fp);

error_label_bin:
			if (fp) fclose(fp);
		}
	}

	// Listendaten wegschreiben. TXT
	if (pChannel->logIsWriteList() > 1) {

		char fileNameTxt[_MAX_PATH];
		char basePath[_MAX_PATH];
		unsigned char listName[32], fieldCount, varList;
		unsigned char fieldName[32], fieldNum, fieldType, fieldLength;
		int listOffs, listNum;
		FILE *fp = NULL;
		time_t t;
		tm *ltm;
		char line[512];

		t = time(NULL);
		ltm = localtime(&t);

		for (listNum=0; listNum < 7; listNum++) {

			// Listendaten vorhanden?
			if (pChannel->m_ulEntrance2ListSize[listNum] == 0) {
				continue;
			}

			unsigned char *pucBuf = &pChannel->m_pucEntrance2ListMem[listNum][sizeof(unsigned long)];
			unsigned long size = pChannel->m_ulEntrance2ListSize[listNum] - sizeof(unsigned long);
			unsigned long recordCount = *(unsigned long*)pChannel->m_pucEntrance2ListMem[listNum];

			if (!pChannel->GetTableRecord(200 + listNum, listName, &fieldCount, &varList, TRUE)) {
				goto error_label_txt;
			}

			// Dateiname der Logdatei
			sprintf(fileNameTxt, LOG_LIST_FILENAME_TXT, pChannel->logGetBasePath(basePath),
				ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
				ltm->tm_hour, ltm->tm_min, ltm->tm_sec, iChannelNum, 'E', listName);

			// Daten in Textform ablegen
			fp = fopen(fileNameTxt, "wb");
			if (!fp) {
				goto error_label_txt;
			}

			listOffs = 0;
			unsigned long recordNum;
			for (recordNum=0; recordNum < recordCount; recordNum++) {

				memset(line, 0, sizeof(line));
				for (fieldNum=0; fieldNum < fieldCount; fieldNum++) {

					if (!pChannel->GetTableField(200 + listNum, fieldNum, fieldName, &fieldType, &fieldLength, TRUE)) {
						fclose(fp);
						goto error_label_txt;
					}

					strcat(line, (char*)&pucBuf[listOffs]);
					if (fieldNum != (fieldCount-1)) strcat(line, "\t");
					listOffs += fieldLength;
				}

				fprintf(fp, "%s\x00D\x00A", line);
			}
error_label_txt:
			if (fp) fclose(fp);
		}
	}

	iRes = pChannel->LoadEntrance2ListData(iBusNum, iListNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

// ###################### ZUTRITTSKONTROLLE #########################
// ###################### ----------------- #########################

#undef FUNC_NAME
#define FUNC_NAME "DFCSetupLaden"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetupLaden(int iChannelNum,int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, iBusNum,
		(szFileName == NULL) ? "" : szFileName);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes;
	// Kommunikationsversionsnummer
	int commVersion = 0;

	if (pChannel->GetProtocolVersion(iBusNum) == 1) {
		// Bei AEIII+ zusaetzliche Pruefung wegen Befehlsunterstuetzung notwendig
		commVersion = pChannel->GetProtocolVersion2(iBusNum, TRUE, piError);
		switch(commVersion) {
		case 0x0400:
		case 0x0401:
		case 0x0402:
			break;
		default:
			*piError = IV_ERROR_VERSION_FIRMWARE;
			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}
	}

	if (commVersion != 0x0400 && commVersion != 0x0401) {

		char version[32];
		memset(version, 0, sizeof(version));
		// Firmwareversion lesen
		if (!(iRes = pChannel->GetVersionFirmware(iBusNum, version, piError))) {

			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Firmwareversion pruefen
		if (!theApp.m_util.CheckVersionFirmware(version)) {

			*piError = IV_ERROR_VERSION_FIRMWARE;
			pChannel->logEnd("\t%i\t(%i)", *piError, 0);
			return 0;
		}
	}

	// Setupdaten uebertragen
	iRes = pChannel->UpLoadSetupData(iBusNum, szFileName, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	if (iRes) {
		// Nur wenn Gert Verschluesselung nicht untersttzt
		if (pChannel->GetProtocolVersion(iBusNum) == 4 && !pChannel->m_channel4.IsPasswordProtection()) {

			// Datenpointer loeschen
			iRes = pChannel->ClearData(iBusNum, piError);
		}
		// Nach Kommunikationsfehler, Kanal bereinigen
		if (iRes == 0) pChannel->ResetWithError(*piError);
	}

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDownload"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCDownload(int iChannelNum,int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, iBusNum,
		(szFileName == NULL) ? "" : szFileName);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Setupdaten aus Geraet holen
	iRes = pChannel->DownLoadSetupData(iBusNum, szFileName, piError);

	if (!iRes && (*piError != IV_ERROR_SUCCESS)) {

		pChannel->ResetWithError(*piError);
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Setupdaten in Datei geschrieben
	pChannel->logEnd("\t%i\t(%i)", *piError, 1);

	return 1;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComVersion"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComVersion(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetProtocolVersion2(iBusNum, TRUE, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(0x%04X)", *piError, iRes);
	// Kommunikationsversion z.B: 0x00000103
	// Die Versionsnummer ist ein 16Bit - Wert
	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetVersionFirmware"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetVersionFirmware(int iChannelNum, int iBusNum, char *szVersion, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szVersion == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetVersionFirmware(iBusNum, szVersion, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t(%i)", (iRes) ? szVersion : "", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetVersionFirmwareFromFile"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetVersionFirmwareFromFile(int iChannelNum, char *szFileName, char *szVersion, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if ((szFileName == NULL) || (szVersion == NULL) || (piError == NULL) ||
		(strlen(szFileName) < 1)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes = pChannel->GetVersionFirmwareFromFile(szFileName, szVersion, piError);

	pChannel->logEnd("\t%s\t%s\t%i\t(%i)", szFileName, (iRes) ? szVersion : "", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetSeriennummer"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetSeriennummer(int iChannelNum, int iBusNum, int *piError, int *piSerialNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) || (piSerialNum == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetSerialNum(iBusNum, piSerialNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t%i\t(%i)", *piError, *piSerialNum, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetCallBack"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetCallBack(int iChannelNum, int (*CBFunction)(void))
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t", FUNC_NAME, iChannelNum);

	pChannel->SetCBFunction(CBFunction);

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckData"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCheckData(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
			return -1;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, -1);
		return -1;
	}

	int iRes = pChannel->CheckData(iBusNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == -1) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetGlobVar"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetGlobVar(int iChannelNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucVar == NULL) || (pucValue == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich pruefen.
	switch(iType) {
	case 0: // Ansprechen ueber Nummer
		if (*pucVar < 1 || *pucVar > 8) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		pChannel->logAppend("\t%i\t%i\t%s", *pucVar, iType, (char*)pucValue);
		break;
	case 1: // Ansprechen ueber Name
		if ((strlen((const char*)pucVar) < 1) || (strlen((const char*)pucVar) > MAX_NAME)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		pChannel->logAppend("\t%s\t%i\t%s", (char*)pucVar, iType, (char*)pucValue);
		break;
	case 2: // Ansprechen ueber Systemname
		if (strlen((const char*)pucVar) < 1) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		// Bereichspruefung der Systemvariablen
		if ((strstr((char*)pucVar, "SERVICE.DBLINPUT") && (atoi((char*)pucValue) < 0 || atoi((char*)pucValue) > 65499)) ||
			(strstr((char*)pucVar, "SERVICE.DEMANDTIMER") && (atoi((char*)pucValue) < 0 || atoi((char*)pucValue) > 255)) ) {

				*piError = IV_ERROR_VARIABLE_VALUE;
				pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
				return 0;
		}
		pChannel->logAppend("\t%s\t%i\t%s", (char*)pucVar, iType, (char*)pucValue);
		break;
	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->SetGlobVar(iBusNum, pucVar, iType, (char*)pucValue, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetGlobVar"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetGlobVar(int iChannelNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int iValueLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucVar == NULL) || (pucValue == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich pruefen.
	switch(iType) {
	case 0: // Ansprechen ueber Nummer
		if (*pucVar < 1 || *pucVar > 8) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		pChannel->logAppend("\t%i\t%i", *pucVar, iType);
		break;
	case 1: // Ansprechen ueber Name
		if ((strlen((const char*)pucVar) < 1) || (strlen((const char*)pucVar) > MAX_NAME)) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		pChannel->logAppend("\t%s\t%i", (char*)pucVar, iType);
		break;
	case 2: // Ansprechen ueber Systemname
		if (strlen((const char*)pucVar) < 1) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		pChannel->logAppend("\t%s\t%i", (char*)pucVar, iType);
		break;
	default:
		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t\t%i\t%i\t(%i)", iValueLength, *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetGlobVar(iBusNum, pucVar, iType, (char*)pucValue, iValueLength, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (char*)pucValue, iValueLength, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCloseRelay"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCloseRelay(int iChannelNum, int iBusNum, int iNum, int iTime, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum, iTime);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->CloseRelay(iBusNum, iNum, iTime, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetRelayState"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetRelayState(int iChannelNum, int iBusNum, int iNum, int *piState, int *piTime, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piState == NULL) || (piTime == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t\t\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetRelayState(iBusNum, iNum, piState, piTime, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t%i\t%i\t(%i)", *piState, *piTime, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCOpenRelay"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCOpenRelay(int iChannelNum, int iBusNum, int iNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->OpenRelay(iBusNum, iNum, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCWrite"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCWrite(int iChannelNum, char *pBuf, int iLength, int *piWriteLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if ((pBuf == NULL) || (piWriteLength == NULL) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(0)", iLength, IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t\t%i\t(0)", iLength, *piError);
		return 0;
	}

	if (pChannel->Transmit(pBuf, iLength, piWriteLength) == IV_ERROR_SUCCESS) {

		iRes = TRUE;
	}

	pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (strlen(pBuf) <= (unsigned int)iLength) ? pBuf : "",
		iLength, *piWriteLength, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCRead"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCRead(int iChannelNum, char *pBuf, int iLength, int *piReadLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if ((pBuf == NULL) || (piReadLength == NULL) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t\t%i\t(0)", iLength, IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t\t%i\t(0)", iLength, *piError);
		return 0;
	}

	int iRes = FALSE;

	if (pChannel->Receive(pBuf, iLength, piReadLength) == IV_ERROR_SUCCESS) {

		iRes = TRUE;
	}

	pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (strlen(pBuf) <= (unsigned int)iLength) ? pBuf : "",
		iLength, *piReadLength, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCUpload"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCUpload(int iChannelNum, int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t(%i)", (szFileName == NULL) ? "" : szFileName, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes;
	char fileName[0x200];
	memset(fileName, 0, sizeof(fileName));
	strcpy(fileName, szFileName);

	// Schnittstelle geoeffnet und CCom-Kanalklasse?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t(%i)", fileName, *piError, 0);
		return 0;
	}

	// Endung gibt funktion vor.
	// Nur ".dfl"
	int pos = strlen(fileName) - 4;

	if (fileName[pos] == '.' &&
		(((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
		(fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
		(fileName[pos + 3] == 'l' || fileName[pos + 3] == 'L'))) ) {

			// Textdaten uebertragen
			iRes = pChannel->UploadTextData(iBusNum, fileName);
			if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

				*piError = IV_ERROR_SUCCESS;
				iRes = 1;
			} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

				// Modulfehler auswerten.
				*piError = IV_ERROR_PROCESSING;
				iRes = 0;
			} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

				*piError = -iRes;
				iRes = 0;
			}
	} else {

		// Daten uebertragen
		iRes = pChannel->UploadFileDevice(iBusNum, fileName, piError);
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t(%i)", fileName, *piError, iRes);

	if ((iRes == 0) && (*piError == IV_ERROR_DEVICEFILE_MATCH) && strlen(pChannel->m_fileDevice.m_errorExtended)) {

		// Zusatzinfo in Logdatei schreiben
		int count = 0;
		char *token = strtok(pChannel->m_fileDevice.m_errorExtended, "\n");
		while (token != NULL) {


			pChannel->logBegin("ERROR.%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, ++count, token);
			pChannel->logEnd("");
			token = strtok(NULL, "\n");
		}

		// Hardwareinfo als zusaetzliche Information rausgeben
		if (strlen(pChannel->m_info)) {
			count = 0;
			token = strtok(pChannel->m_info, "\n");
			while (token != NULL) {


				pChannel->logBegin("INFO.%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, ++count, token);
				pChannel->logEnd("");
				token = strtok(NULL, "\n");
			}
		}

	}

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCUploadModule"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCUploadModule(int iChannelNum, int iBusNum, int type, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, type);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) || (type != 0) ||
		(szFileName == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t(%i)", (szFileName == NULL) ? "" : szFileName, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes;
	// Schnittstelle geoeffnet und CCom-Kanalklasse?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t(%i)", szFileName, *piError, 0);
		return 0;
	}

	// Dateityp ermitteln
	struct stat st;
	if (stat(szFileName, &st) == -1) {

		*piError = IV_ERROR_FILE_OPEN;
		return 0;
	}
	// Nur ".hex" gueltig.
	int pos = strlen(szFileName) - 4;
	if (
#ifdef _WIN32
		!((st.st_mode & _S_IFMT) == _S_IFREG) 
#else
		!((st.st_mode & S_IFMT) == S_IFREG) 
#endif
		|| szFileName[pos] != '.' || 
		!(szFileName[pos + 1] == 'u' || szFileName[pos + 1] == 'U') ||
		!(szFileName[pos + 2] == 'p' || szFileName[pos + 2] == 'P') ||
		!(szFileName[pos + 3] == 'd' || szFileName[pos + 3] == 'D') ) {

			*piError = IV_ERROR_FILE_OPEN;
			return 0;
	}

	// Daten uebertragen
	iRes = pChannel->UploadModule(iBusNum, type, szFileName, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t(%i)", szFileName, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCOpenTable"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCOpenTable(int channelID, int deviceID, char *name, int options, int *handle, char *reserved, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(name == NULL) || (handle == NULL) || (reserved == NULL) || (piError == NULL) ||
		(options != 0) || ((reserved != NULL) && strlen(reserved)) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", (name == NULL) ? "" : name, options, (handle == NULL) ? 0 : *handle, (reserved == NULL) ? "" : reserved, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", name, options, *handle, reserved, *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableOpen(deviceID, name, options, (int&)*handle, reserved);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
			case 2:
				*piError = IV_ERROR_NAME_UNKNOWN;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t%s\t%i\t(%i)", name, options, *handle, reserved, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCloseTable"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCloseTable(int channelID, int deviceID, int handle, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableClose(deviceID, handle);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetFilter"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetFilter(int channelID, int deviceID, int handle, char *filter, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(filter == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t(%i)", (filter == NULL) ? "" : filter, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t(%i)", filter, *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableSetFilter(deviceID, handle, filter);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 3:
				*piError = IV_ERROR_NULL_RESULT;
				break;
			case 4:
				*piError = IV_ERROR_NAME_UNKNOWN;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetFilter"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetFilter(int channelID, int deviceID, int handle, char *filter, int *len, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(filter == NULL) || (len == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (filter == NULL) ? "" : filter, (len == NULL) ? 0 : *len, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)", filter, *len, *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableGetFilter(deviceID, handle, filter, (short&)*len);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", filter, *len, *piError, iRes);

	return iRes;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCClearFilter"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCClearFilter(int channelID, int deviceID, int handle, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableClearFilter(deviceID, handle);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCSkip"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSkip(int channelID, int deviceID, int handle, int offset, int origin, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle, offset, origin);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL) || ((origin != 0) && (origin != 1) && (origin != 2)) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableSkip(deviceID, handle, offset, origin);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 3:
				*piError = IV_ERROR_BOF;
				break;
			case 4:
				*piError = IV_ERROR_EOF;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCSetField"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetField(int channelID, int deviceID, int handle, char *name, char *value, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(name == NULL) || (value == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%s\t%i\t(%i)", (name == NULL) ? "" : name, (value == NULL) ? "" : value, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%s\t%i\t(%i)", name, value, *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableSetField(deviceID, handle, name, value);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 2:
				*piError = IV_ERROR_NAME_UNKNOWN;
				break;
			case 3:
				*piError = IV_ERROR_VALUE;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%s\t%i\t(%i)", name, value, *piError, iRes);

	return iRes;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCGetField"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetField(int channelID, int deviceID, int handle, char *name, char *value, int *len, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, handle);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(name == NULL) || (value == NULL) || (len == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", (name == NULL) ? "" : name, (value == NULL) ? "" : value, (len == NULL) ? 0 : *len, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", name, value, *len, *piError, 0);
		return 0;
	}

	int iRes = pChannel->TableGetField(deviceID, handle, name, value, (short&)*len);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*piError = IV_ERROR_HANDLE;
				break;
			case 2:
				*piError = IV_ERROR_NAME_UNKNOWN;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%s\t%i\t%i\t(%i)", name, value, *len, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetInfo"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetInfo(int channelID, int deviceID, char *name, char *params, char *info, int *len, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(name == NULL) || (params == NULL) || (info == NULL) || (len == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", (name == NULL) ? "" : name, (params == NULL) ? "" : params,
				(info == NULL) ? "" : info, (len == NULL) ? 0 : *len, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", name, params, info, *len, *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetInfo(deviceID, name, params, info, (int&)*len);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 6:
				*piError = IV_ERROR_ENTRANCE2_OFF;
				break;
			case 7:
				*piError = IV_ERROR_MODULE_BUSY;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", name, params, info, *len, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCEntrance2Identification"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCEntrance2Identification(int channelID, int deviceID, char *TM, char *Id, char *Pin, char *status, int *len, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(TM == NULL) || (Id == NULL) || (Pin == NULL) || (status == NULL) || (len == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%s\t%s\t%s\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, (Id == NULL) ? "" : Id,
				(Pin == NULL) ? "" : Pin, (len == NULL) ? 0 : *len, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	// laengen pruefen
	if (strlen(TM) != 3 || strlen(Id) > 20 || strlen(Pin) > 8) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%s\t%s\t%s\t%s\t%i\t%i\t(%i)", TM, Id, strlen(Pin) ? "*****" : "", status, *len, *piError, 0);
		return 0;
	}
	// Wertebereich pruefen
	int i;
	for (i=0; i < 20; i++) {
		if ( ((i < 3) && !isdigit(TM[i])) ||
			((strlen(Id) > (size_t)i) && !isalnum(Id[i])) ||
			((i < 8) && (strlen(Pin) > (size_t)i) && !isprint(Pin[i])) ) {

				*piError = IV_ERROR_FUNCTION_PARAMETER;
				pChannel->logEnd("\t%s\t%s\t%s\t%s\t%i\t%i\t(%i)", TM, Id, strlen(Pin) ? "*****" : "", status, *len, *piError, 0);
				return 0;
		}
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%s\t%s\t%s\t%i\t%i\t(%i)", TM, Id, strlen(Pin) ? "*****" : "", status, *len, *piError, 0);
		return 0;
	}

	int iRes = pChannel->Entrance2Identification(deviceID, TM, Id, Pin, status, (int&)*len);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 6:
				*piError = IV_ERROR_ENTRANCE2_OFF;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%s\t%s\t%s\t%i\t%i\t(%i)", TM, Id, strlen(Pin) ? "*****" : "", status, *len, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCEntrance2OnlineAction"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCEntrance2OnlineAction(int channelID, int deviceID, char *TM, int mask, int type, int duration, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(TM == NULL) || (mask == 0) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t0x%08X\t%i\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, mask, type, duration, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	// laengen pruefen
	if (strlen(TM) != 3) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%s\t0x%08X\t%i\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, mask, type, duration, *piError, 0);
		return 0;
	}
	// Wertebereich pruefen
	int i;
	for (i=0; i < 3; i++) {
		if ( !isdigit(TM[i]) ) {

			*piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t0x%08X\t%i\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, mask, type, duration, *piError, 0);
			return 0;
		}
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t0x%08X\t%i\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, mask, type, duration, *piError, 0);
		return 0;
	}

	int iRes = pChannel->Entrance2OnlineAction(deviceID, TM, mask, type, duration);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 6:
				*piError = IV_ERROR_ENTRANCE2_OFF;
				break;
			case 28:
				*piError = IV_ERROR_NEED_PASSWORD;
				break;
			case 29:
				*piError = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t0x%08X\t%i\t%i\t%i\t(%i)", (TM == NULL) ? "" : TM, mask, type, duration, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintAppendRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintAppendRecord(int channelID, int deviceID, int type, unsigned char *fingertemplate, int size, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, type);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		((type != 0) && (type != 1)) || (fingertemplate == NULL) || (piError == NULL) || (size < 20) || ((type == 0) && (size > 161)) || ((type == 1) && (size > 216)) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	// Fingertemplate pruefen (Beide Templatetypen gleichen sich in den ersten 11 Byte)
	structFingertemplateDIN *fp = (structFingertemplateDIN*)fingertemplate;
	if (fp->pidLo != 0 || fp->fid > 10 || fp->quality == 0 || fp->count == 0) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, *piError, 0);
		return 0;
	}

	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintAppendRecord(deviceID, type, fingertemplate, size);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintGetRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintGetRecord(int channelID, int deviceID, int type, int pid, int fid, unsigned char *fingertemplate, int *size, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, type);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		((type != 0) && (type != 1)) || (pid == 0) || (fid < 1)  || (fid > 10) || (fingertemplate == NULL) || (size == NULL) || (piError == NULL) || (*size < 20) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t%i\t0x%08X\t%i\t%i\t(%i)", pid, fid, fingertemplate, (size == NULL) ? 0 : *size, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}

	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintGetRecord(deviceID, (unsigned char&)type, pid, fid, fingertemplate, (int&)*size);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t0x%08X\t%i\t%i\t(%i)", fingertemplate, size, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintDeleteRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintDeleteRecord(int channelID, int deviceID, int pid, int fid, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		((pid == 0) && (fid != 0)) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t%i\t%i\t(%i)", pid, fid, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t%i\t(%i)", pid, fid, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintDeleteRecord(deviceID, pid, fid);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t%i\t%i\t(%i)", pid, fid, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintList"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintList(int channelID, int deviceID, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(flags != 0) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t%i\t(%i)", flags, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t(%i)", flags, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintList(deviceID, flags);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t%i\t(%i)", flags, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintBackup"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintBackup(int channelID, int deviceID, char *backupFile, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(backupFile == 0) || (flags < 0) ||  (flags > 1) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (backupFile == NULL) ? "" : backupFile, flags, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)", backupFile, flags, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintBackup(deviceID, backupFile, flags);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", backupFile, flags, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCFingerprintRestore"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCFingerprintRestore(int channelID, int deviceID, char *backupFile, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(backupFile == 0) || (flags < 0) || (flags > 1) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (backupFile == NULL) ? "" : backupFile, flags, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)", backupFile, flags, *piError, 0);
		return 0;
	}

	int iRes = pChannel->FingerprintRestore(deviceID, backupFile, flags);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
			/*
			switch(iRes & 0xFF) {
			case 0:
			break;
			default:
			}*/
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", backupFile, flags, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCClearTimeboyListBuffer"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCClearTimeboyListBuffer(int iChannelNum, int iGroupID, int iListNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iGroupID, iListNum);

	// zulaessige Bereiche pruefen
	if (!(((iListNum >= 0) && (iListNum <= 19)) || (iListNum == -1)) ) {

		pChannel->logEnd("");
		return;
	}

	pChannel->ClearTimeboyListBuffer(iGroupID, iListNum);

	pChannel->logEnd("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeTimeboyList"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCMakeTimeboyList(int iChannelNum, int iGroupID, int iListNum, int iLineCount, int iListSize,
								 unsigned char *pucBuf, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iGroupID, iListNum, iLineCount, iListSize);

	// zulaessige Bereiche pruefen
	if ((iGroupID >= 0) && !((iListNum >= 0) && (iListNum <= 19)) ||
		(pucBuf == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t%i\t(%i)", flags, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes = pChannel->MakeTimeboyList(iGroupID, iListNum, iLineCount, pucBuf, iListSize, flags, piError);

	pChannel->logEnd("\t\t%i\t%i\t(%i)", flags, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadTimeboyList"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLoadTimeboyList(int iChannelNum, int iBusNum, int iGroupID, int iListNum, int reserve, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iGroupID, iListNum, reserve);

	// zulaessige Bereiche pruefen
	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!(((iListNum >= 0) && (iListNum <= 19)) || (iListNum == -1)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Timeboy Listenuebertragung wird erst ab 04.01.xx unterstuetzt.
	unsigned char protocolVersion = pChannel->GetProtocolVersion(iBusNum);
	if (protocolVersion == 0) {

		*piError = IV_ERROR_UNREACHABLE;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	} else if (protocolVersion < 4) {

		*piError = IV_ERROR_VERSION_FIRMWARE;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes;
	char version[32];
	memset(version, 0, sizeof(version));
	// Firmwareversion lesen
	if (!(iRes = pChannel->GetVersionFirmware(iBusNum, version, piError))) {

		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}
	/*
	// Firmwareversion pruefen
	if (!theApp.m_util.CheckVersionFirmwareZK(version)) {

	*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
	pChannel->logEnd("\t%i\t(%i)", *piError, 0);
	return 0;
	}
	*/

	// Listendaten wegschreiben.
	if (pChannel->logIsWriteList()) {

		char fileNameBin[_MAX_PATH];
		char basePath[_MAX_PATH];
		int listID, listNum;
		FILE *fp = NULL;
		time_t t;
		tm *ltm;

		t = time(NULL);
		ltm = localtime(&t);

		for (listID=0; listID < MAX_DATENSATZ_BESCHREIBUNG; listID++)
			for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++) {

				// Listendaten vorhanden?
				if (pChannel->m_ulTimeboyListSize[listID][listNum] == 0) {
					continue;
				}

				unsigned char *pucBuf = &pChannel->m_pucTimeboyListMem[listID][listNum][sizeof(unsigned long)];
				unsigned long size = pChannel->m_ulTimeboyListSize[listID][listNum] - sizeof(unsigned long);
				unsigned long recordCount = *(unsigned long*)pChannel->m_pucTimeboyListMem[listID][listNum];

				// Dateiname der Logdatei
				sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
					ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
					ltm->tm_hour, ltm->tm_min, ltm->tm_sec, iChannelNum, (char)(0x30 + listID), listNum);

				// Daten Binaer rausschreiben
				fp = fopen(fileNameBin, "wb");
				if (!fp) {
					goto error_label;
				}
				fwrite((const void*)pucBuf, 1, size, fp);
				fclose(fp);

error_label:
				if (fp) fclose(fp);
			}
	}

	iRes = pChannel->LoadTimeboyListData(iBusNum, iGroupID, iListNum, reserve, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}




#undef FUNC_NAME
#define FUNC_NAME "DFCOpenComServerMode"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCOpenComServerMode(int channelID, int deviceID, char *comm, int baudRate, int byteSize, int parity, int stopBits, int flags, int timeoutToClose, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (/*!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254))*/ (deviceID != 254) ||
		(comm == NULL) || ((baudRate != 9600) && (baudRate != 19200) && (baudRate != 38400)) || (flags & 0xFFFF0000) ||
		(error == NULL) ) {

			if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i)", (comm == NULL) ? "" : comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*error = IV_ERROR_SUCCESS;

	int iRes = pChannel->OpenComServerMode(deviceID, comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*error = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*error = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*error = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1: // Es wurden zu wenige Paramteter angegeben.
			case 2: // Es wurden zu viele Paramteter angegeben.
			case 3: // Die Angabe des comm Parameters wird nicht untersttzt.
			case 4: // Die Angabe des baudRate Parameters wird nicht untersttzt.
			case 5: // Die Angabe des byteSize Parameters wird nicht untersttzt.
			case 6: // Die Angabe des parity Paramters wird nicht untersttzt.
			case 7: // Die Angabe des stopBits Paramters wird nicht untersttzt.
			case 8: // Die Angabe des flags Parameters wird nicht untersttzt.
				*error = IV_ERROR_FUNCTION_PARAMETER;
				break;
			case 28: // Passwortangabe fehlerhaft
				*error = IV_ERROR_NEED_PASSWORD;
				break;
			case 29: // 3 malige flascheingabe, Gert geblockt.
				*error = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*error = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*error = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*error);

	pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i)", comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, *error, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCloseComServerMode"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCloseComServerMode(int channelID, int deviceID, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (/*!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254))*/ (deviceID != 254) || (error == NULL) ) {

		if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
		return 0;
	}
	*error = IV_ERROR_SUCCESS;

	// Eine Sekunde Pause, fuer Pufferleerung.
	Sleep(1000);

	int iRes = pChannel->CloseComServerMode(deviceID);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*error = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*error = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*error = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			switch(iRes & 0xFF) {
			case 1:
				*error = IV_ERROR_OVERFLOW;
				break;
			case 28: // Passwort fehlerhaft
				*error = IV_ERROR_NEED_PASSWORD;
				break;
			case 29: // 3 malige flascheingabe, Gert geblockt.
				*error = IV_ERROR_PASSWORD_LOCK;
				break;
			default:
				*error = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*error = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*error);

	pChannel->logEnd("\t%i\t(%i)", *error, iRes);

	return iRes;
}

#ifdef WIN32

#undef FUNC_NAME
#define FUNC_NAME "DFCStartActiveConnection"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCStartActiveConnection(char *ip, int port, int deviceID, int timeout, int aliveTO, int maxConnections, int infoFlags, int reserved, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s", FUNC_NAME);

	// zulaessige Bereiche pruefen
	if ((ip == NULL) || inet_addr(ip) == INADDR_NONE || !((deviceID >= 1) && (deviceID <= 31)) ||
		(piError == NULL) || maxConnections < 1 || maxConnections > 100) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t(%i)", (ip != NULL) ? ip : "", port, deviceID, timeout, aliveTO, maxConnections, infoFlags, reserved,  IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	int iRes = theApp.StartActiveConnection(ip, port, deviceID, timeout, aliveTO, maxConnections, infoFlags, reserved, piError);

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t(%i)", ip, port, deviceID, timeout, aliveTO, maxConnections, infoFlags, reserved,  *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCStopActiveConnection"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCStopActiveConnection(int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s", FUNC_NAME);

	// zulaessige Bereiche pruefen
	if (piError == NULL) {

		if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
		return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	int iRes = theApp.StopActiveConnection();

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetFirstActiveChannelID"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetFirstActiveChannelID()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s", FUNC_NAME);

	int i, iRes = -1, connected = 0;
	// Besteht die Aktive Verbindung
	if (theApp.IsActiveConnection()) {

		// Erste aktive Verbindung ermitteln
		for (i=1; i < ACTIVE_CHANNEL_UPPER_BOUND; i++) {

			if ((theApp.m_channelArray[i] != NULL) && (theApp.m_channelArray[i]->IsActive())) {

				iRes = i;
				break;
			}
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetNextActiveChannelID"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetNextActiveChannelID(int prev)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, prev);

	// Naechste aktive Verbindung ermitteln
	int i, iRes = -1;
	// Besteht die Aktive Verbindung
	if (theApp.IsActiveConnection()) {

		for (i=prev+1; i < ACTIVE_CHANNEL_UPPER_BOUND; i++) {

			if ((theApp.m_channelArray[i] != NULL) && (theApp.m_channelArray[i]->IsActive())) {

				iRes = i;
				break;
			}
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetInfoActiveChannel"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetInfoActiveChannel(int channelID, char *infoString, int *infoStringLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	int iRes = 0;
	pChannel->logBegin("%s\t%i", FUNC_NAME, channelID);

	// zulaessige Bereiche pruefen
	if ((infoString == NULL) || (infoStringLength == NULL)) {

		pChannel->logEnd("\t\t(%i)", 0);
		return 0;
	}

	// Besteht die Aktive Verbindung
	if (theApp.IsActiveConnection()) {

		if ((theApp.m_channelArray[channelID] != NULL) && (theApp.m_channelArray[channelID]->IsActive())) {

			char info[255];
			int infoLength = sizeof(info);
			memset(info, 0, sizeof(info));
			CChannel4 &channel = theApp.m_channelArray[channelID]->m_channel4;
			infoLength = snprintf(info, sizeof(info), "%i\r%i\r", channel.m_hwInfo.typeDevice, channel.m_serial);
			if (channel.m_serial == -1) {

				int i = 0;
			}
			// Prfen ob die Werte kopiert werden knnen; die Puffergre muss passen!
			if (infoLength != -1 && infoLength < *infoStringLength)
			{
				strcpy(infoString, info);
				*infoStringLength = infoLength;
				iRes = 1;
			}
		}
	}

	pChannel->logEnd("\t%s\t%i\t(%i)", infoString, *infoStringLength, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetRecordAvailable"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetRecordAvailable(int enable)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, enable);

	int iRes = theApp.SetRecordAvailable(enable);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCRecordAvailable"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCRecordAvailable(int *channelID, int *deviceID, char *infoString, int *infoStringLength, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s", FUNC_NAME);

	// zulaessige Bereiche pruefen
	if ((channelID == NULL) || (deviceID == NULL) ||
		(infoString == NULL) || (infoStringLength == NULL) || (error == NULL) ) {

			if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*error = IV_ERROR_SUCCESS;

	int iRes = 0;
	// Ist das Melden von Datentzen aktiviert?
	if (theApp.IsNotifyRecord()) {

		iRes = theApp.RecordAvailable(channelID, deviceID, infoString, infoStringLength);
		if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

			*error = IV_ERROR_SUCCESS;
			iRes = 1;
		} else if (iRes > IV_ERROR_SUCCESS) {

			*error = IV_ERROR_PROCESSING;
			iRes = 0;
		} else {

			*error = -iRes;
			iRes = 0;
		}
	} else {

		*error = IV_ERROR_AC_NO_NOTIFY_RECORD;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*error);

	pChannel->logEnd("\t%i\t%i\t%s\t%i\t%i\t(%i)", *channelID, *deviceID, infoString, *infoStringLength, *error, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCIsChannelOpen"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCIsChannelOpen(int channelID)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	int iRes = -1;
	pChannel->logBegin("%s\t%i", FUNC_NAME, channelID);

	if (channelID > 0 && channelID < MAX_CHANNEL && (theApp.m_channelArray[channelID] != NULL)) {

		if (theApp.m_channelArray[channelID]->IsOpen()) {

			iRes = 1;
		} else {

			iRes = 0;
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCBindDeviceToChannel"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCBindDeviceToChannel(int channelID, int deviceType, int deviceSerial, char *deviceIp, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	int iRes = 0;
	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceType, deviceSerial);

	// Liegen die Werte im gueltigen Bereich.
	if (channelID < 1 || channelID >= ACTIVE_CHANNEL_LOWER_BOUND || ((deviceType != -1) && (deviceType < 0 || deviceType > 5)) ||
		(deviceIp != NULL && strlen(deviceIp) && inet_addr(deviceIp) == INADDR_NONE) || piError == NULL) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Es gibt 4 moegliche mappings
	// 1. Loeschen eines bestehenden mappings
	// 2. map channelID => deviceType + deviceSerial
	// 3. map channelID => deviceType + deviceSerial + deviceIP
	// 4. map channelID => deviceIP
	CAEChannel *channel = theApp.GetAEChannel(channelID);
	if (channel->IsBind() && deviceType == -1 && (deviceIp == 0 || !strlen(deviceIp))) {

		// Bindungsstruktur aufbauen und zuweisen
		StructBindInfo bi;
		memset(&bi, 0, sizeof(StructBindInfo));
		channel->SetBind(bi);

	} else if ((deviceType > -1 && deviceSerial < 1) || (deviceType == -1 && (deviceIp == NULL || !strlen(deviceIp))) ) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", IV_ERROR_FUNCTION_PARAMETER, 0);
		return 0;
	} else {

		// Der Kanal darf nicht bereits gebunden oder geoeffnet sein.
		if (channel->IsBind() || channel->IsOpen()) {

			*piError = IV_ERROR_CHANNEL_USED;
			pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", *piError, 0);
			return 0;
		}

		// Bindungsstruktur aufbauen und zuweisen
		StructBindInfo bi;
		memset(&bi, 0, sizeof(StructBindInfo));
		bi.state = 1;
		bi.typeDevice = deviceType;
		bi.serial = deviceSerial;
		if (deviceIp != 0) strcpy((char*)bi.ip, deviceIp);
		channel->SetBind(bi);

		// Erfolgte Anbindung dem Listen-Thread signalisieren
		theApp.m_bindSignal = 1;
	}

	iRes = 1;
	pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", *piError, iRes);

	return iRes;
}
#endif

#undef FUNC_NAME
#define FUNC_NAME "DFCModifyStudioFile"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCModifyStudioFile(char *sourceFile, char *destFile, char *changeData, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int id = LOG_CHANNEL_ID;
	CAEChannel *pChannel = theApp.GetAEChannel(id);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s", FUNC_NAME);

	// zulaessige Bereiche pruefen
	if ((sourceFile == NULL) || (destFile == NULL) || (changeData == NULL) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%s\t%s\t%i\t(%i)", (sourceFile != NULL) ? sourceFile : "", (destFile != NULL) ? destFile : "", (changeData != NULL) ? changeData : "", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	int iRes = pChannel->ModifyStudioFile(sourceFile, destFile, changeData, flags, piError);

	pChannel->logEnd("\t%s\t%s\t%s\t%i\t(%i)", sourceFile, destFile, changeData, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetOptionFirmware"
/*
mask:
0x00000001 External Reader options 0=GIS, 1=PHG
0x00000002 IOModul
0x00000004 TBAdaption
0x00000008 ComServer
0x00000010 TransponderWrite
0x00000020 UpdateBiokey
0x00000040 Http-Protokoll
0x00000080 Signalverarbeitung

options: zustand 0 ist off, und 1 on
*/
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetOptionFirmware(int channelID, int deviceID, int mask, int options, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, mask, options);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		!(((unsigned)mask > 0) && ((unsigned)mask <= 0x000000FF)) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->SetOptionFirmware(deviceID, mask, options);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetOptionFirmware"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetOptionFirmware(int channelID, int deviceID, int *mask, int *options, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(mask == NULL) || (options == NULL) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t%i\t%i\t(%i)", (mask == NULL) ? 0 : mask, (options == NULL) ? 0 : options, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t%i\t(%i)", mask, options, *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetOptionFirmware(deviceID, *mask, *options);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler
		*piError = IV_ERROR_SUCCESS;
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten
			if ((iRes & 0xFF) == 28) {

				// Passwort wird benoetigt?
				*piError = IV_ERROR_NEED_PASSWORD;
			} else if ((iRes & 0xFF) == 29) {

				// Nach 3 maliger Passwortfalscheingabe ist Gert fr 60 Sekunden gesperrt.
				*piError = IV_ERROR_PASSWORD_LOCK;
			} else {

				// Funktionsfehler : Fehler in Verarbeitung.
				*piError = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t%i\t%i\t(%i)", mask, options, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetFontType"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetFontType(int channelID, int deviceID, int iType, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, iType);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Bereich der gueltigen Nummer pruefen.
	if ((iType < 0) || (iType > 6)) {

		*piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->SetTypeFont(deviceID, iType, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCPressVirtualKey"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCPressVirtualKey(int channelID, int deviceID, int vk, int flags, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, vk, flags);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(vk < 0x70) || (vk > 0x7E) || (flags > 1) || (piError == NULL) ) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	int iRes = pChannel->PressVirtualKey(deviceID, vk, flags, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

/*
#undef FUNC_NAME
#define FUNC_NAME "DFCSetNotifyDevice"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetNotifyDevice(int type, char *ip, int port, int maxConnections, int state,  int *error)
{
AFX_MANAGE_STATE(AfxGetStaticModuleState());

CAEChannel *pChannel = theApp.GetAEChannel(0);
// Wurde Channel-Objekt ermittelt.
if (pChannel == NULL) {

if (error) *error = IV_ERROR_CHANNEL;
return 0;
}

pChannel->logBegin("%s\t%i", FUNC_NAME, type);

// zulaessige Bereiche pruefen
if ((ip == NULL) || (error == NULL) ) {

if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t(%i)", (ip == NULL) ? "" : ip, port, maxConnections, state, IV_ERROR_FUNCTION_PARAMETER, 0);
return 0;
}
*error = IV_ERROR_SUCCESS;

int iRes = theApp.SetNotifyDevice(type, ip, port, maxConnections, state);
if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

*error = IV_ERROR_SUCCESS;
iRes = 1;
} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

// Modulfehler auswerten.
if ((iRes >> 8) & 0xFF) {

if (((iRes >> 8) & 0xFF) == 2) {

*error = IV_ERROR_FUNCTION_UNAVAILABLE;
} else {

*error = IV_ERROR_PROCESSING;
}
} else {

// Funktionsfehler auswerten.
switch(iRes & 0xFF) {
case 1:
*error = IV_ERROR_OVERFLOW;
break;
default:
*error = IV_ERROR_PROCESSING;
}
}

iRes = 0;
} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

*error = -iRes;
iRes = 0;
}

// Nach Kommunikationsfehler, Kanal bereinigen
//if (iRes == 0) pChannel->ResetWithError(*error);

pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t(%i)", ip, port, maxConnections, state, *error, iRes);

return iRes;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCSetNotifySoftware"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetNotifySoftware(int type, char *ip, int port, int retries, int delay, int state,  int *error)
{
AFX_MANAGE_STATE(AfxGetStaticModuleState());

CAEChannel *pChannel = theApp.GetAEChannel(0);
// Wurde Channel-Objekt ermittelt.
if (pChannel == NULL) {

if (error) *error = IV_ERROR_CHANNEL;
return 0;
}

pChannel->logBegin("%s\t%i", FUNC_NAME, type);

// zulaessige Bereiche pruefen
if ((ip == NULL) || (error == NULL) ) {

if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t(%i)", (ip == NULL) ? "" : ip, port, retries, delay, state, IV_ERROR_FUNCTION_PARAMETER, 0);
return 0;
}
*error = IV_ERROR_SUCCESS;

int iRes = theApp.SetNotifySoftware(type, ip, port, retries, delay, state);
if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

*error = IV_ERROR_SUCCESS;
iRes = 1;
} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

// Modulfehler auswerten.
if ((iRes >> 8) & 0xFF) {

if (((iRes >> 8) & 0xFF) == 2) {

*error = IV_ERROR_FUNCTION_UNAVAILABLE;
} else {

*error = IV_ERROR_PROCESSING;
}
} else {

// Funktionsfehler auswerten.
switch(iRes & 0xFF) {
case 1:
*error = IV_ERROR_OVERFLOW;
break;
default:
*error = IV_ERROR_PROCESSING;
}
}

iRes = 0;
} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

*error = -iRes;
iRes = 0;
}

// Nach Kommunikationsfehler, Kanal bereinigen
//if (iRes == 0) pChannel->ResetWithError(*error);

pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t(%i)", ip, port, retries, delay, state, *error, iRes);

return iRes;
}
*/

#undef FUNC_NAME
#define FUNC_NAME "DFCGetPasswordKey"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetPasswordKey(int channelID, int deviceID, char *key, int *len, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(key == NULL) || (len == NULL) || (piError == NULL) || (*len < 17)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (key == NULL) ? "" : key, (len == NULL) ? 0 : *len, IV_ERROR_FUNCTION_PARAMETER, 0);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)", key, *len, *piError, 0);
		return 0;
	}

	int iRes = pChannel->GetPasswordKey(deviceID, key);
	if (iRes == IV_ERROR_SUCCESS) { // Kein Fehler

		*piError = IV_ERROR_SUCCESS;
		*len = strlen(key);
		iRes = 1;
	} else if (iRes > IV_ERROR_SUCCESS) { // Fehler bei Durchfuehrung des Befehls

		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF) {

			if (((iRes >> 8) & 0xFF) == 2) {

				*piError = IV_ERROR_FUNCTION_UNAVAILABLE;
			} else {

				*piError = IV_ERROR_PROCESSING;
			}
		} else {

			// Funktionsfehler auswerten.
			*piError = IV_ERROR_PROCESSING;
		}

		iRes = 0;
	} else { // Fehler waehrend Kommunikation oder Verarbeitung in DLL

		*piError = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)", key, *len, *piError, iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetPassword"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetPassword(int channelID, char *password, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, channelID);

	// zulaessige Bereiche pruefen
	if ((password == NULL) || strlen(password) > 16 || (piError == NULL)) {

		if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%s\t%i\t(%i)", (password == NULL) ? "" : "****", IV_ERROR_FUNCTION_PARAMETER, 0);
		return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	pChannel->SetPassword(password);

	pChannel->logEnd("\t****\t%i\t(%i)", *piError, 1);

	return 1;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetFlashStatus"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetFlashStatus(int channelID, int deviceID, int *piStatus, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) || (piStatus == NULL) || (piError == NULL)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}
	*piError = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t%i\t(0)", *piStatus, *piError);
		return 0;
	}

	int iRes = pChannel->GetFlashStatus(deviceID, piStatus, piError);

	pChannel->logEnd("\t%i\t%i\t(%i)", *piStatus, *piError, iRes);

	return iRes;
}

// ########################################################
// #############    ALT   #################################
// ########################################################

#undef FUNC_NAME
#define FUNC_NAME "DFCLogOn"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCLogOn(char *szFileName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	//	pChannel->logBegin("%s\t%s", FUNC_NAME, (szFileName == 0) ? "" : szFileName);

	//	pChannel->logEnd("\t(%i)", 0);
	// Wird nicht mehr unterstuetzt!
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLogOff"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCLogOff(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	//	pChannel->logBegin("%s", FUNC_NAME);
	//	pChannel->logEnd("");
	// Wird nicht mehr unterstuetzt!
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComUseHandle"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComUseHandle(int iChannelNum, HANDLE hCom)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, hCom);

	int iRes = (pChannel->ComUseHandle(hCom) == IV_ERROR_SUCCESS);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComReleaseHandle"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComReleaseHandle(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	int iRes = (pChannel->ComReleaseHandle() == IV_ERROR_SUCCESS);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComInit"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComInit(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i", FUNC_NAME, iChannelNum);

	char szComStr[14];

#ifdef _WIN32
	if (iChannelNum > 9) {

		sprintf(szComStr, "\\\\?\\COM%i", iChannelNum);
	} else {

		sprintf(szComStr, "COM%i", iChannelNum);
	}

#else
	sprintf(szComStr, "/dev/ttyS%i", iChannelNum - 1);
#endif

	int iRes = FALSE;
	// Is Channel not engaged?
	if (!pChannel->IsOpen()) {

		if (pChannel->OpenAEIII(1, szComStr, 38400, 800) == IV_ERROR_SUCCESS) {

			iRes = TRUE;
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpen"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComOpen(int iChannelNum, char *szHost)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%s", FUNC_NAME, iChannelNum, (szHost == NULL) ? "" : szHost);

	if (szHost == NULL ) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iRes = FALSE;
	// Is Channel not engaged?
	if (!pChannel->IsOpen()) {

		if (pChannel->OpenAEIII(3, szHost, 8000, 5000) == IV_ERROR_SUCCESS) {

			iRes = TRUE;
		}
	}

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckAE"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCheckAE(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes = pChannel->CheckDevice(iBusNum, &iError, 1);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetClass"
// ########################################################
extern "C" __declspec(dllexport)

void* __stdcall DFCGetClass(int channelID, int classID)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return NULL;
	}

	switch(classID) {
	case 0:
		return pChannel;

	case 1:
	case 2:
		return &pChannel->m_channel1_2;

	case 3:
		return &pChannel->m_channel3;

	case 4:
		return &pChannel->m_channel4;

	default:
		return NULL;
	}

	return NULL;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCommandInspectionStation"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCommandInspectionStation(int channelID, int deviceID, int address, char *data, int sizeData, int *addressAnswer, char *dataAnswer, int *sizeAnswer, int toPending)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return -1;
	}
	// Parameter pruefen
	if (toPending < 1) {

		return -IV_ERROR_FUNCTION_PARAMETER;
	}

	return pChannel->m_channel4.CommandInspectionStation(deviceID, address, data, sizeData, (unsigned char&)*addressAnswer, dataAnswer, (int&)*sizeAnswer, toPending);
}


#undef FUNC_NAME
#define FUNC_NAME "DFCSystemReadRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSystemReadRecord(int channelID, int deviceID, char *buf, int *length, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if ((buf == NULL) || (length == NULL) || (error == NULL) ) {

		if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
		return -1;
	}
	*error = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t\t\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->SystemReadRecord(deviceID, buf, length, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) {
		pChannel->ResetWithError(*error);

		pChannel->logAppend("\t\t");
	} else if (res == 0) {

		pChannel->logAppend("\t\t");
	} else {

		// Daten fuer Log in Puffer uebergeben
		unsigned char ucTmp[MAX_MB_PAKET_SYSTEM_DATA * 3 + 1];
		int i;
		for (i=0; i < *length; i++) {

			sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buf[i]);
		}
		ucTmp[i * 3] = 0;

		pChannel->logAppend("\t%s\t%i", ucTmp, *length);
	}
	pChannel->logEnd("\t%i\t(%i)", *error, res);
	return res;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSystemQuitRecord"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSystemQuitRecord(int channelID, int deviceID, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = IV_ERROR_CHANNEL;
		return -1;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (error == NULL ) {

		if (error) *error = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, -1);
		return -1;
	}
	*error = IV_ERROR_SUCCESS;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->SystemQuitRecord(deviceID, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) pChannel->ResetWithError(*error);

	pChannel->logEnd("\t%i\t(%i)", *error, res);
	return res;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSystemRestoreRecords"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSystemRestoreRecords(int channelID, int deviceID, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL)) {

			if (piError) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*piError = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%i\t(0)", *piError);
		return 0;
	}

	*piError = IV_ERROR_SUCCESS;
	int iRes = pChannel->SystemRestoreRecords(deviceID, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);
	return iRes;
}



#undef FUNC_NAME
#define FUNC_NAME "DFCReset"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCReset(int channelID, int deviceID, int mode, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i", FUNC_NAME, channelID, deviceID, mode);

	if (!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254)) ||
		(piError == NULL) || ((mode != 1) && (mode != 2))) {

			if (piError != NULL) *piError = IV_ERROR_FUNCTION_PARAMETER;
			pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
			return 0;
	}

	*piError = IV_ERROR_SUCCESS;
	int iRes;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	iRes = pChannel->Reset(deviceID, mode, piError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(*piError);

	pChannel->logEnd("\t%i\t(%i)", *piError, iRes);

	return iRes;
}

#ifdef _DEBUG

#undef FUNC_NAME
#define FUNC_NAME "DFCWatchdogReset"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCWatchdogReset(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAEChannel *pChannel = theApp.GetAEChannel(iChannelNum);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		//if (piError) *piError = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	if (!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	int iError = IV_ERROR_SUCCESS;
	int iRes;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		pChannel->logEnd("\t(0)");
		return 0;
	}

	iRes = pChannel->Restart(iBusNum, &iError);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->ResetWithError(iError);

	pChannel->logEnd("\t(%i)", iRes);

	return iRes;
}

#endif //_DEBUG

